@@ -1,3 +1,27 @@
+/******************************************************************************
+ * The MIT License (MIT)
+ *
+ * Copyright (c) 2017-2018 Baldur Karlsson
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ ******************************************************************************/
+
 #pragma once
 
 // don't add any Qt headers visible to SWIG, as we don't want a Qt dependency for the SWIG-generated
@@ -29,6 +53,7 @@
 
 // this is pre-declared as an opaque type as we only support converting to QWidget* via PySide
 class QWidget;
+class QMenu;
 
 // we only support QVariant as an 'internal' interface, it's not exposed to python. However we need
 // to use it in constructors/operators so conditionally compile it rather than split small structs
@@ -60,12 +85,13 @@ class QWidget;
 // we depend on the internal RenderDoc API, but the bindings for that are imported entirely
 #include "renderdoc_replay.h"
 
+struct ICaptureContext;
+
 #include "Analytics.h"
+#include "Extensions.h"
 #include "PersistantConfig.h"
 #include "RemoteHost.h"
 
-struct ICaptureContext;
-
 DOCUMENT("Contains all of the settings that control how to capture an executable.");
 struct CaptureSettings
 {
@@ -510,8 +536,11 @@ DOCUMENT(R"(A shader window used for viewing, editing, or debugging.
 
   :param CaptureContext context: The current capture context.
   :param ShaderViewer viewer: The open shader viewer.
-  :param list files: A ``list`` with 2-tuples of ``str``, the first element being the filename and
-    the second element being the file contents.
+  :param ShaderEncoding encoding: The encoding of the files being passed.
+  :param ShaderCompileFlags flags: The flags to use during compilation.
+  :param str entryFunc: The name of the entry point.
+  :param bytes source: The byte buffer containing the source - may just be text depending on the
+    encoding.
 
 .. function:: CloseCallback(context)
 
@@ -523,7 +552,9 @@ DOCUMENT(R"(A shader window used for viewing, editing, or debugging.
 )");
 struct IShaderViewer
 {
-  typedef std::function<void(ICaptureContext *ctx, IShaderViewer *, const rdcstrpairs &)> SaveCallback;
+  typedef std::function<void(ICaptureContext *ctx, IShaderViewer *, ShaderEncoding,
+                             ShaderCompileFlags, rdcstr, bytebuf)>
+      SaveCallback;
   typedef std::function<void(ICaptureContext *ctx)> CloseCallback;
 
   DOCUMENT(
@@ -703,7 +734,7 @@ struct IReplayManager
 :return: The host connected to, or ``None`` if no connection is active.
 :rtype: RemoteHost
 )");
-  virtual const RemoteHost *CurrentRemote() = 0;
+  virtual RemoteHost *CurrentRemote() = 0;
 
   DOCUMENT(R"(Retrieves the capture file handle for the currently open file.
 
@@ -1184,6 +1215,15 @@ the UI which aren't reflected in the capture file on disk.
 )");
   virtual const APIProperties &APIProps() = 0;
 
+  DOCUMENT(R"(Retrieve the list of :class:`~renderdoc.ShaderEncoding` that are available for
+building target shaders for the currently loaded capture. See
+:meth:`~renderdoc.ReplayController.BuildTargetShader`.
+
+:return: The available encodings.
+:rtype: ``list`` of :class:`~renderdoc.ShaderEncoding`
+)");
+  virtual rdcarray<ShaderEncoding> TargetShaderEncodings() = 0;
+
   DOCUMENT(R"(Retrieve the currently selected :data:`eventId <renderdoc.APIEvent.eventId>`.
 
 In most cases, prefer using :meth:`CurEvent`. See :meth:`CaptureViewer.OnSelectedEventChanged` for more
@@ -1405,11 +1445,14 @@ called.
   DOCUMENT(R"(Create an opaque pointer suitable for passing to
 :meth:`~renderdoc.ReplayController.CreateOutput` or other functions that expect windowing data.
 
-:param int winId: The window ID as returned from ``QWidget.winId()``.
+.. note::
+  This function must be called on the main UI thread.
+
+:param QWidget window: The window to create windowing data for.
 :return: The windowing data.
 :rtype: ~renderdoc.WindowingData
 )");
-  virtual WindowingData CreateWindowingData(uintptr_t winId) = 0;
+  virtual WindowingData CreateWindowingData(QWidget *window) = 0;
 
   DOCUMENT(R"(Retrieve the current list of debug messages. This includes messages from the capture
 as well as messages generated during replay and analysis.
@@ -1495,7 +1538,7 @@ If no bookmark exists, this function will do nothing.
   DOCUMENT(R"(Retrieve the current singleton :class:`EventBrowser`.
 
 :return: The current window, which is created (but not shown) it there wasn't one open.
-:rtype: EventBrowser
+:rtype: ~qrenderdoc.EventBrowser
 )");
   virtual IEventBrowser *GetEventBrowser() = 0;
 
@@ -1509,7 +1552,7 @@ If no bookmark exists, this function will do nothing.
   DOCUMENT(R"(Retrieve the current singleton :class:`TextureViewer`.
 
 :return: The current window, which is created (but not shown) it there wasn't one open.
-:rtype: TextureViewer
+:rtype: ~qrenderdoc.TextureViewer
 )");
   virtual ITextureViewer *GetTextureViewer() = 0;
 
@@ -1523,7 +1566,7 @@ If no bookmark exists, this function will do nothing.
   DOCUMENT(R"(Retrieve the current singleton :class:`PipelineStateViewer`.
 
 :return: The current window, which is created (but not shown) it there wasn't one open.
-:rtype: PipelineStateViewer
+:rtype: ~qrenderdoc.PipelineStateViewer
 )");
   virtual IPipelineStateViewer *GetPipelineViewer() = 0;
 
@@ -1712,9 +1755,12 @@ place if needed.
   DOCUMENT(R"(Show a new :class:`ShaderViewer` window, showing an editable view of a given shader.
 
 :param bool customShader: ``True`` if the shader being edited is a custom display shader.
+:param ~renderdoc.ShaderStage stage: The shader stage for this shader.
 :param str entryPoint: The entry point to be used when compiling the edited shader.
 :param list files: The files stored in a ``list`` with 2-tuples of ``str``. The first element being
   the filename and the second being the file contents.
+:param ~renderdoc.ShaderEncoding shaderEncoding: The encoding of the input files.
+:param ~renderdoc.ShaderCompileFlags flags: The flags originally used to compile the shader.
 :param ShaderViewer.SaveCallback saveCallback: The callback function to call when a save/update is
   triggered.
 :param ShaderViewer.CloseCallback closeCallback: The callback function to call when the shader
@@ -1722,8 +1768,9 @@ place if needed.
 :return: The new :class:`ShaderViewer` window opened but not shown for editing.
 :rtype: ShaderViewer
 )");
-  virtual IShaderViewer *EditShader(bool customShader, const rdcstr &entryPoint,
-                                    const rdcstrpairs &files,
+  virtual IShaderViewer *EditShader(bool customShader, ShaderStage stage, const rdcstr &entryPoint,
+                                    const rdcstrpairs &files, ShaderEncoding shaderEncoding,
+                                    ShaderCompileFlags flags,
                                     IShaderViewer::SaveCallback saveCallback,
                                     IShaderViewer::CloseCallback closeCallback) = 0;
 
@@ -1903,6 +1950,13 @@ capture's API.
 )");
   virtual PersistantConfig &Config() = 0;
 
+  DOCUMENT(R"(Retrieve the manager for extensions.
+
+:return: The current extension manager.
+:rtype: ExtensionManager
+)");
+  virtual IExtensionManager &Extensions() = 0;
+
 protected:
   ICaptureContext() = default;
   ~ICaptureContext() = default;
