@@ -48,8 +48,10 @@ public:
   DOCUMENT(
       "Ping the host to check current status - if the server is running, connection status, etc.");
   void CheckStatus();
-  DOCUMENT("Runs the command specified in :data:`runCommand`.");
-  void Launch();
+  DOCUMENT(
+      "Runs the command specified in :data:`runCommand`. Returns :class:`ReplayStatus` which "
+      "indicates success or the type of failure.");
+  ReplayStatus Launch();
 
   DOCUMENT("``True`` if a remote server is currently running on this host.");
   bool serverRunning : 1;
@@ -67,6 +69,9 @@ public:
   DOCUMENT("The command to run locally to try to launch the server remotely.");
   rdcstr runCommand;
 
+  DOCUMENT("The last folder browser to on this host, to provide a reasonable default path.");
+  rdcstr lastCapturePath;
+
   DOCUMENT(R"(
 Returns the name to display for this host in the UI, either :data:`friendlyName` or :data:`hostname`
 )");
