@@ -83,6 +83,8 @@ public:
   static QList<FormatElement> ParseFormatString(const QString &formatString, uint64_t maxLen,
                                                 bool tightPacking, QString &errors);
 
+  static QString GenerateTextureBufferFormat(const TextureDescription &tex);
+
   QVariantList GetVariants(const byte *&data, const byte *end) const;
   ShaderVariable GetShaderVar(const byte *&data, const byte *end) const;
 
@@ -313,6 +315,17 @@ public:
   void detach() { setProcessState(QProcess::NotRunning); }
 };
 
+class RegisteredMenuItem : public QObject
+{
+private:
+  Q_OBJECT
+public:
+  ContextMenu context;
+  PanelMenu panel;
+  rdcarray<rdcstr> submenus;
+  IExtensionManager::ExtensionCallback callback;
+};
+
 class QFileFilterModel : public QSortFilterProxyModel
 {
   Q_OBJECT
@@ -328,6 +341,25 @@ private:
   QDir::Filters m_requireMask, m_excludeMask;
 };
 
+class QCollator;
+
+class QCollatorSortFilterProxyModel : public QSortFilterProxyModel
+{
+  Q_OBJECT
+
+public:
+  explicit QCollatorSortFilterProxyModel(QObject *parent = Q_NULLPTR);
+  ~QCollatorSortFilterProxyModel();
+
+  QCollator *collator() { return m_collator; }
+protected:
+  virtual bool lessThan(const QModelIndex &source_left,
+                        const QModelIndex &source_right) const override;
+
+private:
+  QCollator *m_collator;
+};
+
 // Simple QStyledItemDelegate child that will either forward to an external delegate (allowing
 // chaining) or to the base implementation. Delegates can derive from this and specialise a couple
 // of functions to still be able to chain
