@@ -55,6 +55,12 @@ public:
   static void GlobalInit();
   static void GlobalShutdown();
 
+  static void ProcessExtensionWork(std::function<void()> callback);
+  static bool LoadExtension(ICaptureContext &ctx, const rdcstr &extension);
+  static void ConvertPyArgs(const ExtensionCallbackData &data,
+                            rdcarray<rdcpair<rdcstr, PyObject *>> &args);
+  static void FreePyArgs(rdcarray<rdcpair<rdcstr, PyObject *>> &args);
+
   bool CheckInterfaces();
 
   QString versionString();
@@ -112,6 +118,9 @@ private:
   // python context
   static PyObject *main_dict;
 
+  // the list of extension objects, to be able to reload them
+  static QMap<rdcstr, PyObject *> extensions;
+
   static bool initialised();
 
   // this is local to this context, containing a dict copied from a pristine __main__ that any
