@@ -31,6 +31,8 @@
 
 class RDTreeView;
 
+typedef QSet<uint> RDTreeViewExpansionState;
+
 class RDTreeViewDelegate : public ForwardingDelegate
 {
 private:
@@ -94,6 +96,39 @@ public:
   void setItemDelegate(QAbstractItemDelegate *delegate);
   QAbstractItemDelegate *itemDelegate() const;
 
+  void saveInternalExpansion(uint key, int keyColumn, int role = Qt::DisplayRole);
+  bool hasInternalExpansion(uint key);
+  void applyInternalExpansion(uint key, int keyColumn, int role = Qt::DisplayRole);
+  void clearInternalExpansions();
+
+  void saveExpansionExternal(RDTreeViewExpansionState &state, uint key, int keyColumn,
+                             int role = Qt::DisplayRole);
+  void applyExternalExpansion(const RDTreeViewExpansionState &state, uint key, int keyColumn,
+                              int role = Qt::DisplayRole);
+
+  // convenience overloads taking a string as a key that just hashes the string
+  void saveInternalExpansion(QString keystring, int keyColumn, int role = Qt::DisplayRole)
+  {
+    saveInternalExpansion(qHash(keystring), keyColumn, role);
+  }
+
+  bool hasInternalExpansion(QString keystring) { return hasInternalExpansion(qHash(keystring)); }
+  void applyInternalExpansion(QString keystring, int keyColumn, int role = Qt::DisplayRole)
+  {
+    applyInternalExpansion(qHash(keystring), keyColumn, role);
+  }
+
+  void saveExpansionExternal(RDTreeViewExpansionState &state, QString keystring, int keyColumn,
+                             int role = Qt::DisplayRole)
+  {
+    saveExpansionExternal(state, qHash(keystring), keyColumn, role);
+  }
+  void applyExternalExpansion(const RDTreeViewExpansionState &state, QString keystring,
+                              int keyColumn, int role = Qt::DisplayRole)
+  {
+    applyExternalExpansion(state, qHash(keystring), keyColumn, role);
+  }
+
 signals:
   void leave(QEvent *e);
   void keyPress(QKeyEvent *e);
@@ -119,6 +154,13 @@ private:
   bool m_VisibleGridLines = true;
   bool m_TooltipElidedItems = true;
 
+  QMap<uint, RDTreeViewExpansionState> m_Expansions;
+
+  void saveExpansion(RDTreeViewExpansionState &state, QModelIndex idx, uint seed, int keyColumn,
+                     int role);
+  void applyExpansion(const RDTreeViewExpansionState &state, QModelIndex idx, uint seed,
+                      int keyColumn, int role);
+
   QAbstractItemDelegate *m_userDelegate = NULL;
   RDTreeViewDelegate *m_delegate;
 
