@@ -26,6 +26,7 @@
 #include <QMouseEvent>
 #include <QSortFilterProxyModel>
 #include <QStandardItemModel>
+#include <QStandardPaths>
 #include "3rdparty/flowlayout/FlowLayout.h"
 #include "3rdparty/toolwindowmanager/ToolWindowManager.h"
 #include "Code/QRDUtils.h"
@@ -147,7 +148,7 @@ CaptureDialog::CaptureDialog(ICaptureContext &ctx, OnCaptureMethod captureCallba
 
   // Set up warning for host layer config
   initWarning(ui->vulkanLayerWarn);
-  ui->vulkanLayerWarn->setVisible(RENDERDOC_NeedVulkanLayerRegistration(NULL, NULL, NULL));
+  ui->vulkanLayerWarn->setVisible(RENDERDOC_NeedVulkanLayerRegistration(NULL));
   QObject::connect(ui->vulkanLayerWarn, &RDLabel::clicked, this,
                    &CaptureDialog::vulkanLayerWarn_mouseClick);
 
@@ -282,26 +283,24 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
 {
   QString caption = tr("Configure Vulkan layer settings in registry?");
 
-  VulkanLayerFlags flags = VulkanLayerFlags::NoFlags;
-  rdcarray<rdcstr> myJSONs;
-  rdcarray<rdcstr> otherJSONs;
+  VulkanLayerRegistrationInfo info;
 
-  RENDERDOC_NeedVulkanLayerRegistration(&flags, &myJSONs, &otherJSONs);
+  RENDERDOC_NeedVulkanLayerRegistration(&info);
 
-  const bool hasOtherJSON = bool(flags & VulkanLayerFlags::OtherInstallsRegistered);
-  const bool thisRegistered = bool(flags & VulkanLayerFlags::ThisInstallRegistered);
-  const bool needElevation = bool(flags & VulkanLayerFlags::NeedElevation);
-  const bool couldElevate = bool(flags & VulkanLayerFlags::CouldElevate);
-  const bool registerAll = bool(flags & VulkanLayerFlags::RegisterAll);
-  const bool updateAllowed = bool(flags & VulkanLayerFlags::UpdateAllowed);
+  const bool hasOtherJSON = bool(info.flags & VulkanLayerFlags::OtherInstallsRegistered);
+  const bool thisRegistered = bool(info.flags & VulkanLayerFlags::ThisInstallRegistered);
+  const bool needElevation = bool(info.flags & VulkanLayerFlags::NeedElevation);
+  const bool couldElevate = bool(info.flags & VulkanLayerFlags::CouldElevate);
+  const bool registerAll = bool(info.flags & VulkanLayerFlags::RegisterAll);
+  const bool updateAllowed = bool(info.flags & VulkanLayerFlags::UpdateAllowed);
 
-  if(flags & VulkanLayerFlags::Unfixable)
+  if(info.flags & VulkanLayerFlags::Unfixable)
   {
     QString msg =
         tr("There is an unfixable problem with your vulkan layer configuration. Please consult the "
            "RenderDoc documentation, or package/distribution documentation on linux\n\n");
 
-    for(const rdcstr &j : otherJSONs)
+    for(const rdcstr &j : info.otherJSONs)
       msg += j + lit("\n");
 
     RDDialog::critical(this, tr("Unfixable vulkan layer configuration"), msg);
@@ -313,7 +312,7 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
 
   if(hasOtherJSON)
   {
-    if(otherJSONs.size() > 1)
+    if(info.otherJSONs.size() > 1)
       msg +=
           tr("there are other RenderDoc builds registered already. They must be disabled so that "
              "capture can happen without nasty clashes.");
@@ -338,7 +337,7 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
 
   if(hasOtherJSON)
   {
-    for(const rdcstr &j : otherJSONs)
+    for(const rdcstr &j : info.otherJSONs)
       msg += (updateAllowed ? tr("Unregister/update: %1\n") : tr("Unregister: %1\n")).arg(j);
 
     msg += lit("\n");
@@ -348,13 +347,13 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
   {
     if(registerAll)
     {
-      for(const rdcstr &j : myJSONs)
+      for(const rdcstr &j : info.myJSONs)
         msg += (updateAllowed ? tr("Register/update: %1\n") : tr("Register: %1\n")).arg(j);
     }
     else
     {
       msg += updateAllowed ? tr("Register one of:\n") : tr("Register/update one of:\n");
-      for(const rdcstr &j : myJSONs)
+      for(const rdcstr &j : info.myJSONs)
         msg += tr("  -- %1\n").arg(j);
     }
 
@@ -401,21 +400,59 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
     {
       if(admin)
       {
+// linux sometimes can't run GUI apps as root, so we have to run renderdoccmd. Check that it's
+// installed, error if not, then invoke it.
+#if defined(Q_OS_LINUX)
+        QDir binDir = QFileInfo(qApp->applicationFilePath()).absoluteDir();
+
+        QString cmd = lit("renderdoccmd");
+
+        if(binDir.exists(cmd))
+        {
+          // it's next to our exe, run that
+          cmd = binDir.absoluteFilePath(cmd);
+        }
+        else
+        {
+          QString inPath = QStandardPaths::findExecutable(cmd);
+
+          if(inPath.isEmpty())
+          {
+            RDDialog::critical(
+                this, tr("Can't locate renderdoccmd"),
+                tr("On linux we must run renderdoccmd as root to register the layer, because "
+                   "graphical applications like qrenderdoc may fail to launch.\n\n"
+                   "renderdoccmd could not be located either next to this qrenderdoc executable or "
+                   "in PATH."));
+            return;
+          }
+
+          // it's in the path, we can continue
+        }
+
+        RunProcessAsAdmin(cmd, QStringList() << lit("vulkanregister") << lit("--system"), this,
+                          [this]() { ui->vulkanLayerWarn->setVisible(false); });
+#else
         RunProcessAsAdmin(qApp->applicationFilePath(),
                           QStringList() << lit("--install_vulkan_layer") << lit("root"), this,
                           [this]() { ui->vulkanLayerWarn->setVisible(false); });
+#endif
         return;
       }
       else
       {
-        QProcess process;
-        process.start(qApp->applicationFilePath(), QStringList() << lit("--install_vulkan_layer")
-                                                                 << lit("user"));
-        process.waitForFinished(300);
+        QProcess *process = new QProcess;
+        process->start(qApp->applicationFilePath(), QStringList() << lit("--install_vulkan_layer")
+                                                                  << lit("user"));
+        process->waitForFinished(300);
+
+        // when the process exits, delete
+        QObject::connect(process, OverloadedSlot<int>::of(&QProcess::finished),
+                         [process](int exitCode) { process->deleteLater(); });
       }
     }
 
-    ui->vulkanLayerWarn->setVisible(RENDERDOC_NeedVulkanLayerRegistration(NULL, NULL, NULL));
+    ui->vulkanLayerWarn->setVisible(RENDERDOC_NeedVulkanLayerRegistration(NULL));
   }
 }
 
@@ -564,7 +601,6 @@ bool CaptureDialog::checkAllowClose()
 void CaptureDialog::on_exePathBrowse_clicked()
 {
   QString initDir;
-  QString file;
 
   QFileInfo f(ui->exePath->text());
   QDir dir = f.dir();
@@ -572,18 +608,20 @@ void CaptureDialog::on_exePathBrowse_clicked()
   {
     initDir = dir.absolutePath();
   }
+  else if(m_Ctx.Replay().CurrentRemote())
+  {
+    initDir = m_Ctx.Replay().CurrentRemote()->lastCapturePath;
+  }
   else if(!m_Ctx.Config().LastCapturePath.isEmpty())
   {
     initDir = m_Ctx.Config().LastCapturePath;
-    if(!m_Ctx.Config().LastCaptureExe.isEmpty())
-      file = m_Ctx.Config().LastCaptureExe;
   }
 
   QString filename;
 
   if(m_Ctx.Replay().CurrentRemote())
   {
-    VirtualFileDialog vfd(m_Ctx, this);
+    VirtualFileDialog vfd(m_Ctx, initDir, this);
     RDDialog::show(&vfd);
     filename = vfd.chosenPath();
   }
@@ -605,26 +643,31 @@ void CaptureDialog::on_exePathBrowse_clicked()
 
 void CaptureDialog::on_workDirBrowse_clicked()
 {
-  QString initDir;
+  QString initDir = ui->workDirPath->text();
 
-  if(QDir(ui->workDirPath->text()).exists())
+  if(initDir.isEmpty())
   {
-    initDir = ui->workDirPath->text();
-  }
-  else
-  {
-    QDir dir = QFileInfo(ui->exePath->text()).dir();
-    if(dir.exists())
-      initDir = dir.absolutePath();
-    else if(!m_Ctx.Config().LastCapturePath.isEmpty())
-      initDir = m_Ctx.Config().LastCapturePath;
+    if(m_Ctx.Replay().CurrentRemote())
+    {
+      initDir = m_Ctx.Replay().CurrentRemote()->lastCapturePath;
+    }
+    else if(!QDir(initDir).exists())
+    {
+      QDir dir = QFileInfo(ui->exePath->text()).dir();
+      if(dir.exists())
+        initDir = dir.absolutePath();
+      else if(!m_Ctx.Config().LastCapturePath.isEmpty())
+        initDir = m_Ctx.Config().LastCapturePath;
+      else
+        initDir = QString();
+    }
   }
 
   QString dir;
 
   if(m_Ctx.Replay().CurrentRemote())
   {
-    VirtualFileDialog vfd(m_Ctx, this);
+    VirtualFileDialog vfd(m_Ctx, initDir, this);
     vfd.setDirBrowse();
     RDDialog::show(&vfd);
     dir = vfd.chosenPath();
@@ -825,8 +868,9 @@ void CaptureDialog::SetSettings(CaptureSettings settings)
   ui->CaptureCallstacksOnlyDraws->setChecked(settings.options.captureCallstacksOnlyDraws);
   ui->APIValidation->setChecked(settings.options.apiValidation);
   ui->RefAllResources->setChecked(settings.options.refAllResources);
+  ui->CaptureAllCmdLists->setChecked(settings.options.captureAllCmdLists);
   ui->DelayForDebugger->setValue(settings.options.delayForDebugger);
-  ui->VerifyMapWrites->setChecked(settings.options.verifyMapWrites);
+  ui->VerifyBufferAccess->setChecked(settings.options.verifyBufferAccess);
   ui->AutoStart->setChecked(settings.autoStart);
 
   // force flush this state
@@ -861,7 +905,7 @@ CaptureSettings CaptureDialog::Settings()
   ret.options.refAllResources = ui->RefAllResources->isChecked();
   ret.options.captureAllCmdLists = ui->CaptureAllCmdLists->isChecked();
   ret.options.delayForDebugger = (uint32_t)ui->DelayForDebugger->value();
-  ret.options.verifyMapWrites = ui->VerifyMapWrites->isChecked();
+  ret.options.verifyBufferAccess = ui->VerifyBufferAccess->isChecked();
 
   return ret;
 }
@@ -919,11 +963,33 @@ void CaptureDialog::SetExecutableFilename(const rdcstr &filename)
 
   ui->exePath->setText(fn);
 
-  if(!m_Ctx.Replay().CurrentRemote())
+  if(m_Ctx.Replay().CurrentRemote())
+  {
+    // remove the filename itself before setting the last capture path. Try /, then \ as path
+    // separator. If no separators are found, there is no path to set.
+    int idx = fn.lastIndexOf(QLatin1Char('/'));
+    if(idx >= 0)
+    {
+      fn = fn.mid(0, idx);
+    }
+    else
+    {
+      idx = fn.lastIndexOf(QLatin1Char('\\'));
+
+      if(idx >= 0)
+        fn = fn.mid(0, idx);
+      else
+        fn = QString();
+    }
+
+    m_Ctx.Replay().CurrentRemote()->lastCapturePath = fn;
+  }
+  else
   {
     m_Ctx.Config().LastCapturePath = QFileInfo(fn).absolutePath();
-    m_Ctx.Config().LastCaptureExe = QFileInfo(fn).completeBaseName();
   }
+
+  m_Ctx.Config().Save();
 }
 
 void CaptureDialog::SetWorkingDirectory(const rdcstr &dir)
