@@ -550,7 +550,7 @@ protected:
   }
 };
 
-VirtualFileDialog::VirtualFileDialog(ICaptureContext &ctx, QWidget *parent)
+VirtualFileDialog::VirtualFileDialog(ICaptureContext &ctx, QString initialDirectory, QWidget *parent)
     : QDialog(parent), ui(new Ui::VirtualFileDialog)
 {
   ui->setupUi(this);
@@ -590,8 +590,16 @@ VirtualFileDialog::VirtualFileDialog(ICaptureContext &ctx, QWidget *parent)
 
   ui->buttonBox->button(QDialogButtonBox::Ok)->setDefault(false);
 
+  QModelIndex index;
+
+  if(!initialDirectory.isEmpty())
+    index = m_Model->indexForPath(initialDirectory);
+
+  if(!index.isValid())
+    index = m_Model->homeFolder();
+
   // switch to home folder and expand it
-  changeCurrentDir(m_Model->homeFolder());
+  changeCurrentDir(index);
   ui->dirList->expand(m_DirProxy->mapFromSource(currentDir()));
 
   QObject::connect(ui->fileList->selectionModel(), &QItemSelectionModel::selectionChanged, this,
@@ -764,21 +772,21 @@ void VirtualFileDialog::on_filename_keyPress(QKeyEvent *e)
   re.setPatternSyntax(QRegExp::Wildcard);
 
   int fileCount = m_FileProxy->rowCount(curDir);
-  int matches = 0;
+  int matches = 0, dirmatches = 0;
   QString match;
+  QModelIndex idx;
 
   for(int f = 0; f < fileCount; f++)
   {
     QModelIndex file = m_FileProxy->index(f, 0, curDir);
     bool isDir = m_FileProxy->data(file, RemoteFileModel::FileIsDirRole).toBool();
 
-    if(isDir)
-      continue;
-
     QString filename = m_FileProxy->data(file, RemoteFileModel::FileNameRole).toString();
 
     if(re.exactMatch(filename))
     {
+      idx = file;
+      dirmatches += isDir ? 1 : 0;
       matches++;
       match = m_FileProxy->data(file, RemoteFileModel::FilePathRole).toString();
     }
@@ -786,8 +794,16 @@ void VirtualFileDialog::on_filename_keyPress(QKeyEvent *e)
 
   if(matches == 1)
   {
-    m_ChosenPath = match;
-    QDialog::accept();
+    if(dirmatches == 1)
+    {
+      changeCurrentDir(m_FileProxy->mapToSource(idx));
+      return;
+    }
+    else
+    {
+      m_ChosenPath = match;
+      QDialog::accept();
+    }
   }
 
   if(matches == 0 && !text.trimmed().isEmpty())
