@@ -189,6 +189,18 @@ EventBrowser::EventBrowser(ICaptureContext &ctx, QWidget *parent)
   QObject::connect(ui->events->header(), &QHeaderView::customContextMenuRequested, this,
                    &EventBrowser::events_contextMenu);
 
+  {
+    QMenu *extensionsMenu = new QMenu(this);
+
+    ui->extensions->setMenu(extensionsMenu);
+    ui->extensions->setPopupMode(QToolButton::InstantPopup);
+
+    QObject::connect(extensionsMenu, &QMenu::aboutToShow, [this, extensionsMenu]() {
+      extensionsMenu->clear();
+      m_Ctx.Extensions().MenuDisplaying(PanelMenu::EventBrowser, extensionsMenu, ui->extensions, {});
+    });
+  }
+
   OnCaptureClosed();
 
   m_redPalette = palette();
@@ -276,6 +288,57 @@ void EventBrowser::OnEventChanged(uint32_t eventId)
   highlightBookmarks();
 }
 
+bool EventBrowser::ShouldHide(const DrawcallDescription &drawcall)
+{
+  if(drawcall.flags & DrawFlags::PushMarker)
+  {
+    if(m_Ctx.Config().EventBrowser_HideEmpty)
+    {
+      if(drawcall.children.isEmpty())
+        return true;
+
+      bool allhidden = true;
+
+      for(const DrawcallDescription &child : drawcall.children)
+      {
+        if(ShouldHide(child))
+          continue;
+
+        allhidden = false;
+        break;
+      }
+
+      if(allhidden)
+        return true;
+    }
+
+    if(m_Ctx.Config().EventBrowser_HideAPICalls)
+    {
+      if(drawcall.children.isEmpty())
+        return false;
+
+      bool onlyapi = true;
+
+      for(const DrawcallDescription &child : drawcall.children)
+      {
+        if(ShouldHide(child))
+          continue;
+
+        if(!(child.flags & DrawFlags::APICalls))
+        {
+          onlyapi = false;
+          break;
+        }
+      }
+
+      if(onlyapi)
+        return true;
+    }
+  }
+
+  return false;
+}
+
 QPair<uint32_t, uint32_t> EventBrowser::AddDrawcalls(RDTreeWidgetItem *parent,
                                                      const rdcarray<DrawcallDescription> &draws)
 {
@@ -285,6 +348,9 @@ QPair<uint32_t, uint32_t> EventBrowser::AddDrawcalls(RDTreeWidgetItem *parent,
   {
     const DrawcallDescription &d = draws[i];
 
+    if(ShouldHide(d))
+      continue;
+
     QVariant name = QString(d.name);
 
     RichResourceTextInitialise(name);
@@ -955,6 +1021,11 @@ void EventBrowser::events_contextMenu(const QPoint &pos)
                      [this, rgp]() { rgp->SelectRGPEvent(m_Ctx.CurEvent()); });
   }
 
+  contextMenu.addSeparator();
+
+  m_Ctx.Extensions().MenuDisplaying(ContextMenu::EventBrowser_Event, &contextMenu,
+                                    {{"eventId", m_Ctx.CurEvent()}});
+
   RDDialog::show(&contextMenu, ui->events->viewport()->mapToGlobal(pos));
 }
 
