@@ -42,6 +42,7 @@
 #include "Windows/Dialogs/AboutDialog.h"
 #include "Windows/Dialogs/CaptureDialog.h"
 #include "Windows/Dialogs/CrashDialog.h"
+#include "Windows/Dialogs/ExtensionManager.h"
 #include "Windows/Dialogs/LiveCapture.h"
 #include "Windows/Dialogs/RemoteManager.h"
 #include "Windows/Dialogs/SettingsDialog.h"
@@ -159,6 +160,8 @@ MainWindow::MainWindow(ICaptureContext &ctx) : QMainWindow(NULL), ui(new Ui::Mai
 
   m_RemoteProbeSemaphore.release();
   m_RemoteProbe = new LambdaThread([this]() {
+    RENDERDOC_AndroidInitialise();
+
     while(m_RemoteProbeSemaphore.available())
     {
       // do a remoteProbe immediately to populate the android hosts list on startup.
@@ -386,6 +389,13 @@ MainWindow::MainWindow(ICaptureContext &ctx) : QMainWindow(NULL), ui(new Ui::Mai
     if(a->menu())
       actions.append(a->menu()->actions());
   }
+
+  // hide the dummy extension markers. They shouldn't be visible, they're just there so the code can
+  // easily find where to insert new extension menu items.
+  ui->extension_dummy_File->setVisible(false);
+  ui->extension_dummy_Window->setVisible(false);
+  ui->extension_dummy_Tools->setVisible(false);
+  ui->extension_dummy_Help->setVisible(false);
 }
 
 MainWindow::~MainWindow()
@@ -510,11 +520,13 @@ void MainWindow::OnCaptureTrigger(const QString &exe, const QString &workingDir,
       {
         RDDialog::critical(
             this, tr("Error connecting to debugger"),
-            tr("Error launching %1 for capture.\n\n"
+            tr("<html>Error launching %1 for capture.\n\n"
                "Something went wrong connecting to the debugger on the Android device.\n\n"
                "This can happen if the package is not "
-               "marked as debuggable, or if another android tool such as android studio "
-               "or other tool is interfering with the debug connection.")
+               "marked as debuggable, or if another android tool such as Android Studio "
+               "is interfering with the debug connection.\n\n"
+               "Close <b>all</b> instances of Android Studio or other Android programs "
+               "and try again.</html>")
                 .arg(exe));
         return;
       }
@@ -568,11 +580,13 @@ void MainWindow::OnInjectTrigger(uint32_t PID, const rdcarray<EnvironmentModific
       {
         RDDialog::critical(
             this, tr("Error connecting to debugger"),
-            tr("Error injecting into process %1 for capture.\n\n"
+            tr("<html>Error injecting into process %1 for capture.\n\n"
                "Something went wrong connecting to the debugger on the Android device.\n\n"
                "This can happen if the package is not "
-               "marked as debuggable, or if another android tool such as android studio "
-               "or other tool is interfering with the debug connection.")
+               "marked as debuggable, or if another android tool such as Android Studio "
+               "is interfering with the debug connection.\n\n"
+               "Close <b>all</b> instances of Android Studio or other Android programs "
+               "and try again.</html>")
                 .arg(PID));
         return;
       }
@@ -771,7 +785,7 @@ void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local
       {
         ANALYTIC_SET(UIFeatures.ImageViewer, true);
       }
-      else
+      else if(!driver.isEmpty())
       {
         ANALYTIC_ADDUNIQ(APIs, driver);
       }
@@ -1251,6 +1265,9 @@ void MainWindow::CheckUpdates(bool forceCheck, UpdateResultMethod callback)
                    });
 
   QObject::connect(req, &QNetworkReply::finished, [this, req, callback]() {
+    if(req->error() != QNetworkReply::NoError)
+      return;
+
     QString response = QString::fromUtf8(req->readAll());
 
     statusText->setText(QString());
@@ -1317,6 +1334,42 @@ void MainWindow::LiveCaptureClosed(LiveCapture *live)
   m_LiveCaptures.removeOne(live);
 }
 
+QMenu *MainWindow::GetBaseMenu(WindowMenu base, rdcstr name)
+{
+  switch(base)
+  {
+    case WindowMenu::File: return ui->menu_File;
+    case WindowMenu::Window: return ui->menu_Window;
+    case WindowMenu::Tools: return ui->menu_Tools;
+    case WindowMenu::Help: return ui->menu_Help;
+    case WindowMenu::NewMenu: break;
+    default: return NULL;
+  }
+
+  // new menu. See if we have one for name already. If not, create a new one and add it to the menu
+  // bar.
+  for(QAction *m : ui->menuBar->actions())
+  {
+    // if it has an object name it's a built-in, ignore it.
+    if(!m->objectName().isEmpty())
+      continue;
+
+    if(m->text() == name)
+      return m->menu();
+  }
+
+  // no existing menu, add a new one
+  QMenu *menu = new QMenu(name, this);
+  menu->setIcon(Icons::plugin());
+  ui->menuBar->insertMenu(ui->menu_Help->menuAction(), menu);
+  return menu;
+}
+
+QList<QAction *> MainWindow::GetMenuActions()
+{
+  return ui->menuBar->actions();
+}
+
 ToolWindowManager *MainWindow::mainToolManager()
 {
   return ui->toolWindowManager;
@@ -1774,6 +1827,17 @@ void MainWindow::switchContext()
       // see if the server is up
       host->CheckStatus();
 
+      if(host->IsADB() && !RENDERDOC_IsAndroidSupported(host->hostname.c_str()))
+      {
+        GUIInvoke::call(this, [this]() {
+          statusText->setText(tr("Device unsupported, Android 6.0 is required."));
+          contextChooser->setIcon(Icons::disconnect());
+          contextChooser->setText(tr("Replay Context: Local"));
+          contextChooser->setEnabled(true);
+        });
+        return;
+      }
+
       if(!host->serverRunning && !host->runCommand.isEmpty())
       {
         GUIInvoke::call(this, [this]() {
@@ -1782,7 +1846,11 @@ void MainWindow::switchContext()
           statusProgress->setMaximum(0);
         });
 
-        host->Launch();
+        ReplayStatus launchStatus = host->Launch();
+        if(launchStatus != ReplayStatus::Succeeded)
+        {
+          showLaunchError(launchStatus);
+        }
 
         // check if it's running now
         host->CheckStatus();
@@ -2310,7 +2378,7 @@ void MainWindow::on_action_Start_Replay_Loop_triggered()
                              .arg(tr("nothing")));
   }
 
-  WindowingData winData = m_Ctx.CreateWindowingData(popup.winId());
+  WindowingData winData = m_Ctx.CreateWindowingData(&popup);
 
   m_Ctx.Replay().AsyncInvoke([winData, id](IReplayController *r) { r->ReplayLoop(winData, id); });
 
@@ -2369,7 +2437,7 @@ void MainWindow::on_action_Create_RGP_Profile_triggered()
   popup.resize(128, 16);
   popup.setWindowTitle(tr("Making RGP Profile from %1").arg(m_Ctx.GetCaptureFilename()));
 
-  WindowingData winData = m_Ctx.CreateWindowingData(popup.winId());
+  WindowingData winData = m_Ctx.CreateWindowingData(&popup);
 
   rdcstr path;
 
@@ -2415,6 +2483,12 @@ void MainWindow::on_action_Attach_to_Running_Instance_triggered()
   on_action_Manage_Remote_Servers_triggered();
 }
 
+void MainWindow::on_action_Manage_Extensions_triggered()
+{
+  ExtensionManager manager(m_Ctx);
+  RDDialog::show(&manager);
+}
+
 void MainWindow::on_action_Manage_Remote_Servers_triggered()
 {
   RemoteManager *rm = new RemoteManager(m_Ctx, this);
@@ -2733,3 +2807,32 @@ bool MainWindow::LoadLayout(int layout)
 
   return false;
 }
+
+void MainWindow::showLaunchError(ReplayStatus status)
+{
+  QString title;
+  QString message;
+  switch(status)
+  {
+    case ReplayStatus::AndroidGrantPermissionsFailed:
+      title = tr("Permission is required");
+      message = tr("Enable RenderDocCmd to access storage on your device.");
+      break;
+    case ReplayStatus::AndroidABINotFound:
+      title = tr("Failed to install RenderDoc server");
+      message = tr("Couldn't determine supported ABIs.");
+      break;
+    case ReplayStatus::AndroidAPKFolderNotFound:
+      title = tr("Failed to install RenderDoc server");
+      message = tr("APK folder missing.");
+      break;
+    case ReplayStatus::AndroidAPKInstallFailed:
+      title = tr("Failed to install RenderDoc server");
+      message = tr("Couldn't find any installed APKs.");
+    default:
+      title = tr("Failed to install RenderDoc server");
+      message = tr("Unknown error.");
+      break;
+  }
+  GUIInvoke::call(this, [this, title, message]() { RDDialog::warning(this, title, message); });
+}
