@@ -68,7 +68,6 @@ private slots:
 
   // manual slots
   void shaderView_clicked();
-  void shaderEdit_clicked();
 
   void shaderSave_clicked();
   void resource_itemActivated(RDTreeWidgetItem *item, int column);
@@ -118,4 +117,6 @@ private:
 
   // keep track of the VB nodes (we want to be able to highlight them easily on hover)
   QList<RDTreeWidgetItem *> m_VBNodes;
+  // list of empty VB nodes that shouldn't be highlighted on hover
+  QList<RDTreeWidgetItem *> m_EmptyNodes;
 };
