@@ -52,6 +52,7 @@ struct CaptureContextInvoker : ICaptureContext
   {
     return m_Ctx.TempCaptureFilename(appname);
   }
+  virtual IExtensionManager &Extensions() override { return m_Ctx.Extensions(); }
   virtual IReplayManager &Replay() override { return m_Ctx.Replay(); }
   virtual bool IsCaptureLoaded() override { return m_Ctx.IsCaptureLoaded(); }
   virtual bool IsCaptureLocal() override { return m_Ctx.IsCaptureLocal(); }
@@ -64,6 +65,10 @@ struct CaptureContextInvoker : ICaptureContext
   }
   virtual const FrameDescription &FrameInfo() override { return m_Ctx.FrameInfo(); }
   virtual const APIProperties &APIProps() override { return m_Ctx.APIProps(); }
+  virtual rdcarray<ShaderEncoding> TargetShaderEncodings() override
+  {
+    return m_Ctx.TargetShaderEncodings();
+  }
   virtual uint32_t CurSelectedEvent() override { return m_Ctx.CurSelectedEvent(); }
   virtual uint32_t CurEvent() override { return m_Ctx.CurEvent(); }
   virtual const DrawcallDescription *CurSelectedDrawcall() override
@@ -107,10 +112,6 @@ struct CaptureContextInvoker : ICaptureContext
   virtual IRGPInterop *GetRGPInterop() override { return m_Ctx.GetRGPInterop(); }
   virtual const SDFile &GetStructuredFile() override { return m_Ctx.GetStructuredFile(); }
   virtual WindowingSystem CurWindowingSystem() override { return m_Ctx.CurWindowingSystem(); }
-  virtual WindowingData CreateWindowingData(uintptr_t winId) override
-  {
-    return m_Ctx.CreateWindowingData(winId);
-  }
   virtual const rdcarray<DebugMessage> &DebugMessages() override { return m_Ctx.DebugMessages(); }
   virtual int UnreadMessageCount() override { return m_Ctx.UnreadMessageCount(); }
   virtual void MarkMessagesRead() override { return m_Ctx.MarkMessagesRead(); }
@@ -164,6 +165,10 @@ struct CaptureContextInvoker : ICaptureContext
     return (m_Ctx.*ptr)(params...);
   }
 
+  virtual WindowingData CreateWindowingData(QWidget *window) override
+  {
+    return InvokeRetFunction<WindowingData>(&ICaptureContext::CreateWindowingData, window);
+  }
   virtual void LoadCapture(const rdcstr &capture, const rdcstr &origFilename, bool temporary,
                            bool local) override
   {
@@ -372,13 +377,15 @@ struct CaptureContextInvoker : ICaptureContext
   {
     InvokeVoidFunction(&ICaptureContext::ShowResourceInspector);
   }
-  virtual IShaderViewer *EditShader(bool customShader, const rdcstr &entryPoint,
-                                    const rdcstrpairs &files,
+  virtual IShaderViewer *EditShader(bool customShader, ShaderStage stage, const rdcstr &entryPoint,
+                                    const rdcstrpairs &files, ShaderEncoding shaderEncoding,
+                                    ShaderCompileFlags flags,
                                     IShaderViewer::SaveCallback saveCallback,
                                     IShaderViewer::CloseCallback closeCallback) override
   {
-    return InvokeRetFunction<IShaderViewer *>(&ICaptureContext::EditShader, customShader,
-                                              entryPoint, files, saveCallback, closeCallback);
+    return InvokeRetFunction<IShaderViewer *>(&ICaptureContext::EditShader, customShader, stage,
+                                              entryPoint, files, shaderEncoding, flags,
+                                              saveCallback, closeCallback);
   }
 
   virtual IShaderViewer *DebugShader(const ShaderBindpointMapping *bind,
