@@ -248,7 +248,7 @@ private:
                                  int &prevIndex, bool copy, bool rw);
 
   void AddResourceUsageEntry(QMenu &menu, uint32_t start, uint32_t end, ResourceUsage usage);
-  void OpenResourceContextMenu(ResourceId id, const rdcarray<EventUsage> &usage);
+  void OpenResourceContextMenu(ResourceId id, bool input, const rdcarray<EventUsage> &usage);
 
   void AutoFitRange();
   void rangePoint_Update();
