@@ -28,6 +28,7 @@
 
 namespace Android
 {
+static bool adbKillServer = false;
 bool toolExists(const std::string &path)
 {
   if(path.empty())
@@ -248,13 +249,21 @@ std::string getToolPath(ToolDir subdir, const std::string &toolname, bool checkE
 
   // finally try to locate it in our own distributed android subfolder
   {
-    std::string exepath;
-    FileIO::GetExecutableFilename(exepath);
-    std::string exedir = dirname(FileIO::GetFullPathname(exepath));
+    std::string libpath;
+    FileIO::GetLibraryFilename(libpath);
+    std::string libdir = dirname(FileIO::GetFullPathname(libpath));
 
-    toolpath = exedir + "/plugins/android/" + toolname;
+    toolpath = libdir + "/plugins/android/" + toolname;
     if(toolExists(toolpath))
+    {
+      if(toolname == "adb")
+      {
+        // if we're using our own adb, we should kill the server upon shutdown
+        adbKillServer = true;
+      }
+
       return toolpath;
+    }
   }
 
   toolpath = "";
@@ -298,4 +307,15 @@ Process::ProcessResult adbExecCommand(const std::string &device, const std::stri
     deviceArgs = StringFormat::Fmt("-s %s %s", device.c_str(), args.c_str());
   return execCommand(adb, deviceArgs, workDir, silent);
 }
+void initAdb()
+{
+  // we don't use adbExecCommand because we need to be sure we don't wait for it to exit
+  std::string adb = getToolPath(ToolDir::PlatformTools, "adb", false);
+  Process::LaunchProcess(adb.c_str(), ".", "start-server", true);
+}
+void shutdownAdb()
+{
+  if(adbKillServer)
+    adbExecCommand("", "kill-server", ".", false);
+}
 };
