@@ -279,6 +279,9 @@ DOCUMENT(R"(This is an opaque identifier that uniquely locates a resource.
 struct ResourceId
 {
   ResourceId() : id() {}
+#if defined(SWIG)
+  ResourceId(const ResourceId &other) : id(other.id) {}
+#endif
   DOCUMENT("A helper function that explicitly creates an empty/invalid/null :class:`ResourceId`.");
   inline static ResourceId Null() { return ResourceId(); }
   DOCUMENT("Compares two ``ResourceId`` objects for equality.");
@@ -308,6 +311,7 @@ DOCUMENT(R"(Specifies a windowing system to use for creating an output window.
 .. data:: Unknown
 
   No windowing data is passed and no native window is described.
+  See :func:`CreateHeadlessWindowingData`.
 
 .. data:: Win32
 
@@ -324,6 +328,10 @@ DOCUMENT(R"(Specifies a windowing system to use for creating an output window.
 .. data:: Android
 
   The windowing data refers to an Android window. See :func:`CreateAndroidWindowingData`.
+
+.. data:: MacOS
+
+  The windowing data refers to a MacOS / OS X CALayer. See :func:`CreateMacOSWindowingData`.
 )");
 enum class WindowingSystem : uint32_t
 {
@@ -332,6 +340,7 @@ enum class WindowingSystem : uint32_t
   Xlib,
   XCB,
   Android,
+  MacOS,
 };
 
 DECLARE_REFLECTION_ENUM(WindowingSystem);
@@ -391,6 +400,11 @@ struct WindowingData
     {
       ANativeWindow *window;
     } android;
+
+    struct
+    {
+      void *layer;
+    } macOS;
   };
 };
 
@@ -398,6 +412,20 @@ DECLARE_REFLECTION_ENUM(WindowingData);
 
 #endif
 
+DOCUMENT(R"(Create a :class:`WindowingData` for no backing window, it will be headless.
+
+:return: A :class:`WindowingData` corresponding to an 'empty' backing window.
+:rtype: WindowingData
+)");
+inline const WindowingData CreateHeadlessWindowingData()
+{
+  WindowingData ret = {};
+
+  ret.system = WindowingSystem::Unknown;
+
+  return ret;
+}
+
 DOCUMENT(R"(Create a :class:`WindowingData` for a Win32 ``HWND`` handle.
 
 :param HWND window: The native ``HWND`` handle for this window.
@@ -466,6 +494,22 @@ inline const WindowingData CreateAndroidWindowingData(ANativeWindow *window)
   return ret;
 }
 
+DOCUMENT(R"(Create a :class:`WindowingData` for an macOS ``CALayer`` handle (as void pointer).
+
+:param CALayer window: The native ``CALayer`` handle for this window.
+:return: A :class:`WindowingData` corresponding to the given window.
+:rtype: WindowingData
+)");
+inline const WindowingData CreateMacOSWindowingData(void *view)
+{
+  WindowingData ret = {};
+
+  ret.system = WindowingSystem::MacOS;
+  ret.macOS.layer = view;
+
+  return ret;
+}
+
 // declare metatype/reflection for ResourceId here as the struct itself is declared before including
 // all relevant headers above
 #if defined(RENDERDOC_QT_COMPAT)
@@ -484,6 +528,10 @@ DECLARE_REFLECTION_STRUCT(ResourceId);
 DOCUMENT(R"(Internal structure used for initialising environment in a replay application.)");
 struct GlobalEnvironment
 {
+  DOCUMENT("");
+  GlobalEnvironment() = default;
+  GlobalEnvironment(const GlobalEnvironment &) = default;
+
   DOCUMENT("The handle to the X display to use internally. If left ``NULL``, one will be opened.");
   Display *xlibDisplay = NULL;
 };
@@ -491,6 +539,10 @@ struct GlobalEnvironment
 DOCUMENT("The result of executing or injecting into a program.")
 struct ExecuteResult
 {
+  DOCUMENT("");
+  ExecuteResult() = default;
+  ExecuteResult(const ExecuteResult &) = default;
+
   DOCUMENT(
       "The :class:`ReplayStatus` resulting from the operation, indicating success or failure.");
   ReplayStatus status;
@@ -566,6 +618,17 @@ outputs.
   DOCUMENT("Sets the :class:`MeshDisplay` configuration for a mesh output.");
   virtual void SetMeshDisplay(const MeshDisplay &o) = 0;
 
+  DOCUMENT(R"(Sets the dimensions of the output, useful only for headless outputs that don't have a
+backing window which don't have any implicit dimensions. This allows configuring a virtual viewport
+which is useful for operations like picking vertices that depends on the output dimensions.
+
+.. note:: For outputs with backing windows, this will be ignored.
+
+:param int width: The width to use.
+:param int height: The height to use.
+)");
+  virtual void SetDimensions(int32_t width, int32_t height) = 0;
+
   DOCUMENT(
       "Clear and release all thumbnails associated with this output. See :meth:`AddThumbnail`.");
   virtual void ClearThumbnails() = 0;
@@ -881,17 +944,34 @@ See :meth:`BuildCustomShader`.
 The language used is native to the API's renderer - HLSL for D3D based renderers, GLSL otherwise.
 
 :param str entry: The entry point to use when compiling.
-:param str source: The source file.
+:param ShaderEncoding sourceEncoding: The encoding of the source data.
+:param bytes source: The source data itself.
 :param ShaderCompileFlags compileFlags: API-specific compilation flags.
 :param ShaderStage type: The stage that this shader will be executed at.
 :return: A ``tuple`` with the id of the new shader if compilation was successful,
   :meth:`ResourceId.Null` otherwise, and a ``str`` with any warnings/errors from compilation.
 :rtype: ``tuple`` of :class:`ResourceId` and ``str``.
 )");
-  virtual rdcpair<ResourceId, rdcstr> BuildTargetShader(const char *entry, const char *source,
+  virtual rdcpair<ResourceId, rdcstr> BuildTargetShader(const char *entry,
+                                                        ShaderEncoding sourceEncoding, bytebuf source,
                                                         const ShaderCompileFlags &flags,
                                                         ShaderStage type) = 0;
 
+  DOCUMENT(R"(Retrieve the list of supported :class:`ShaderEncoding` which can be build using
+:meth:`BuildTargetShader`.
+
+The list is sorted in priority order, so if the caller has a shader in a form but could
+compile/translate it to another, prefer to satisfy the first encoding before later encodings.
+
+This typically means the 'native' encoding is listed first, and then subsequent encodings are
+compiled internally - so compiling externally could be preferable as it allows better customisation
+of the compile process or using alternate/updated tools.
+
+:return: The list of target shader encodings available.
+:rtype: ``list`` of :class:`ShaderEncoding`
+)");
+  virtual rdcarray<ShaderEncoding> GetTargetShaderEncodings() = 0;
+
   DOCUMENT(R"(Replace one resource with another for subsequent replay and analysis work.
 
 This is commonly used for modifying the capture by selectively replacing resources with newly
@@ -1254,10 +1334,16 @@ The details of the types of messages that can be received are listed under
   This function will block but only to a limited degree. If no message is waiting after a small time
   it will return with a No-op message to allow further processing.
 
+:param ProgressCallback progress: A callback that will be repeatedly called with an updated progress
+  value when a long blocking message is coming through, e.g. a capture copy. Can be ``None`` if no
+  progress is desired.
 :return: The message that was received.
 :rtype: TargetControlMessage
 )");
-  virtual TargetControlMessage ReceiveMessage() = 0;
+  virtual TargetControlMessage ReceiveMessage(RENDERDOC_ProgressCallback progress) = 0;
+
+  DOCUMENT("Cycle the currently active window if there are more windows to capture.");
+  virtual void CycleActiveWindow() = 0;
 
 protected:
   ITargetControl() = default;
@@ -1953,10 +2039,11 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_BecomeRemoteServer(
 
 DOCUMENT(R"(Retrieve the default and recommended set of capture options.
 
-:param CaptureOptions opts: A reference to a :class:`CaptureOptions` where the options will be
-  stored.
+:return: The default capture options.
+:rtype: CaptureOptions
 )");
-extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_GetDefaultCaptureOptions(CaptureOptions *opts);
+extern "C" RENDERDOC_API void RENDERDOC_CC
+RENDERDOC_GetDefaultCaptureOptions(CaptureOptions *defaultOpts);
 
 DOCUMENT(R"(Begin injecting speculatively into all new processes started on the system. Where
 supported by platform, configuration, and setup begin injecting speculatively into all new processes
@@ -2010,6 +2097,7 @@ DOCUMENT(R"(Launch an application and inject into it to allow capturing.
 :param str cmdLine: The command line to use when running the application, it will be processed in a
   platform specific way to generate arguments.
 :param list env: Any :class:`EnvironmentModification` that should be made when running the program.
+:param str capturefile: The capture file path template, or blank to use a default location.
 :param CaptureOptions opts: The capture options to use when injecting into the program.
 :param bool waitForExit: If ``True`` this function will block until the process exits.
 :return: The :class:`ExecuteResult` indicating both the status of the operation (success or failure)
@@ -2019,13 +2107,14 @@ DOCUMENT(R"(Launch an application and inject into it to allow capturing.
 )");
 extern "C" RENDERDOC_API ExecuteResult RENDERDOC_CC
 RENDERDOC_ExecuteAndInject(const char *app, const char *workingDir, const char *cmdLine,
-                           const rdcarray<EnvironmentModification> &env, const char *logfile,
+                           const rdcarray<EnvironmentModification> &env, const char *capturefile,
                            const CaptureOptions &opts, bool waitForExit);
 
 DOCUMENT(R"(Where supported by operating system and permissions, inject into a running process.
 
 :param int pid: The Process ID (PID) to inject into.
 :param list env: Any :class:`EnvironmentModification` that should be made when running the program.
+:param str capturefile: The capture file path template, or blank to use a default location.
 :param CaptureOptions opts: The capture options to use when injecting into the program.
 :param bool waitForExit: If ``True`` this function will block until the process exits.
 :return: The :class:`ExecuteResult` indicating both the status of the operation (success or failure)
@@ -2035,7 +2124,7 @@ DOCUMENT(R"(Where supported by operating system and permissions, inject into a r
 )");
 extern "C" RENDERDOC_API ExecuteResult RENDERDOC_CC
 RENDERDOC_InjectIntoProcess(uint32_t pid, const rdcarray<EnvironmentModification> &env,
-                            const char *logfile, const CaptureOptions &opts, bool waitForExit);
+                            const char *capturefile, const CaptureOptions &opts, bool waitForExit);
 
 DOCUMENT(R"(When debugging RenderDoc it can be useful to capture itself by doing a side-build with a
 temporary name. This function wraps up the use of the in-application API to start a capture.
@@ -2055,9 +2144,22 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_EndSelfHostCapture(const ch
 // Vulkan layer handling
 //////////////////////////////////////////////////////////////////////////
 
+DOCUMENT("Structure containing all the information about vulkan layer registration");
+struct VulkanLayerRegistrationInfo
+{
+  DOCUMENT(":class:`VulkanLayerFlags` detailing the current registration.");
+  VulkanLayerFlags flags;
+
+  DOCUMENT("A list of jsons that should be registered");
+  rdcarray<rdcstr> myJSONs;
+
+  DOCUMENT("A list of jsons that should be unregistered / updated");
+  rdcarray<rdcstr> otherJSONs;
+};
+
 DOCUMENT("Internal function for determining vulkan layer registration status.");
-extern "C" RENDERDOC_API bool RENDERDOC_CC RENDERDOC_NeedVulkanLayerRegistration(
-    VulkanLayerFlags *flags, rdcarray<rdcstr> *myJSONs, rdcarray<rdcstr> *otherJSONs);
+extern "C" RENDERDOC_API bool RENDERDOC_CC
+RENDERDOC_NeedVulkanLayerRegistration(VulkanLayerRegistrationInfo *info);
 
 DOCUMENT("Internal function for updating vulkan layer registration.");
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_UpdateVulkanLayerRegistration(bool systemLevel);
@@ -2116,6 +2218,23 @@ This will be in the form "MAJOR.MINOR"
 )");
 extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetVersionString();
 
+DOCUMENT(R"(Retrieves the commit hash used to build.
+
+This will be in the form "0123456789abcdef0123456789abcdef01234567"
+
+:return: The commit hash.
+:rtype: ``str``
+)");
+extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetCommitHash();
+
+DOCUMENT(R"(Retrieves the driver information (if available) for a given graphics API.
+
+:param GraphicsAPI api: The API to get driver information for.
+:return: A :class:`DriverInformation` containing the driver information.
+:rtype: DriverInformation
+)");
+extern "C" RENDERDOC_API DriverInformation RENDERDOC_CC RENDERDOC_GetDriverInformation(GraphicsAPI api);
+
 DOCUMENT("Internal function for retrieving a config setting.");
 extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetConfigSetting(const char *name);
 
@@ -2135,8 +2254,18 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_GetAndroidFriendlyName(cons
 DOCUMENT("Internal function for enumerating android devices.");
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_EnumerateAndroidDevices(rdcstr *deviceList);
 
+DOCUMENT("Internal function for initialising android use.");
+extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_AndroidInitialise();
+
+DOCUMENT("Internal function for shutting down android use.");
+extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_AndroidShutdown();
+
+DOCUMENT("Internal function for checking android support.");
+extern "C" RENDERDOC_API bool RENDERDOC_CC RENDERDOC_IsAndroidSupported(const char *device);
+
 DOCUMENT("Internal function for starting an android remote server.");
-extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_StartAndroidRemoteServer(const char *device);
+extern "C" RENDERDOC_API ReplayStatus RENDERDOC_CC
+RENDERDOC_StartAndroidRemoteServer(const char *device);
 
 DOCUMENT("Internal function for checking remote Android package for requirements");
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_CheckAndroidPackage(const char *hostname,
