@@ -76,6 +76,13 @@ version of RenderDoc that addes a new section type. They should be considered eq
   This section contains a .rgp profile from AMD's RGP tool, which can be extracted and loaded.
 
   The name for this section will be "amd/rgp/profile".
+
+.. data:: ExtendedThumbnail
+
+  This section contains a thumbnail in format other than JPEG. For example, when it needs to be
+  lossless.
+
+  The name for this section will be "renderdoc/internal/exthumb".
 )");
 enum class SectionType : uint32_t
 {
@@ -87,6 +94,7 @@ enum class SectionType : uint32_t
   Notes,
   ResourceRenames,
   AMDRGPProfile,
+  ExtendedThumbnail,
   Count,
 };
 
@@ -1320,6 +1328,12 @@ enum class GPUVendor : uint32_t
 
 DECLARE_REFLECTION_ENUM(GPUVendor);
 
+DOCUMENT(R"(Get the GPUVendor for a given PCI Vendor ID.
+
+:param int vendorID: The PCI Vendor ID
+:return: The vendor identified
+:rtype: GPUVendor
+)");
 constexpr GPUVendor GPUVendorFromPCIVendor(uint32_t vendorID)
 {
   // temporarily disable clang-format to make this more readable.
@@ -1376,6 +1390,10 @@ constexpr inline bool IsD3D(GraphicsAPI api)
 
 DOCUMENT(R"(Identifies a shader encoding used to pass shader code to an API.
 
+.. data:: Unknown
+
+  Unknown or unprocessable format.
+
 .. data:: DXBC
 
   DXBC binary shader, used by D3D11 and D3D12.
@@ -1387,16 +1405,43 @@ DOCUMENT(R"(Identifies a shader encoding used to pass shader code to an API.
 .. data:: SPIRV
 
   SPIR-V binary shader, used by Vulkan and with an extension by OpenGL.
+
+.. data:: SPIRVAsm
+
+  Canonical SPIR-V assembly form, used (indirectly via :data:`SPIRV`) by Vulkan and with an
+  extension by OpenGL.
+
+.. data:: HLSL
+
+  HLSL in string format, used by D3D11, D3D12, and Vulkan/GL via compilation to SPIR-V.
 )");
 enum class ShaderEncoding : uint32_t
 {
+  Unknown,
+  First = Unknown,
   DXBC,
   GLSL,
   SPIRV,
+  SPIRVAsm,
+  HLSL,
+  Count,
 };
 
+ITERABLE_OPERATORS(ShaderEncoding);
 DECLARE_REFLECTION_ENUM(ShaderEncoding);
 
+DOCUMENT(R"(Check whether or not this is a human readable text representation.
+
+:param ShaderEncoding e: The encoding to check.
+:return: ``True`` if it describes a text representation, ``False`` for a bytecode representation.
+:rtype: ``bool``
+)");
+constexpr inline bool IsTextRepresentation(ShaderEncoding encoding)
+{
+  return encoding == ShaderEncoding::HLSL || encoding == ShaderEncoding::GLSL ||
+         encoding == ShaderEncoding::SPIRVAsm;
+}
+
 DOCUMENT(R"(A primitive topology used for processing vertex data.
 
 .. data:: Unknown
@@ -1664,7 +1709,7 @@ DOCUMENT(R"(Check whether or not this is a strip-type topology.
 
 :param Topology t: The topology to check.
 :return: ``True`` if it describes a strip topology, ``False`` for a list.
-:rtype: ``int``
+:rtype: ``bool``
 )");
 constexpr inline bool IsStrip(Topology topology)
 {
@@ -2838,6 +2883,17 @@ enum class GPUCounter : uint32_t
 ITERABLE_OPERATORS(GPUCounter);
 DECLARE_REFLECTION_ENUM(GPUCounter);
 
+DOCUMENT(R"(Check whether or not this is a Generic counter.
+
+:param GPUCounter c: The counter.
+:return: ``True`` if it is a generic counter, ``False`` if it's not.
+:rtype: ``bool``
+)");
+inline constexpr bool IsGenericCounter(GPUCounter c)
+{
+  return c < GPUCounter::Count;
+}
+
 DOCUMENT(R"(Check whether or not this is an AMD private counter.
 
 :param GPUCounter c: The counter.
@@ -3030,6 +3086,22 @@ a remote server.
 
   The API failed to replay the capture, with some runtime error that couldn't be determined until
   the replay began.
+
+.. data:: AndroidGrantPermissionsFailed
+
+  Failed to grant runtime permissions when installing Android remote server.
+
+.. data:: AndroidABINotFound
+
+  Couldn't determine supported ABIs when installing Android remote server.
+
+.. data:: AndroidAPKFolderNotFound
+
+  Couldn't find the build-android folder which contains the Android remote server APK.
+
+.. data:: AndroidAPKInstallFailed
+
+  Failed to install Android remote server.
 )");
 enum class ReplayStatus : uint32_t
 {
@@ -3053,6 +3125,10 @@ enum class ReplayStatus : uint32_t
   APIDataCorrupted,
   APIReplayFailed,
   JDWPFailure,
+  AndroidGrantPermissionsFailed,
+  AndroidABINotFound,
+  AndroidAPKFolderNotFound,
+  AndroidAPKInstallFailed
 };
 
 DECLARE_REFLECTION_ENUM(ReplayStatus);
@@ -3106,6 +3182,7 @@ enum class TargetControlMessageType : uint32_t
   RegisterAPI,
   NewChild,
   CaptureProgress,
+  CapturableWindowCount
 };
 
 DECLARE_REFLECTION_ENUM(TargetControlMessageType);
