@@ -32,6 +32,9 @@ DOCUMENT("The contents of a single binding element within a descriptor set, poss
 struct BindingElement
 {
   DOCUMENT("");
+  BindingElement() = default;
+  BindingElement(const BindingElement &) = default;
+
   bool operator==(const BindingElement &o) const
   {
     return viewResourceId == o.viewResourceId && resourceResourceId == o.resourceResourceId &&
@@ -179,6 +182,9 @@ DOCUMENT("The contents of a single binding within a descriptor set, either array
 struct DescriptorBinding
 {
   DOCUMENT("");
+  DescriptorBinding() = default;
+  DescriptorBinding(const DescriptorBinding &) = default;
+
   bool operator==(const DescriptorBinding &o) const
   {
     return descriptorCount == o.descriptorCount && type == o.type && stageFlags == o.stageFlags &&
@@ -215,6 +221,9 @@ DOCUMENT("The contents of a descriptor set.");
 struct DescriptorSet
 {
   DOCUMENT("");
+  DescriptorSet() = default;
+  DescriptorSet(const DescriptorSet &) = default;
+
   bool operator==(const DescriptorSet &o) const
   {
     return layoutResourceId == o.layoutResourceId &&
@@ -249,6 +258,10 @@ This list is indexed by the binding, so it may be sparse (some entries do not co
 DOCUMENT("Describes the object and descriptor set bindings of a Vulkan pipeline object.");
 struct Pipeline
 {
+  DOCUMENT("");
+  Pipeline() = default;
+  Pipeline(const Pipeline &) = default;
+
   DOCUMENT("The :class:`ResourceId` of the pipeline object.");
   ResourceId pipelineResourceId;
   DOCUMENT("The :class:`ResourceId` of the pipeline layout object.");
@@ -263,6 +276,10 @@ struct Pipeline
 DOCUMENT("Describes the Vulkan index buffer binding.")
 struct IndexBuffer
 {
+  DOCUMENT("");
+  IndexBuffer() = default;
+  IndexBuffer(const IndexBuffer &) = default;
+
   DOCUMENT("The :class:`ResourceId` of the index buffer.");
   ResourceId resourceId;
 
@@ -273,6 +290,10 @@ struct IndexBuffer
 DOCUMENT("Describes the vulkan input assembly configuration.");
 struct InputAssembly
 {
+  DOCUMENT("");
+  InputAssembly() = default;
+  InputAssembly(const InputAssembly &) = default;
+
   DOCUMENT("``True`` if primitive restart is enabled for strip primitives.");
   bool primitiveRestartEnable = false;
 
@@ -284,6 +305,9 @@ DOCUMENT("Describes the configuration of a single vertex attribute.");
 struct VertexAttribute
 {
   DOCUMENT("");
+  VertexAttribute() = default;
+  VertexAttribute(const VertexAttribute &) = default;
+
   bool operator==(const VertexAttribute &o) const
   {
     return location == o.location && binding == o.binding && format == o.format &&
@@ -317,6 +341,9 @@ DOCUMENT("Describes a vertex binding.");
 struct VertexBinding
 {
   DOCUMENT("");
+  VertexBinding() = default;
+  VertexBinding(const VertexBinding &) = default;
+
   bool operator==(const VertexBinding &o) const
   {
     return vertexBufferBinding == o.vertexBufferBinding && byteStride == o.byteStride &&
@@ -354,6 +381,9 @@ DOCUMENT("Describes a single Vulkan vertex buffer binding.")
 struct VertexBuffer
 {
   DOCUMENT("");
+  VertexBuffer() = default;
+  VertexBuffer(const VertexBuffer &) = default;
+
   bool operator==(const VertexBuffer &o) const
   {
     return resourceId == o.resourceId && byteOffset == o.byteOffset;
@@ -375,6 +405,10 @@ struct VertexBuffer
 DOCUMENT("Describes the fixed-function vertex input fetch setup.");
 struct VertexInput
 {
+  DOCUMENT("");
+  VertexInput() = default;
+  VertexInput(const VertexInput &) = default;
+
   DOCUMENT("A list of :class:`VKVertexAttribute` with the vertex attributes.");
   rdcarray<VertexAttribute> attributes;
   DOCUMENT("A list of :class:`VKVertexBinding` with the vertex bindings.");
@@ -387,6 +421,9 @@ DOCUMENT("The provided value for a specialization constant.");
 struct SpecializationConstant
 {
   DOCUMENT("");
+  SpecializationConstant() = default;
+  SpecializationConstant(const SpecializationConstant &) = default;
+
   bool operator==(const SpecializationConstant &o) const
   {
     return specializationId == o.specializationId && data == o.data;
@@ -408,6 +445,10 @@ struct SpecializationConstant
 DOCUMENT("Describes a Vulkan shader stage.");
 struct Shader
 {
+  DOCUMENT("");
+  Shader() = default;
+  Shader(const Shader &) = default;
+
   DOCUMENT("The :class:`ResourceId` of the shader module object.");
   ResourceId resourceId;
   DOCUMENT("The name of the entry point in the shader module that is used.");
@@ -431,6 +472,10 @@ mapping data.
 DOCUMENT("Describes the state of the fixed-function tessellator.");
 struct Tessellation
 {
+  DOCUMENT("");
+  Tessellation() = default;
+  Tessellation(const Tessellation &) = default;
+
   DOCUMENT("The number of control points in each input patch.");
   uint32_t numControlPoints = 0;
 
@@ -438,10 +483,74 @@ struct Tessellation
   bool domainOriginUpperLeft = true;
 };
 
+DOCUMENT("Describes a single transform feedback binding.");
+struct XFBBuffer
+{
+  DOCUMENT("");
+  XFBBuffer() = default;
+  XFBBuffer(const XFBBuffer &) = default;
+
+  bool operator==(const XFBBuffer &o) const
+  {
+    return active == o.active && bufferResourceId == o.bufferResourceId &&
+           byteOffset == o.byteOffset && byteSize == o.byteSize &&
+           counterBufferResourceId == o.counterBufferResourceId &&
+           counterBufferOffset == o.counterBufferOffset;
+  }
+  bool operator<(const XFBBuffer &o) const
+  {
+    if(!(active == o.active))
+      return active < o.active;
+    if(!(bufferResourceId == o.bufferResourceId))
+      return bufferResourceId < o.bufferResourceId;
+    if(!(byteOffset == o.byteOffset))
+      return byteOffset < o.byteOffset;
+    if(!(byteSize == o.byteSize))
+      return byteSize < o.byteSize;
+    if(!(counterBufferResourceId == o.counterBufferResourceId))
+      return counterBufferResourceId < o.counterBufferResourceId;
+    if(!(counterBufferOffset == o.counterBufferOffset))
+      return counterBufferOffset < o.counterBufferOffset;
+    return false;
+  }
+
+  DOCUMENT("A flag indicating if this buffer is active or not.");
+  bool active = false;
+
+  DOCUMENT("The :class:`ResourceId` of the bound data buffer.");
+  ResourceId bufferResourceId;
+
+  DOCUMENT("The offset in bytes to the start of the data in the :data:`bufferResourceId`.");
+  uint64_t byteOffset = 0;
+
+  DOCUMENT("The size in bytes of the data buffer.");
+  uint64_t byteSize = 0;
+
+  DOCUMENT("The :class:`ResourceId` of the buffer storing the counter value (if set).");
+  ResourceId counterBufferResourceId;
+
+  DOCUMENT("The offset in bytes to the counter in the :data:`counterBufferResourceId`.");
+  uint64_t counterBufferOffset = 0;
+};
+
+DOCUMENT("Describes the state of the fixed-function transform feedback.");
+struct TransformFeedback
+{
+  DOCUMENT("");
+  TransformFeedback() = default;
+  TransformFeedback(const TransformFeedback &) = default;
+
+  DOCUMENT("The bound transform feedback buffers.");
+  rdcarray<XFBBuffer> buffers;
+};
+
 DOCUMENT("Describes a combined viewport and scissor region.");
 struct ViewportScissor
 {
   DOCUMENT("");
+  ViewportScissor() = default;
+  ViewportScissor(const ViewportScissor &) = default;
+
   bool operator==(const ViewportScissor &o) const { return vp == o.vp && scissor == o.scissor; }
   bool operator<(const ViewportScissor &o) const { return vp == o.vp && scissor == o.scissor; }
   DOCUMENT("The :class:`Viewport`.");
@@ -453,6 +562,10 @@ struct ViewportScissor
 DOCUMENT("Describes the view state in the pipeline.");
 struct ViewState
 {
+  DOCUMENT("");
+  ViewState() = default;
+  ViewState(const ViewState &) = default;
+
   DOCUMENT("A list of :class:`VKViewportScissor`.");
   rdcarray<ViewportScissor> viewportScissors;
 };
@@ -460,6 +573,10 @@ struct ViewState
 DOCUMENT("Describes the rasterizer state in the pipeline.");
 struct Rasterizer
 {
+  DOCUMENT("");
+  Rasterizer() = default;
+  Rasterizer(const Rasterizer &) = default;
+
   DOCUMENT(R"(``True`` if pixels outside of the near and far depth planes should be clamped and
 to ``0.0`` to ``1.0`` and not clipped.
 )");
@@ -500,6 +617,10 @@ See :data:`conservativeRasterizationMode`
 DOCUMENT("Describes the multisampling state in the pipeline.");
 struct MultiSample
 {
+  DOCUMENT("");
+  MultiSample() = default;
+  MultiSample(const MultiSample &) = default;
+
   DOCUMENT("How many samples to use when rasterizing.");
   uint32_t rasterSamples = 0;
   DOCUMENT("``True`` if rendering should happen at sample-rate frequency.");
@@ -513,6 +634,10 @@ struct MultiSample
 DOCUMENT("Describes the pipeline blending state.");
 struct ColorBlendState
 {
+  DOCUMENT("");
+  ColorBlendState() = default;
+  ColorBlendState(const ColorBlendState &) = default;
+
   DOCUMENT("``True`` if alpha-to-coverage should be used when blending to an MSAA target.");
   bool alphaToCoverageEnable = false;
   DOCUMENT("``True`` if alpha-to-one should be used when blending to an MSAA target.");
@@ -528,6 +653,10 @@ struct ColorBlendState
 DOCUMENT("Describes the pipeline depth-stencil state.");
 struct DepthStencil
 {
+  DOCUMENT("");
+  DepthStencil() = default;
+  DepthStencil(const DepthStencil &) = default;
+
   DOCUMENT("``True`` if depth testing should be performed.");
   bool depthTestEnable = false;
   DOCUMENT("``True`` if depth values should be written to the depth target.");
@@ -554,6 +683,10 @@ struct DepthStencil
 DOCUMENT("Describes the setup of a renderpass and subpasses.");
 struct RenderPass
 {
+  DOCUMENT("");
+  RenderPass() = default;
+  RenderPass(const RenderPass &) = default;
+
   DOCUMENT("The :class:`ResourceId` of the render pass.");
   ResourceId resourceId;
 
@@ -586,6 +719,9 @@ DOCUMENT("Describes a single attachment in a framebuffer object.");
 struct Attachment
 {
   DOCUMENT("");
+  Attachment() = default;
+  Attachment(const Attachment &) = default;
+
   bool operator==(const Attachment &o) const
   {
     return viewResourceId == o.viewResourceId && imageResourceId == o.imageResourceId &&
@@ -642,6 +778,10 @@ struct Attachment
 DOCUMENT("Describes a framebuffer object and its attachments.");
 struct Framebuffer
 {
+  DOCUMENT("");
+  Framebuffer() = default;
+  Framebuffer(const Framebuffer &) = default;
+
   DOCUMENT("The :class:`ResourceId` of the framebuffer object.");
   ResourceId resourceId;
 
@@ -659,6 +799,10 @@ struct Framebuffer
 DOCUMENT("Describes the render area for a render pass instance.");
 struct RenderArea
 {
+  DOCUMENT("");
+  RenderArea() = default;
+  RenderArea(const RenderArea &) = default;
+
   DOCUMENT("The X co-ordinate of the render area.");
   int32_t x = 0;
   DOCUMENT("The Y co-ordinate of the render area.");
@@ -672,6 +816,10 @@ struct RenderArea
 DOCUMENT("Describes the current pass instance at the current time.");
 struct CurrentPass
 {
+  DOCUMENT("");
+  CurrentPass() = default;
+  CurrentPass(const CurrentPass &) = default;
+
   DOCUMENT("The :class:`VKRenderPass` that is currently active.");
   RenderPass renderpass;
   DOCUMENT("The :class:`VKFramebuffer` that is currently being used.");
@@ -684,6 +832,9 @@ DOCUMENT("Contains the layout of a range of subresources in an image.");
 struct ImageLayout
 {
   DOCUMENT("");
+  ImageLayout() = default;
+  ImageLayout(const ImageLayout &) = default;
+
   bool operator==(const ImageLayout &o) const
   {
     return baseMip == o.baseMip && baseLayer == o.baseLayer && numMip == o.numMip &&
@@ -719,6 +870,9 @@ DOCUMENT("Contains the current layout of all subresources in the image.");
 struct ImageData
 {
   DOCUMENT("");
+  ImageData() = default;
+  ImageData(const ImageData &) = default;
+
   bool operator==(const ImageData &o) const { return resourceId == o.resourceId; }
   bool operator<(const ImageData &o) const
   {
@@ -771,6 +925,9 @@ struct State
   DOCUMENT("A :class:`VKTessellation` describing the tessellation stage.");
   Tessellation tessellation;
 
+  DOCUMENT("A :class:`VKTransformFeedback` describing the tessellation stage.");
+  TransformFeedback transformFeedback;
+
   DOCUMENT("A :class:`VKViewState` describing the viewport setup.");
   ViewState viewportScissor;
   DOCUMENT("A :class:`VKRasterizer` describing rasterization.");
@@ -805,6 +962,8 @@ DECLARE_REFLECTION_STRUCT(VKPipe::VertexInput);
 DECLARE_REFLECTION_STRUCT(VKPipe::SpecializationConstant);
 DECLARE_REFLECTION_STRUCT(VKPipe::Shader);
 DECLARE_REFLECTION_STRUCT(VKPipe::Tessellation);
+DECLARE_REFLECTION_STRUCT(VKPipe::XFBBuffer);
+DECLARE_REFLECTION_STRUCT(VKPipe::TransformFeedback);
 DECLARE_REFLECTION_STRUCT(VKPipe::ViewportScissor);
 DECLARE_REFLECTION_STRUCT(VKPipe::ViewState);
 DECLARE_REFLECTION_STRUCT(VKPipe::Rasterizer);
