@@ -86,9 +86,21 @@ std::string DoStringise(const ReplayLogType &el)
 {
   BEGIN_ENUM_STRINGISE(ReplayLogType);
   {
-    STRINGISE_ENUM_CLASS_NAMED(eReplay_Full, "Full replay including draw");
-    STRINGISE_ENUM_CLASS_NAMED(eReplay_WithoutDraw, "Replay without draw");
-    STRINGISE_ENUM_CLASS_NAMED(eReplay_OnlyDraw, "Replay only draw");
+    STRINGISE_ENUM_NAMED(eReplay_Full, "Full replay including draw");
+    STRINGISE_ENUM_NAMED(eReplay_WithoutDraw, "Replay without draw");
+    STRINGISE_ENUM_NAMED(eReplay_OnlyDraw, "Replay only draw");
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+std::string DoStringise(const VendorExtensions &el)
+{
+  BEGIN_ENUM_STRINGISE(VendorExtensions);
+  {
+    STRINGISE_ENUM_CLASS(NvAPI);
+    STRINGISE_ENUM_CLASS_NAMED(OpenGL_Ext, "Unsupported GL extensions");
+    STRINGISE_ENUM_CLASS_NAMED(Vulkan_Ext, "Unsupported Vulkan extensions");
   }
   END_ENUM_STRINGISE();
 }
@@ -181,8 +193,6 @@ RenderDoc &RenderDoc::Inst()
 
 void RenderDoc::RecreateCrashHandler()
 {
-  UnloadCrashHandler();
-
 #if ENABLED(RDOC_CRASH_HANDLER)
   m_ExHandler = new CrashHandler(m_ExHandler);
 #endif
@@ -368,6 +378,8 @@ RenderDoc::~RenderDoc()
     m_RemoteThread = 0;
   }
 
+  Process::Shutdown();
+
   Network::Shutdown();
 
   Threading::Shutdown();
@@ -542,27 +554,7 @@ void RenderDoc::Tick()
 
   if(!prev_focus && cur_focus)
   {
-    m_Cap = 0;
-
-    // can only shift focus if we have multiple windows
-    if(m_WindowFrameCapturers.size() > 1)
-    {
-      for(auto it = m_WindowFrameCapturers.begin(); it != m_WindowFrameCapturers.end(); ++it)
-      {
-        if(it->first == m_ActiveWindow)
-        {
-          auto nextit = it;
-          ++nextit;
-
-          if(nextit != m_WindowFrameCapturers.end())
-            m_ActiveWindow = nextit->first;
-          else
-            m_ActiveWindow = m_WindowFrameCapturers.begin()->first;
-
-          break;
-        }
-      }
-    }
+    CycleActiveWindow();
   }
   if(!prev_cap && cur_cap)
   {
@@ -573,6 +565,31 @@ void RenderDoc::Tick()
   prev_cap = cur_cap;
 }
 
+void RenderDoc::CycleActiveWindow()
+{
+  m_Cap = 0;
+
+  // can only shift focus if we have multiple windows
+  if(m_WindowFrameCapturers.size() > 1)
+  {
+    for(auto it = m_WindowFrameCapturers.begin(); it != m_WindowFrameCapturers.end(); ++it)
+    {
+      if(it->first == m_ActiveWindow)
+      {
+        auto nextit = it;
+        ++nextit;
+
+        if(nextit != m_WindowFrameCapturers.end())
+          m_ActiveWindow = nextit->first;
+        else
+          m_ActiveWindow = m_WindowFrameCapturers.begin()->first;
+
+        break;
+      }
+    }
+  }
+}
+
 string RenderDoc::GetOverlayText(RDCDriver driver, uint32_t frameNumber, int flags)
 {
   const bool activeWindow = (flags & eOverlay_ActiveWindow);
@@ -700,7 +717,7 @@ bool RenderDoc::ShouldTriggerCapture(uint32_t frameNumber)
 }
 
 RDCFile *RenderDoc::CreateRDC(RDCDriver driver, uint32_t frameNum, void *thpixels, size_t thlen,
-                              uint16_t thwidth, uint16_t thheight)
+                              uint16_t thwidth, uint16_t thheight, FileType thformat)
 {
   RDCFile *ret = new RDCFile;
 
@@ -729,6 +746,9 @@ RDCFile *RenderDoc::CreateRDC(RDCDriver driver, uint32_t frameNum, void *thpixel
     th.pixels = (const byte *)thpixels;
     th.width = thwidth;
     th.height = thheight;
+    RDCASSERT(thformat == FileType::JPG || thformat == FileType::PNG || thformat == FileType::BMP ||
+              thformat == FileType::TGA || thformat == FileType::Raw);
+    th.format = thformat;
     thumb = &th;
   }
 
@@ -1039,6 +1059,53 @@ map<RDCDriver, string> RenderDoc::GetRemoteDrivers()
   return ret;
 }
 
+DriverInformation RenderDoc::GetDriverInformation(GraphicsAPI api)
+{
+  DriverInformation ret = {};
+
+  RDCDriver driverType = RDCDriver::Unknown;
+  switch(api)
+  {
+    case GraphicsAPI::D3D11: driverType = RDCDriver::D3D11; break;
+    case GraphicsAPI::D3D12: driverType = RDCDriver::D3D12; break;
+    case GraphicsAPI::OpenGL: driverType = RDCDriver::OpenGL; break;
+    case GraphicsAPI::Vulkan: driverType = RDCDriver::Vulkan; break;
+  }
+
+  if(driverType == RDCDriver::Unknown || !HasReplayDriver(driverType))
+    return ret;
+
+  IReplayDriver *driver = NULL;
+  ReplayStatus status = CreateProxyReplayDriver(driverType, &driver);
+
+  if(status == ReplayStatus::Succeeded)
+  {
+    ret = driver->GetDriverInfo();
+  }
+  else
+  {
+    RDCERR("Couldn't create proxy replay driver for %s: %s", ToStr(driverType).c_str(),
+           ToStr(status).c_str());
+  }
+
+  if(driver)
+    driver->Shutdown();
+
+  return ret;
+}
+
+void RenderDoc::EnableVendorExtensions(VendorExtensions ext)
+{
+  m_VendorExts[(int)ext] = true;
+
+  RDCWARN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+  RDCWARN("!!! Vendor Extension enabled: %s", ToStr(ext).c_str());
+  RDCWARN("!!! ");
+  RDCWARN("!!! This can cause crashes, incorrect replay, or other problems and");
+  RDCWARN("!!! is explicitly unsupported. Do not enable without understanding.");
+  RDCWARN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+}
+
 void RenderDoc::SetCaptureOptions(const CaptureOptions &opts)
 {
   m_Options = opts;
@@ -1087,15 +1154,36 @@ void RenderDoc::FinishCaptureWriting(RDCFile *rdc, uint32_t frameNumber)
       delete w;
     }
 
-    delete rdc;
+    const RDCThumb &thumb = rdc->GetThumbnail();
+    if(thumb.format != FileType::JPG && thumb.width > 0 && thumb.height > 0)
+    {
+      SectionProperties props = {};
+      props.type = SectionType::ExtendedThumbnail;
+      props.version = 1;
+      StreamWriter *w = rdc->WriteSection(props);
+
+      ExtThumbnailHeader header;
+      header.width = thumb.width;
+      header.height = thumb.height;
+      header.len = thumb.len;
+      header.format = (uint32_t)thumb.format;
+      w->Write(header);
+      w->Write(thumb.pixels, thumb.len);
+
+      w->Finish();
+
+      delete w;
+    }
 
     RDCLOG("Written to disk: %s", m_CurrentLogFile.c_str());
 
-    CaptureData cap(m_CurrentLogFile, Timing::GetUnixTimestamp(), frameNumber);
+    CaptureData cap(m_CurrentLogFile, Timing::GetUnixTimestamp(), rdc->GetDriver(), frameNumber);
     {
       SCOPED_LOCK(m_CaptureLock);
       m_Captures.push_back(cap);
     }
+
+    delete rdc;
   }
 
   RenderDoc::Inst().SetProgress(CaptureProgress::FileWriting, 1.0f);
@@ -1116,7 +1204,7 @@ void RenderDoc::RemoveDeviceFrameCapturer(void *dev)
 {
   if(dev == NULL)
   {
-    RDCERR("Invalid device pointer: %#p / %#p", dev);
+    RDCERR("Invalid device pointer: %#p", dev);
     return;
   }
 
