@@ -52,9 +52,6 @@ bool is_exr_file(FILE *f);
 struct ICrashHandler
 {
   virtual ~ICrashHandler() {}
-  virtual void WriteMinidump() = 0;
-  virtual void WriteMinidump(void *data) = 0;
-
   virtual void RegisterMemoryRegion(void *mem, size_t size) = 0;
   virtual void UnregisterMemoryRegion(void *mem) = 0;
 };
@@ -209,14 +206,27 @@ enum ReplayLogType
 
 DECLARE_REFLECTION_ENUM(ReplayLogType);
 
+enum class VendorExtensions
+{
+  NvAPI = 0,
+  First = NvAPI,
+  OpenGL_Ext,
+  Vulkan_Ext,
+  Count,
+};
+
+DECLARE_REFLECTION_ENUM(VendorExtensions);
+ITERABLE_OPERATORS(VendorExtensions);
+
 struct CaptureData
 {
-  CaptureData(string p, uint64_t t, uint32_t f)
-      : path(p), timestamp(t), frameNumber(f), retrieved(false)
+  CaptureData(string p, uint64_t t, RDCDriver d, uint32_t f)
+      : path(p), timestamp(t), driver(d), frameNumber(f), retrieved(false)
   {
   }
   string path;
   uint64_t timestamp;
+  RDCDriver driver;
   uint32_t frameNumber;
   bool retrieved;
 };
@@ -388,13 +398,20 @@ public:
   void BecomeRemoteServer(const char *listenhost, uint16_t port, RENDERDOC_KillCallback killReplay,
                           RENDERDOC_PreviewWindowCallback previewWindow);
 
+  DriverInformation GetDriverInformation(GraphicsAPI api);
+
+  // can't be disabled, only enabled then latched
+
+  bool IsVendorExtensionEnabled(VendorExtensions ext) { return m_VendorExts[(int)ext]; }
+  void EnableVendorExtensions(VendorExtensions ext);
+
   void SetCaptureOptions(const CaptureOptions &opts);
   const CaptureOptions &GetCaptureOptions() const { return m_Options; }
   void RecreateCrashHandler();
   void UnloadCrashHandler();
   ICrashHandler *GetCrashHandler() const { return m_ExHandler; }
   RDCFile *CreateRDC(RDCDriver driver, uint32_t frameNum, void *thpixels, size_t thlen,
-                     uint16_t thwidth, uint16_t thheight);
+                     uint16_t thwidth, uint16_t thheight, FileType thformat);
   void FinishCaptureWriting(RDCFile *rdc, uint32_t frameNumber);
 
   void AddChildProcess(uint32_t pid, uint32_t ident)
@@ -538,6 +555,8 @@ public:
 
   string GetOverlayText(RDCDriver driver, uint32_t frameNumber, int flags);
 
+  void CycleActiveWindow();
+  uint32_t GetCapturableWindowCount() { return (uint32_t)m_WindowFrameCapturers.size(); }
 private:
   RenderDoc();
   ~RenderDoc();
@@ -642,6 +661,8 @@ private:
 
   IFrameCapturer *MatchFrameCapturer(void *dev, void *wnd);
 
+  bool m_VendorExts[ENUM_ARRAY_SIZE(VendorExtensions)] = {};
+
   volatile bool m_TargetControlThreadShutdown;
   volatile bool m_ControlClientThreadShutdown;
   Threading::CriticalSection m_SingleClientLock;
