@@ -30,7 +30,23 @@
 #include "os/os_specific.h"
 #include "serialise/serialiser.h"
 
-static const uint32_t TargetControlProtocolVersion = 2;
+static const uint32_t TargetControlProtocolVersion = 4;
+
+static bool IsProtocolVersionSupported(const uint32_t protocolVersion)
+{
+  // 2 -> 3 added a RDCDriver per capture
+  if(protocolVersion == 2)
+    return true;
+
+  // 3 -> 4 added active window cycle and window count packets
+  if(protocolVersion == 3)
+    return true;
+
+  if(protocolVersion == TargetControlProtocolVersion)
+    return true;
+
+  return false;
+}
 
 enum PacketType : uint32_t
 {
@@ -45,6 +61,8 @@ enum PacketType : uint32_t
   ePacket_QueueCapture,
   ePacket_NewChild,
   ePacket_CaptureProgress,
+  ePacket_CycleActiveWindow,
+  ePacket_CapturableWindowCount
 };
 
 DECLARE_REFLECTION_ENUM(PacketType);
@@ -64,6 +82,9 @@ std::string DoStringise(const PacketType &el)
     STRINGISE_ENUM_NAMED(ePacket_DeleteCapture, "Delete Capture");
     STRINGISE_ENUM_NAMED(ePacket_QueueCapture, "Queue Capture");
     STRINGISE_ENUM_NAMED(ePacket_NewChild, "New Child");
+    STRINGISE_ENUM_NAMED(ePacket_CaptureProgress, "Capture Progress");
+    STRINGISE_ENUM_NAMED(ePacket_CycleActiveWindow, "Cycle Active Window");
+    STRINGISE_ENUM_NAMED(ePacket_CapturableWindowCount, "Capturable Window Count");
   }
   END_ENUM_STRINGISE();
 }
@@ -118,6 +139,7 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
   std::vector<pair<uint32_t, uint32_t> > children;
   std::map<RDCDriver, bool> drivers;
   float prevCaptureProgress = captureProgress;
+  uint32_t prevWindows = 0;
 
   while(client)
   {
@@ -135,6 +157,8 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
     std::vector<CaptureData> caps = RenderDoc::Inst().GetCaptures();
     std::vector<pair<uint32_t, uint32_t> > childprocs = RenderDoc::Inst().GetChildProcesses();
 
+    uint32_t curWindows = RenderDoc::Inst().GetCapturableWindowCount();
+
     if(curdrivers != drivers)
     {
       // find the first difference, either a new key or a key with a different value, and send it.
@@ -192,6 +216,8 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
         SERIALISE_ELEMENT(captures.back().timestamp);
         SERIALISE_ELEMENT(path);
         SERIALISE_ELEMENT(buf);
+        if(version >= 3)
+          SERIALISE_ELEMENT(captures.back().driver);
       }
     }
     else if(childprocs.size() != children.size())
@@ -227,6 +253,16 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
         }
       }
     }
+    else if(version >= 4 && prevWindows != curWindows)
+    {
+      prevWindows = curWindows;
+
+      WRITE_DATA_SCOPE();
+      {
+        SCOPED_SERIALISE_CHUNK(ePacket_CapturableWindowCount);
+        SERIALISE_ELEMENT(curWindows);
+      }
+    }
 
     if(curtime > pingtime)
     {
@@ -306,6 +342,10 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
             RenderDoc::Inst().MarkCaptureRetrieved(id);
         }
       }
+      else if(type == ePacket_CycleActiveWindow)
+      {
+        RenderDoc::Inst().CycleActiveWindow();
+      }
 
       reader.EndChunk();
 
@@ -378,8 +418,9 @@ void RenderDoc::TargetControlServerThread(Network::Socket *sock)
 
       ser.EndChunk();
 
-      if(newClient.empty())
+      if(newClient.empty() || !IsProtocolVersionSupported(version))
       {
+        RDCLOG("Invalid/Unsupported handshake '%s' / %d", newClient.c_str(), version);
         SAFE_DELETE(client);
         continue;
       }
@@ -496,17 +537,24 @@ public:
     if(m_Socket == NULL)
       return;
 
-    uint32_t version = TargetControlProtocolVersion;
+    m_Version = 0;
 
     {
       READ_DATA_SCOPE();
-      SERIALISE_ELEMENT(version);
+      SERIALISE_ELEMENT(m_Version);
       SERIALISE_ELEMENT(m_Target);
       SERIALISE_ELEMENT(m_PID);
     }
 
     reader.EndChunk();
 
+    if(!IsProtocolVersionSupported(m_Version))
+    {
+      RDCERR("Unsupported version, got %u", m_Version);
+      SAFE_DELETE(m_Socket);
+      return;
+    }
+
     if(type == ePacket_Handshake)
     {
       RDCLOG("Got remote handshake: %s [%u]", m_Target.c_str(), m_PID);
@@ -579,7 +627,19 @@ public:
       SAFE_DELETE(m_Socket);
   }
 
-  TargetControlMessage ReceiveMessage()
+  void CycleActiveWindow()
+  {
+    if(m_Version < 4)
+      return;
+
+    WRITE_DATA_SCOPE();
+    SCOPED_SERIALISE_CHUNK(ePacket_CycleActiveWindow);
+
+    if(ser.IsErrored())
+      SAFE_DELETE(m_Socket);
+  }
+
+  TargetControlMessage ReceiveMessage(RENDERDOC_ProgressCallback progress)
   {
     TargetControlMessage msg;
     if(m_Socket == NULL)
@@ -659,14 +719,21 @@ public:
 
       bytebuf thumbnail;
 
+      RDCDriver driver = RDCDriver::Unknown;
+
       {
         READ_DATA_SCOPE();
         SERIALISE_ELEMENT(msg.newCapture.captureId).Named("Capture ID");
         SERIALISE_ELEMENT(msg.newCapture.timestamp).Named("timestamp");
         SERIALISE_ELEMENT(msg.newCapture.path).Named("path");
         SERIALISE_ELEMENT(thumbnail);
+        if(m_Version >= 3)
+          SERIALISE_ELEMENT(driver);
       }
 
+      if(driver != RDCDriver::Unknown)
+        msg.newCapture.api = ToStr(driver);
+
       msg.newCapture.local = FileIO::exists(msg.newCapture.path.c_str());
 
       RDCLOG("Got a new capture: %d (time %llu) %d byte thumbnail", msg.newCapture.captureId,
@@ -733,7 +800,7 @@ public:
 
       StreamWriter streamWriter(FileIO::fopen(msg.newCapture.path.c_str(), "wb"), Ownership::Stream);
 
-      ser.SerialiseStream(msg.newCapture.path.c_str(), streamWriter, NULL);
+      ser.SerialiseStream(msg.newCapture.path.c_str(), streamWriter, progress);
 
       if(reader.IsErrored())
       {
@@ -748,6 +815,16 @@ public:
       reader.EndChunk();
       return msg;
     }
+    else if(type == ePacket_CapturableWindowCount)
+    {
+      msg.type = TargetControlMessageType::CapturableWindowCount;
+      uint32_t windows = 0;
+      READ_DATA_SCOPE();
+      SERIALISE_ELEMENT(windows);
+      msg.capturableWindowCount = windows;
+      reader.EndChunk();
+      return msg;
+    }
     else
     {
       RDCERR("Unexpected packed received: %d", type);
@@ -763,7 +840,7 @@ private:
   WriteSerialiser writer;
   ReadSerialiser reader;
   std::string m_Target, m_API, m_BusyClient;
-  uint32_t m_PID;
+  uint32_t m_Version, m_PID;
 
   std::map<uint32_t, std::string> m_CaptureCopies;
 };
