@@ -60,7 +60,7 @@ HRESULT STDMETHODCALLTYPE WrappedID3DUserDefinedAnnotation::QueryInterface(REFII
 {
   if(riid == __uuidof(ID3DUserDefinedAnnotation))
   {
-    *ppvObject = (void *)(ID3DUserDefinedAnnotation *)this;
+    *ppvObject = (ID3DUserDefinedAnnotation *)this;
     AddRef();
     return S_OK;
   }
@@ -139,11 +139,10 @@ WrappedID3D11DeviceContext::WrappedID3D11DeviceContext(WrappedID3D11Device *real
   {
     m_ContextRecord = m_pDevice->GetResourceManager()->AddResourceRecord(m_ResourceID);
     m_ContextRecord->DataInSerialiser = false;
-    m_ContextRecord->SpecialResource = true;
+    m_ContextRecord->InternalResource = true;
     m_ContextRecord->Length = 0;
     m_ContextRecord->NumSubResources = 0;
     m_ContextRecord->SubResources = NULL;
-    m_ContextRecord->ignoreSerialise = true;
   }
 
   m_ScratchSerialiser.SetUserData(GetResourceManager());
@@ -224,6 +223,18 @@ void WrappedID3D11DeviceContext::GetDevice(ID3D11Device **ppDevice)
   (*ppDevice)->AddRef();
 }
 
+bool WrappedID3D11DeviceContext::HasNonMarkerEvents()
+{
+  for(const APIEvent &ev : m_CurEvents)
+  {
+    D3D11Chunk chunk = (D3D11Chunk)m_StructuredFile->chunks[ev.chunkIndex]->metadata.chunkID;
+    if(chunk != D3D11Chunk::PushMarker && chunk != D3D11Chunk::PopMarker)
+      return true;
+  }
+
+  return false;
+}
+
 D3D11ResourceManager *WrappedID3D11DeviceContext::GetResourceManager()
 {
   return m_pDevice->GetResourceManager();
@@ -1570,8 +1581,15 @@ void WrappedID3D11DeviceContext::RecordUpdateStats(ID3D11Resource *res, uint32_t
   // #mivance it might be nice to query the buffer to differentiate
   // between bindings for constant buffers
 
-  size_t bucket = BucketForRecord<ResourceUpdateStats>::Get(Size);
-  updates.sizes[bucket] += 1;
+  if(Size > 0)
+  {
+    size_t bucket = BucketForRecord<ResourceUpdateStats>::Get(Size);
+    updates.sizes[bucket] += 1;
+  }
+  else
+  {
+    updates.sizes[0] += 1;
+  }
 }
 
 void WrappedID3D11DeviceContext::RecordDrawStats(bool instanced, bool indirect, UINT InstanceCount)
