@@ -121,7 +121,9 @@ ID3D11Buffer *D3D11DebugManager::MakeCBuffer(const void *data, size_t size)
 {
   int idx = publicCBufIdx;
 
-  FillCBuffer(PublicCBuffers[idx], data, size);
+  RDCASSERT(size <= PublicCBufferSize, size, PublicCBufferSize);
+
+  FillCBuffer(PublicCBuffers[idx], data, RDCMIN(size, PublicCBufferSize));
 
   publicCBufIdx = (publicCBufIdx + 1) % ARRAY_COUNT(PublicCBuffers);
 
@@ -131,6 +133,7 @@ ID3D11Buffer *D3D11DebugManager::MakeCBuffer(const void *data, size_t size)
 void D3D11DebugManager::InitCommonResources()
 {
   D3D11ShaderCache *shaderCache = m_pDevice->GetShaderCache();
+  D3D11ResourceManager *rm = m_pDevice->GetResourceManager();
 
   std::string multisamplehlsl = GetEmbeddedResource(multisample_hlsl);
 
@@ -153,6 +156,14 @@ void D3D11DebugManager::InitCommonResources()
       shaderCache->MakePShader(multisamplehlsl.c_str(), "RENDERDOC_DepthCopyArrayToMS", "ps_5_0");
   m_pDevice->InternalRef();
 
+  // mark created resources as internal during capture so they aren't included in capture files.
+  rm->SetInternalResource(CopyMSToArrayPS);
+  rm->SetInternalResource(CopyArrayToMSPS);
+  rm->SetInternalResource(FloatCopyMSToArrayPS);
+  rm->SetInternalResource(FloatCopyArrayToMSPS);
+  rm->SetInternalResource(DepthCopyMSToArrayPS);
+  rm->SetInternalResource(DepthCopyArrayToMSPS);
+
   std::string displayhlsl = GetEmbeddedResource(debugcbuffers_h);
   displayhlsl += GetEmbeddedResource(debugcommon_hlsl);
   displayhlsl += GetEmbeddedResource(debugdisplay_hlsl);
@@ -160,10 +171,13 @@ void D3D11DebugManager::InitCommonResources()
   MSArrayCopyVS = shaderCache->MakeVShader(displayhlsl.c_str(), "RENDERDOC_FullscreenVS", "vs_4_0");
   m_pDevice->InternalRef();
 
+  rm->SetInternalResource(MSArrayCopyVS);
+
   for(int i = 0; i < ARRAY_COUNT(PublicCBuffers); i++)
   {
-    PublicCBuffers[i] = MakeCBuffer(sizeof(float) * 4 * 100);
+    PublicCBuffers[i] = MakeCBuffer(PublicCBufferSize);
     m_pDevice->InternalRef();
+    rm->SetInternalResource(PublicCBuffers[i]);
   }
 
   publicCBufIdx = 0;
@@ -1233,6 +1247,7 @@ void D3D11Replay::PixelPicking::Init(WrappedID3D11Device *device)
 
 void D3D11Replay::PixelPicking::Release()
 {
+  SAFE_RELEASE(Texture);
   SAFE_RELEASE(RTV);
   SAFE_RELEASE(StageTexture);
 }
