@@ -140,12 +140,42 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
   {
     m_DeviceRecord = GetResourceManager()->AddResourceRecord(m_ResourceID);
     m_DeviceRecord->DataInSerialiser = false;
-    m_DeviceRecord->SpecialResource = true;
+    m_DeviceRecord->InternalResource = true;
     m_DeviceRecord->Length = 0;
     m_DeviceRecord->NumSubResources = 0;
     m_DeviceRecord->SubResources = NULL;
 
     RenderDoc::Inst().AddDeviceFrameCapturer((ID3D11Device *)this, this);
+
+    {
+      IDXGIDevice *pDXGIDevice = NULL;
+      HRESULT hr = m_pDevice->QueryInterface(__uuidof(IDXGIDevice), (void **)&pDXGIDevice);
+
+      if(FAILED(hr))
+      {
+        RDCERR("Couldn't get DXGI device from D3D device");
+      }
+      else
+      {
+        IDXGIAdapter *pDXGIAdapter = NULL;
+        hr = pDXGIDevice->GetParent(__uuidof(IDXGIAdapter), (void **)&pDXGIAdapter);
+
+        if(SUCCEEDED(hr))
+        {
+          DXGI_ADAPTER_DESC desc = {};
+          pDXGIAdapter->GetDesc(&desc);
+
+          GPUVendor vendor = GPUVendorFromPCIVendor(desc.VendorId);
+          std::string descString = GetDriverVersion(desc);
+
+          RDCLOG("New D3D11 device created: %s / %s", ToStr(vendor).c_str(), descString.c_str());
+
+          SAFE_RELEASE(pDXGIAdapter);
+        }
+      }
+
+      SAFE_RELEASE(pDXGIDevice);
+    }
   }
 
   ID3D11DeviceContext *context = NULL;
@@ -398,6 +428,10 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
   static const GUID ID3D10Device_uuid = {
       0x9b7e4c0f, 0x342c, 0x4106, {0xa1, 0x9f, 0x4f, 0x27, 0x04, 0xf6, 0x89, 0xf0}};
 
+  // ID3D12Device UUID {189819f1-1db6-4b57-be54-1821339b85f7}
+  static const GUID ID3D12Device_uuid = {
+      0x189819f1, 0x1db6, 0x4b57, {0xbe, 0x54, 0x18, 0x21, 0x33, 0x9b, 0x85, 0xf7}};
+
   // ID3D11ShaderTraceFactory UUID {1fbad429-66ab-41cc-9617-667ac10e4459}
   static const GUID ID3D11ShaderTraceFactory_uuid = {
       0x1fbad429, 0x66ab, 0x41cc, {0x96, 0x17, 0x66, 0x7a, 0xc1, 0x0e, 0x44, 0x59}};
@@ -494,6 +528,12 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
     *ppvObject = NULL;
     return E_NOINTERFACE;
   }
+  else if(riid == ID3D12Device_uuid)
+  {
+    RDCWARN("Trying to get ID3D12Device - not supported.");
+    *ppvObject = NULL;
+    return E_NOINTERFACE;
+  }
   else if(riid == IDirect3DDevice9_uuid)
   {
     RDCWARN("Trying to get IDirect3DDevice9 - not supported.");
@@ -645,6 +685,29 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
     WarnUnknownGUID("ID3D11Device", riid);
   }
 
+  // nvencode requires ID3D11VideoDevice, so when it's allowed then pass these interfaces through.
+  if(!RenderDoc::Inst().IsVendorExtensionEnabled(VendorExtensions::NvAPI))
+  {
+    if(riid == __uuidof(ID3D11VideoDevice))
+    {
+      RDCWARN("Trying to get ID3D11VideoDevice - not supported.");
+      *ppvObject = NULL;
+      return E_NOINTERFACE;
+    }
+    else if(riid == __uuidof(ID3D11VideoDevice1))
+    {
+      RDCWARN("Trying to get ID3D11VideoDevice - not supported.");
+      *ppvObject = NULL;
+      return E_NOINTERFACE;
+    }
+    else if(riid == __uuidof(ID3D11VideoDevice2))
+    {
+      RDCWARN("Trying to get ID3D11VideoDevice - not supported.");
+      *ppvObject = NULL;
+      return E_NOINTERFACE;
+    }
+  }
+
   return m_RefCounter.QueryInterface(riid, ppvObject);
 }
 
@@ -1081,8 +1144,7 @@ ReplayStatus WrappedID3D11Device::ReadLogInitialisation(RDCFile *rdc, bool store
 
   if(!IsStructuredExporting(m_State))
   {
-    DrawcallDescription *previous = NULL;
-    SetupDrawcallPointers(m_Drawcalls, GetFrameRecord().drawcallList, NULL, previous);
+    SetupDrawcallPointers(m_Drawcalls, GetFrameRecord().drawcallList);
   }
 
 #if ENABLED(RDOC_DEVEL)
@@ -1307,7 +1369,6 @@ IUnknown *WrappedID3D11Device::WrapSwapchainBuffer(WrappedIDXGISwapChain4 *swap,
   {
     D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
     record->DataInSerialiser = false;
-    record->SpecialResource = true;
     record->Length = 0;
     record->NumSubResources = 0;
     record->SubResources = NULL;
@@ -1567,17 +1628,12 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
           {
             byte *data = (byte *)mapped.pData;
 
-            float aspect = float(desc.Width) / float(desc.Height);
-
             thwidth = (uint16_t)RDCMIN(maxSize, desc.Width);
             thwidth &= ~0x7;    // align down to multiple of 8
-            thheight = uint16_t(float(thwidth) / aspect);
+            thheight = uint16_t(thwidth * desc.Height / desc.Width);
 
             thpixels = new byte[3U * thwidth * thheight];
 
-            float widthf = float(desc.Width);
-            float heightf = float(desc.Height);
-
             uint32_t stride = fmt.compByteWidth * fmt.compCount;
 
             bool buf1010102 = false;
@@ -1595,11 +1651,10 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
             {
               for(uint32_t x = 0; x < thwidth; x++)
               {
-                float xf = float(x) / float(thwidth);
-                float yf = float(y) / float(thheight);
+                uint32_t xSource = x * desc.Width / thwidth;
+                uint32_t ySource = y * desc.Height / thheight;
 
-                byte *src =
-                    &data[stride * uint32_t(xf * widthf) + mapped.RowPitch * uint32_t(yf * heightf)];
+                byte *src = &data[stride * xSource + mapped.RowPitch * ySource];
 
                 if(buf1010102)
                 {
@@ -1679,8 +1734,8 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
       }
     }
 
-    RDCFile *rdc = RenderDoc::Inst().CreateRDC(
-        RDCDriver::D3D11, m_CapturedFrames.back().frameNumber, jpgbuf, len, thwidth, thheight);
+    RDCFile *rdc = RenderDoc::Inst().CreateRDC(RDCDriver::D3D11, m_CapturedFrames.back().frameNumber,
+                                               jpgbuf, len, thwidth, thheight, FileType::JPG);
 
     SAFE_DELETE_ARRAY(jpgbuf);
     SAFE_DELETE_ARRAY(thpixels);
