@@ -46,10 +46,7 @@ struct D3D12InitParams
   D3D_FEATURE_LEVEL MinimumFeatureLevel;
 
   // check if a frame capture section version is supported
-  static const uint64_t CurrentVersion = 0x5;
-
-  // 0x4 -> 0x5 - CPU_DESCRIPTOR_HANDLE serialised inline as D3D12Descriptor in appropriate
-  //              list-recording functions
+  static const uint64_t CurrentVersion = 0x6;
 
   static bool IsSupportedVersion(uint64_t ver);
 };
@@ -61,6 +58,7 @@ class WrappedID3D12Resource;
 
 class D3D12TextRenderer;
 class D3D12ShaderCache;
+class D3D12DebugManager;
 
 // give every impression of working but do nothing.
 // Just allow the user to call functions so that they don't
@@ -165,10 +163,16 @@ class WrappedID3D12Device;
 // We can pass through all calls to ID3D12DebugDevice without intercepting, this
 // struct isonly here so that we can intercept QueryInterface calls to return
 // ID3D11InfoQueue
-struct WrappedID3D12DebugDevice : public ID3D12DebugDevice
+//
+// The inheritance is awful for these classes. ID3D12DebugDevice2 inherits from ID3D12DebugDevice
+// but ID3D12DebugDevice1 is separate entirely, although its functions overlap with
+// ID3D12DebugDevice2 and ID3D12DebugDevice
+struct WrappedID3D12DebugDevice : public ID3D12DebugDevice2, public ID3D12DebugDevice1
 {
   WrappedID3D12Device *m_pDevice;
   ID3D12DebugDevice *m_pDebug;
+  ID3D12DebugDevice1 *m_pDebug1;
+  ID3D12DebugDevice2 *m_pDebug2;
 
   WrappedID3D12DebugDevice() : m_pDevice(NULL), m_pDebug(NULL) {}
   //////////////////////////////
@@ -191,7 +195,32 @@ struct WrappedID3D12DebugDevice : public ID3D12DebugDevice
 
   virtual HRESULT STDMETHODCALLTYPE ReportLiveDeviceObjects(D3D12_RLDO_FLAGS Flags)
   {
-    return m_pDebug->ReportLiveDeviceObjects(Flags);
+    if(m_pDebug)
+      return m_pDebug->ReportLiveDeviceObjects(Flags);
+    else
+      return m_pDebug1->ReportLiveDeviceObjects(Flags);
+  }
+
+  //////////////////////////////
+  // implement ID3D12DebugDevice1 / ID3D12DebugDevice2
+  virtual HRESULT STDMETHODCALLTYPE SetDebugParameter(D3D12_DEBUG_DEVICE_PARAMETER_TYPE Type,
+                                                      _In_reads_bytes_(DataSize) const void *pData,
+                                                      UINT DataSize)
+  {
+    if(m_pDebug1)
+      return m_pDebug1->SetDebugParameter(Type, pData, DataSize);
+    else
+      return m_pDebug2->SetDebugParameter(Type, pData, DataSize);
+  }
+
+  virtual HRESULT STDMETHODCALLTYPE GetDebugParameter(D3D12_DEBUG_DEVICE_PARAMETER_TYPE Type,
+                                                      _Out_writes_bytes_(DataSize) void *pData,
+                                                      UINT DataSize)
+  {
+    if(m_pDebug1)
+      return m_pDebug1->GetDebugParameter(Type, pData, DataSize);
+    else
+      return m_pDebug2->GetDebugParameter(Type, pData, DataSize);
   }
 };
 
@@ -199,7 +228,10 @@ struct WrappedID3D12DebugDevice : public ID3D12DebugDevice
 // Same idea as DummyID3D12InfoQueue above, a dummy interface so that users
 // expecting a ID3D12DebugDevice don't get confused if we have turned off the debug
 // layer and can't return the real one.
-struct DummyID3D12DebugDevice : public ID3D12DebugDevice
+//
+// The inheritance is awful for these. See WrappedID3D12DebugDevice for why there are multiple
+// parent classes
+struct DummyID3D12DebugDevice : public ID3D12DebugDevice2, public ID3D12DebugDevice1
 {
   WrappedID3D12Device *m_pDevice;
 
@@ -219,6 +251,21 @@ struct DummyID3D12DebugDevice : public ID3D12DebugDevice
   }
 
   virtual HRESULT STDMETHODCALLTYPE ReportLiveDeviceObjects(D3D12_RLDO_FLAGS Flags) { return S_OK; }
+  //////////////////////////////
+  // implement ID3D12DebugDevice1 / ID3D12DebugDevice2
+  virtual HRESULT STDMETHODCALLTYPE SetDebugParameter(D3D12_DEBUG_DEVICE_PARAMETER_TYPE Type,
+                                                      _In_reads_bytes_(DataSize) const void *pData,
+                                                      UINT DataSize)
+  {
+    return S_OK;
+  }
+
+  virtual HRESULT STDMETHODCALLTYPE GetDebugParameter(D3D12_DEBUG_DEVICE_PARAMETER_TYPE Type,
+                                                      _Out_writes_bytes_(DataSize) void *pData,
+                                                      UINT DataSize)
+  {
+    return S_OK;
+  }
 };
 
 class WrappedID3D12CommandQueue;
@@ -364,6 +411,9 @@ private:
   WrappedIDXGISwapChain4 *m_LastSwap;
 
   D3D12_FEATURE_DATA_D3D12_OPTIONS m_D3D12Opts;
+  D3D12_FEATURE_DATA_D3D12_OPTIONS1 m_D3D12Opts1;
+  D3D12_FEATURE_DATA_D3D12_OPTIONS2 m_D3D12Opts2;
+  D3D12_FEATURE_DATA_D3D12_OPTIONS3 m_D3D12Opts3;
   UINT m_DescriptorIncrements[D3D12_DESCRIPTOR_HEAP_TYPE_NUM_TYPES];
 
   template <typename SerialiserType>
@@ -385,6 +435,10 @@ public:
     return m_DescriptorIncrements[type];
   }
 
+  const D3D12_FEATURE_DATA_D3D12_OPTIONS &GetOpts() { return m_D3D12Opts; }
+  const D3D12_FEATURE_DATA_D3D12_OPTIONS1 &GetOpts1() { return m_D3D12Opts1; }
+  const D3D12_FEATURE_DATA_D3D12_OPTIONS2 &GetOpts2() { return m_D3D12Opts2; }
+  const D3D12_FEATURE_DATA_D3D12_OPTIONS3 &GetOpts3() { return m_D3D12Opts3; }
   void RemoveQueue(WrappedID3D12CommandQueue *queue);
 
   ////////////////////////////////////////////////////////////////
@@ -398,6 +452,7 @@ public:
   static std::string GetChunkName(uint32_t idx);
   D3D12ResourceManager *GetResourceManager() { return m_ResourceManager; }
   D3D12ShaderCache *GetShaderCache() { return m_ShaderCache; }
+  D3D12DebugManager *GetDebugManager() { return m_Replay.GetDebugManager(); }
   ResourceId GetResourceID() { return m_ResourceID; }
   Threading::RWLock &GetCapTransitionLock() { return m_CapTransitionLock; }
   void ReleaseSwapchainResources(IDXGISwapChain *swap, IUnknown **backbuffers, int numBackbuffers);
@@ -566,21 +621,25 @@ public:
     {
       *ppvDevice = (ID3D12Device *)this;
       this->AddRef();
+      return S_OK;
     }
     else if(riid == __uuidof(ID3D12Device1))
     {
       *ppvDevice = (ID3D12Device1 *)this;
       this->AddRef();
+      return S_OK;
     }
     else if(riid == __uuidof(ID3D12Device2))
     {
       *ppvDevice = (ID3D12Device2 *)this;
       this->AddRef();
+      return S_OK;
     }
     else if(riid == __uuidof(ID3D12Device3))
     {
       *ppvDevice = (ID3D12Device3 *)this;
       this->AddRef();
+      return S_OK;
     }
 
     return E_NOINTERFACE;
