@@ -163,7 +163,7 @@ bool WrappedID3D12Device::Serialise_CreateCommandAllocator(SerialiserType &ser,
 
       GetResourceManager()->AddLiveResource(pCommandAllocator, ret);
 
-      AddResource(pCommandAllocator, ResourceType::Pool, "Command Queue");
+      AddResource(pCommandAllocator, ResourceType::Pool, "Command Allocator");
     }
   }
 
@@ -1434,7 +1434,8 @@ bool WrappedID3D12Device::Serialise_CreatePlacedResource(
   SERIALISE_ELEMENT_LOCAL(pResource, ((WrappedID3D12Resource *)*ppvResource)->GetResourceID())
       .TypedAs("ID3D12Resource *");
 
-  SERIALISE_ELEMENT_LOCAL(gpuAddress, ((WrappedID3D12Resource *)*ppvResource)->GetGPUVirtualAddress())
+  SERIALISE_ELEMENT_LOCAL(gpuAddress,
+                          ((WrappedID3D12Resource *)*ppvResource)->GetGPUVirtualAddressIfBuffer())
       .Hidden();
 
   SERIALISE_CHECK_READ_ERRORS();
@@ -1606,9 +1607,8 @@ HRESULT WrappedID3D12Device::CreateReservedResource(const D3D12_RESOURCE_DESC *p
                                                     const D3D12_CLEAR_VALUE *pOptimizedClearValue,
                                                     REFIID riid, void **ppvResource)
 {
-  D3D12NOTIMP("Tiled Resources");
-  return m_pDevice->CreateReservedResource(pDesc, InitialState, pOptimizedClearValue, riid,
-                                           ppvResource);
+  RDCERR("Tiled Resources are not currently implemented on D3D12");
+  return E_NOINTERFACE;
 }
 
 template <typename SerialiserType>
@@ -2245,13 +2245,16 @@ HRESULT WrappedID3D12Device::SetStablePowerState(BOOL Enable)
 HRESULT WrappedID3D12Device::CheckFeatureSupport(D3D12_FEATURE Feature, void *pFeatureSupportData,
                                                  UINT FeatureSupportDataSize)
 {
-  // don't support over sm6.0
   if(Feature == D3D12_FEATURE_SHADER_MODEL)
   {
     D3D12_FEATURE_DATA_SHADER_MODEL *model = (D3D12_FEATURE_DATA_SHADER_MODEL *)pFeatureSupportData;
-    if(FeatureSupportDataSize != sizeof(D3D12_FEATURE_DATA_SHADER_MODEL) ||
-       model->HighestShaderModel >= D3D_SHADER_MODEL_6_0)
+    if(FeatureSupportDataSize != sizeof(D3D12_FEATURE_DATA_SHADER_MODEL))
       return E_INVALIDARG;
+
+    // don't support sm6.0 and over
+    model->HighestShaderModel = RDCMIN(model->HighestShaderModel, D3D_SHADER_MODEL_5_1);
+
+    return S_OK;
   }
   return m_pDevice->CheckFeatureSupport(Feature, pFeatureSupportData, FeatureSupportDataSize);
 }
