@@ -593,14 +593,14 @@ struct D3D12InitialContents
     // this is only valid during capture - it indicates we didn't create a staging texture, we're
     // going to read directly from the resource (only valid for resources that are already READBACK)
     MapDirect,
-    Multisampled,
   };
   D3D12InitialContents(D3D12Descriptor *d, uint32_t n)
       : tag(Copy),
         resourceType(Resource_DescriptorHeap),
         descriptors(d),
         numDescriptors(n),
-        resource(NULL)
+        resource(NULL),
+        srcData(NULL)
   {
   }
   D3D12InitialContents(ID3D12DescriptorHeap *r)
@@ -608,7 +608,8 @@ struct D3D12InitialContents
         resourceType(Resource_DescriptorHeap),
         descriptors(NULL),
         numDescriptors(0),
-        resource(r)
+        resource(r),
+        srcData(NULL)
   {
   }
   D3D12InitialContents(ID3D12Resource *r)
@@ -616,7 +617,8 @@ struct D3D12InitialContents
         resourceType(Resource_DescriptorHeap),
         descriptors(NULL),
         numDescriptors(0),
-        resource(r)
+        resource(r),
+        srcData(NULL)
   {
   }
   D3D12InitialContents(Tag tg)
@@ -624,17 +626,24 @@ struct D3D12InitialContents
         resourceType(Resource_DescriptorHeap),
         descriptors(NULL),
         numDescriptors(0),
-        resource(NULL)
+        resource(NULL),
+        srcData(NULL)
   {
   }
   D3D12InitialContents()
-      : tag(Copy), resourceType(Resource_Unknown), descriptors(NULL), numDescriptors(0), resource(NULL)
+      : tag(Copy),
+        resourceType(Resource_Unknown),
+        descriptors(NULL),
+        numDescriptors(0),
+        resource(NULL),
+        srcData(NULL)
   {
   }
   template <typename Configuration>
   void Free(ResourceManager<Configuration> *rm)
   {
     SAFE_RELEASE(resource);
+    FreeAlignedBuffer(srcData);
   }
 
   Tag tag;
@@ -642,6 +651,7 @@ struct D3D12InitialContents
   D3D12Descriptor *descriptors;
   uint32_t numDescriptors;
   ID3D12DeviceChild *resource;
+  byte *srcData;
 };
 
 struct D3D12ResourceManagerConfiguration
@@ -681,8 +691,9 @@ public:
   template <typename SerialiserType>
   bool Serialise_InitialState(SerialiserType &ser, ResourceId resid, ID3D12DeviceChild *res);
 
+  void SetInternalResource(ID3D12DeviceChild *res);
+
 private:
-  bool SerialisableResource(ResourceId id, D3D12ResourceRecord *record);
   ResourceId GetID(ID3D12DeviceChild *res);
 
   bool ResourceTypeRelease(ID3D12DeviceChild *res);
