@@ -43,8 +43,9 @@ enum TexDisplayFlags
 {
   eTexDisplay_None = 0,
   eTexDisplay_LinearRender = 0x1,
-  eTexDisplay_F32Render = 0x2,
-  eTexDisplay_BlendAlpha = 0x4,
+  eTexDisplay_F16Render = 0x2,
+  eTexDisplay_F32Render = 0x4,
+  eTexDisplay_BlendAlpha = 0x8,
 };
 
 class D3D12Replay : public IReplayDriver
@@ -56,11 +57,13 @@ public:
   void SetRGP(AMDRGPControl *rgp) { m_RGP = rgp; }
   void SetProxy(bool proxy) { m_Proxy = proxy; }
   bool IsRemoteProxy() { return m_Proxy; }
+  void Initialise();
   void Shutdown();
 
   void SetDevice(WrappedID3D12Device *d) { m_pDevice = d; }
   void CreateResources();
   void DestroyResources();
+  DriverInformation GetDriverInfo() { return m_DriverInfo; }
   APIProperties GetAPIProperties();
 
   ResourceDescription &GetResourceDesc(ResourceId id);
@@ -136,8 +139,13 @@ public:
   void GetTextureData(ResourceId tex, uint32_t arrayIdx, uint32_t mip,
                       const GetTextureDataParams &params, bytebuf &data);
 
-  void BuildTargetShader(string source, string entry, const ShaderCompileFlags &compileFlags,
-                         ShaderStage type, ResourceId *id, string *errors);
+  rdcarray<ShaderEncoding> GetTargetShaderEncodings()
+  {
+    return {ShaderEncoding::DXBC, ShaderEncoding::HLSL};
+  }
+  void BuildTargetShader(ShaderEncoding sourceEncoding, bytebuf source, string entry,
+                         const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId *id,
+                         string *errors);
   void ReplaceResource(ResourceId from, ResourceId to);
   void RemoveReplacement(ResourceId id);
 
@@ -201,8 +209,9 @@ private:
 
   void CreateSOBuffers();
 
-  void BuildShader(std::string source, std::string entry, const ShaderCompileFlags &compileFlags,
-                   ShaderStage type, ResourceId *id, std::string *errors);
+  void BuildShader(ShaderEncoding sourceEncoding, bytebuf source, std::string entry,
+                   const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId *id,
+                   std::string *errors);
 
   bool RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, TextureDisplay cfg,
                              TexDisplayFlags flags);
@@ -269,8 +278,6 @@ private:
 
   bool m_Proxy;
 
-  GPUVendor m_Vendor = GPUVendor::Unknown;
-
   vector<ID3D12Resource *> m_ProxyResources;
 
   struct OutputWindow
@@ -328,6 +335,7 @@ private:
     ID3D12RootSignature *RootSig = NULL;
     ID3D12PipelineState *SRGBPipe = NULL;
     ID3D12PipelineState *LinearPipe = NULL;
+    ID3D12PipelineState *F16Pipe = NULL;
     ID3D12PipelineState *F32Pipe = NULL;
     ID3D12PipelineState *BlendPipe = NULL;
   } m_TexRender;
@@ -403,6 +411,8 @@ private:
   AMDCounters *m_pAMDCounters = NULL;
   AMDRGPControl *m_RGP = NULL;
 
+  DriverInformation m_DriverInfo;
+
   D3D12AMDDrawCallback *m_pAMDDrawCallback = NULL;
 
   void FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex, vector<uint32_t> *eventIDs);
