@@ -570,6 +570,9 @@ void DoSerialise(SerialiserType &ser, D3D12_CACHED_PIPELINE_STATE &el)
   SERIALISE_MEMBER_ARRAY_EMPTY(pCachedBlob);
   uint64_t CachedBlobSizeInBytes = 0;
   ser.Serialise("CachedBlobSizeInBytes", CachedBlobSizeInBytes);
+
+  if(ser.IsReading())
+    el.CachedBlobSizeInBytes = (SIZE_T)CachedBlobSizeInBytes;
 }
 
 template <class SerialiserType>
@@ -1256,6 +1259,33 @@ void DoSerialise(SerialiserType &ser, D3D12_WRITEBUFFERIMMEDIATE_PARAMETER &el)
   SERIALISE_MEMBER(Value);
 }
 
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_DRAW_ARGUMENTS &el)
+{
+  SERIALISE_MEMBER(VertexCountPerInstance);
+  SERIALISE_MEMBER(InstanceCount);
+  SERIALISE_MEMBER(StartVertexLocation);
+  SERIALISE_MEMBER(StartInstanceLocation);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_DRAW_INDEXED_ARGUMENTS &el)
+{
+  SERIALISE_MEMBER(IndexCountPerInstance);
+  SERIALISE_MEMBER(InstanceCount);
+  SERIALISE_MEMBER(StartIndexLocation);
+  SERIALISE_MEMBER(BaseVertexLocation);
+  SERIALISE_MEMBER(StartInstanceLocation);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_DISPATCH_ARGUMENTS &el)
+{
+  SERIALISE_MEMBER(ThreadGroupCountX);
+  SERIALISE_MEMBER(ThreadGroupCountY);
+  SERIALISE_MEMBER(ThreadGroupCountZ);
+}
+
 INSTANTIATE_SERIALISE_TYPE(PortableHandle);
 INSTANTIATE_SERIALISE_TYPE(D3D12_CPU_DESCRIPTOR_HANDLE);
 INSTANTIATE_SERIALISE_TYPE(D3D12_GPU_DESCRIPTOR_HANDLE);
@@ -1301,4 +1331,7 @@ INSTANTIATE_SERIALISE_TYPE(D3D12_DEPTH_STENCIL_DESC1);
 INSTANTIATE_SERIALISE_TYPE(D3D12_VIEW_INSTANCING_DESC);
 INSTANTIATE_SERIALISE_TYPE(D3D12_SAMPLE_POSITION);
 INSTANTIATE_SERIALISE_TYPE(D3D12_SUBRESOURCE_RANGE_UINT64);
-INSTANTIATE_SERIALISE_TYPE(D3D12_WRITEBUFFERIMMEDIATE_PARAMETER);
\ No newline at end of file
+INSTANTIATE_SERIALISE_TYPE(D3D12_WRITEBUFFERIMMEDIATE_PARAMETER);
+INSTANTIATE_SERIALISE_TYPE(D3D12_DRAW_ARGUMENTS);
+INSTANTIATE_SERIALISE_TYPE(D3D12_DRAW_INDEXED_ARGUMENTS);
+INSTANTIATE_SERIALISE_TYPE(D3D12_DISPATCH_ARGUMENTS);
