@@ -26,6 +26,11 @@
 #include "common/common.h"
 #include "common/threading.h"
 #include "serialise/serialiser.h"
+#include "strings/string_utils.h"
+
+// for GetDriverVersion()
+#include <devpkey.h>
+#include <setupapi.h>
 
 UINT GetFormatBPP(DXGI_FORMAT f)
 {
@@ -1367,6 +1372,121 @@ void WarnUnknownGUID(const char *name, REFIID riid)
   }
 }
 
+static std::string GetDeviceProperty(HDEVINFO devs, PSP_DEVINFO_DATA data, const DEVPROPKEY *key)
+{
+  DEVPROPTYPE type = {};
+  DWORD bufSize = 0;
+
+  // this ALWAYS fails, we need to check if the er ror was just an insufficient buffer.
+  SetupDiGetDevicePropertyW(devs, data, key, &type, NULL, 0, &bufSize, 0);
+
+  if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
+    return "";
+
+  RDCASSERTEQUAL((uint32_t)type, DEVPROP_TYPE_STRING);
+
+  std::wstring string;
+  string.resize(bufSize);
+  BOOL success =
+      SetupDiGetDevicePropertyW(devs, data, key, &type, (PBYTE)string.data(), bufSize, &bufSize, 0);
+
+  if(!success)
+    return "";
+
+  return StringFormat::Wide2UTF8(string);
+}
+
+static uint32_t HexToInt(char hex)
+{
+  if(hex >= 'a' && hex <= 'f')
+    return (hex - 'a') + 0xa;
+  else if(hex >= 'A' && hex <= 'F')
+    return (hex - 'A') + 0xa;
+  else if(hex >= '0' && hex <= '9')
+    return hex - '0';
+
+  return 0;
+}
+
+std::string GetDriverVersion(DXGI_ADAPTER_DESC &desc)
+{
+  std::string device = StringFormat::Wide2UTF8(desc.Description);
+
+  // fixed GUID for graphics drivers, from
+  // https://msdn.microsoft.com/en-us/library/windows/hardware/ff553426%28v=vs.85%29.aspx
+  GUID display_class = {0x4d36e968, 0xe325, 0x11ce, {0xbf, 0xc1, 0x08, 0x00, 0x2b, 0xe1, 0x03, 0x18}};
+
+  HDEVINFO devs = SetupDiGetClassDevs(&display_class, NULL, NULL, DIGCF_PRESENT);
+
+  if(devs == NULL)
+  {
+    RDCERR("Couldn't enumerate graphics adapters: %d", GetLastError());
+    return device;
+  }
+
+  std::string driverVersion = "";
+
+  DWORD idx = 0;
+  SP_DEVINFO_DATA data = {};
+  data.cbSize = sizeof(data);
+  while(SetupDiEnumDeviceInfo(devs, idx, &data))
+  {
+    std::string version = GetDeviceProperty(devs, &data, &DEVPKEY_Device_DriverVersion);
+
+    if(version.empty())
+    {
+      SetupDiDestroyDeviceInfoList(devs);
+      return device;
+    }
+
+    // if we got a version, and didn't have one yet, set it
+    if(driverVersion.empty())
+      driverVersion = version;
+
+    std::string pciid = GetDeviceProperty(devs, &data, &DEVPKEY_Device_MatchingDeviceId);
+
+    if(pciid.empty())
+    {
+      SetupDiDestroyDeviceInfoList(devs);
+      return device;
+    }
+
+    pciid = strlower(pciid);
+
+    UINT VendorId = 0, DeviceId = 0;
+
+    char *end = &(*pciid.end());
+
+    const char *ven = strstr(pciid.c_str(), "ven_");
+    if(ven && ven + 8 <= end)
+    {
+      ven += 4;
+      VendorId =
+          HexToInt(ven[0]) << 12 | HexToInt(ven[1]) << 8 | HexToInt(ven[2]) << 4 | HexToInt(ven[3]);
+    }
+
+    const char *dev = strstr(pciid.c_str(), "dev_");
+    if(dev && dev + 8 <= end)
+    {
+      dev += 4;
+      DeviceId =
+          HexToInt(dev[0]) << 12 | HexToInt(dev[1]) << 8 | HexToInt(dev[2]) << 4 | HexToInt(dev[3]);
+    }
+
+    // if the PCI id matches, take it
+    if(VendorId == desc.VendorId && DeviceId == desc.DeviceId)
+      driverVersion = version;
+
+    // move to the next device
+    RDCEraseEl(data);
+    data.cbSize = sizeof(data);
+    idx++;
+  }
+
+  SetupDiDestroyDeviceInfoList(devs);
+  return device + " " + driverVersion;
+}
+
 Topology MakePrimitiveTopology(D3D_PRIMITIVE_TOPOLOGY Topo)
 {
   switch(Topo)
