@@ -85,7 +85,7 @@ bool CheckReplayContext()
 
     extensionString += StringFormat::Fmt("[%d]: %s, ", e, ext);
 
-    if(e > 0 && (e % 100) == 0)
+    if(e > 0 && (e % 25) == 0)
     {
       RDCLOG("%s", extensionString.c_str());
       extensionString = "";
@@ -442,6 +442,10 @@ void FetchEnabledExtensions()
       int mn = int(version[12] - '0');
       GLCoreVersion = RDCMAX(GLCoreVersion, mj * 10 + mn);
     }
+    else
+    {
+      IsGLES = false;
+    }
   }
 
   if(GL.glGetIntegerv)
@@ -937,9 +941,12 @@ void GLPushPopState::Push(bool modern)
 
   if(modern)
   {
-    GL.glGetIntegeri_v(eGL_UNIFORM_BUFFER_BINDING, 0, (GLint *)&ubo[0]);
-    GL.glGetIntegeri_v(eGL_UNIFORM_BUFFER_BINDING, 1, (GLint *)&ubo[1]);
-    GL.glGetIntegeri_v(eGL_UNIFORM_BUFFER_BINDING, 2, (GLint *)&ubo[2]);
+    // the non-indexed bind is separate from the indexed binds
+    GL.glGetIntegerv(eGL_UNIFORM_BUFFER_BINDING, (GLint *)&ubo);
+
+    GL.glGetIntegeri_v(eGL_UNIFORM_BUFFER_BINDING, 0, (GLint *)&idxubo[0]);
+    GL.glGetIntegeri_v(eGL_UNIFORM_BUFFER_BINDING, 1, (GLint *)&idxubo[1]);
+    GL.glGetIntegeri_v(eGL_UNIFORM_BUFFER_BINDING, 2, (GLint *)&idxubo[2]);
 
     GL.glGetIntegerv(eGL_VERTEX_ARRAY_BINDING, (GLint *)&VAO);
   }
@@ -1076,9 +1083,11 @@ void GLPushPopState::Pop(bool modern)
 
   if(modern)
   {
-    GL.glBindBufferBase(eGL_UNIFORM_BUFFER, 0, ubo[0]);
-    GL.glBindBufferBase(eGL_UNIFORM_BUFFER, 1, ubo[1]);
-    GL.glBindBufferBase(eGL_UNIFORM_BUFFER, 2, ubo[2]);
+    GL.glBindBufferBase(eGL_UNIFORM_BUFFER, 0, idxubo[0]);
+    GL.glBindBufferBase(eGL_UNIFORM_BUFFER, 1, idxubo[1]);
+    GL.glBindBufferBase(eGL_UNIFORM_BUFFER, 2, idxubo[2]);
+
+    GL.glBindBuffer(eGL_UNIFORM_BUFFER, ubo);
 
     GL.glUseProgram(prog);
 
@@ -1125,6 +1134,10 @@ bool GLInitParams::IsSupportedVersion(uint64_t ver)
   if(ver == 0x1C)
     return true;
 
+  // 0x1D -> 0x1E - added new chunk for context parameters and per-context tracking of backbuffers
+  if(ver == 0x1D)
+    return true;
+
   return false;
 }
 
@@ -1144,6 +1157,29 @@ void DoSerialise(SerialiserType &ser, GLInitParams &el)
 
 INSTANTIATE_SERIALISE_TYPE(GLInitParams);
 
+template <typename SerialiserType>
+void DoSerialise(SerialiserType &ser, DrawElementsIndirectCommand &el)
+{
+  SERIALISE_MEMBER(count);
+  SERIALISE_MEMBER(instanceCount);
+  SERIALISE_MEMBER(firstIndex);
+  SERIALISE_MEMBER(baseVertex);
+  SERIALISE_MEMBER(baseInstance);
+}
+
+INSTANTIATE_SERIALISE_TYPE(DrawElementsIndirectCommand);
+
+template <typename SerialiserType>
+void DoSerialise(SerialiserType &ser, DrawArraysIndirectCommand &el)
+{
+  SERIALISE_MEMBER(count);
+  SERIALISE_MEMBER(instanceCount);
+  SERIALISE_MEMBER(first);
+  SERIALISE_MEMBER(baseInstance);
+}
+
+INSTANTIATE_SERIALISE_TYPE(DrawArraysIndirectCommand);
+
 size_t BufferIdx(GLenum buf)
 {
   switch(buf)
@@ -1967,11 +2003,23 @@ ResourceFormat MakeResourceFormat(GLenum target, GLenum fmt)
         ret.compByteWidth = 4;
         ret.compCount = 1;
         break;
-      case eGL_DEPTH24_STENCIL8: ret.type = ResourceFormatType::D24S8; break;
+      case eGL_DEPTH24_STENCIL8:
+        ret.compByteWidth = 0;
+        ret.compCount = 2;
+        ret.type = ResourceFormatType::D24S8;
+        break;
       case eGL_DEPTH_STENCIL:
-      case eGL_DEPTH32F_STENCIL8: ret.type = ResourceFormatType::D32S8; break;
+      case eGL_DEPTH32F_STENCIL8:
+        ret.compByteWidth = 0;
+        ret.compCount = 2;
+        ret.type = ResourceFormatType::D32S8;
+        break;
       case eGL_STENCIL_INDEX:
-      case eGL_STENCIL_INDEX8: ret.type = ResourceFormatType::S8; break;
+      case eGL_STENCIL_INDEX8:
+        ret.compByteWidth = 1;
+        ret.compCount = 1;
+        ret.type = ResourceFormatType::S8;
+        break;
       default: RDCERR("Unexpected depth or stencil format '%s'", ToStr(fmt).c_str());
     }
   }
