@@ -306,6 +306,32 @@ GLPlatform &GetEGLPlatform();
 
 #endif
 
+// on macOS we used compile time interposing to hook
+#if ENABLED(RDOC_APPLE)
+
+// never declare the actual raw function name as an export, declare the functions with a suffix that
+// will be connected in the struct below
+#define GL_EXPORT_NAME(function) CONCAT(interposed_, function)
+
+// from dyld-interposing.h - DYLD_INTERPOSE
+#define DECL_GL_HOOK_EXPORT(function)                                                                 \
+  __attribute__((used)) static struct                                                                 \
+  {                                                                                                   \
+    const void *replacment;                                                                           \
+    const void *replacee;                                                                             \
+  } _interpose_def_##function __attribute__((section("__DATA,__interpose"))) = {                      \
+      (const void *)(unsigned long)&GL_EXPORT_NAME(function), (const void *)(unsigned long)&function, \
+  };
+
+#else
+
+// on all other platforms we just export functions with the bare name, and don't declare anything to
+// hook.
+#define GL_EXPORT_NAME(function) function
+#define DECL_GL_HOOK_EXPORT(function)
+
+#endif
+
 class RDCFile;
 class IReplayDriver;
 
@@ -406,7 +432,8 @@ struct GLPushPopState
   GLenum ActiveTexture;
   GLuint tex0;
   GLuint arraybuf;
-  GLuint ubo[3];
+  GLuint idxubo[3];
+  GLuint ubo;
   GLuint prog;
   GLuint pipe;
   GLuint VAO;
@@ -616,6 +643,7 @@ extern bool IsGLES;
   EXT_TO_CHECK(32, 32, ARB_geometry_shader4)                     \
   EXT_TO_CHECK(41, 31, ARB_separate_shader_objects)              \
   EXT_TO_CHECK(41, 99, ARB_viewport_array)                       \
+  EXT_TO_CHECK(41, 99, ARB_ES2_compatibility)                    \
   EXT_TO_CHECK(42, 99, ARB_base_instance)                        \
   EXT_TO_CHECK(42, 31, ARB_shader_atomic_counters)               \
   EXT_TO_CHECK(42, 31, ARB_shader_image_load_store)              \
@@ -629,13 +657,16 @@ extern bool IsGLES;
   EXT_TO_CHECK(43, 31, ARB_program_interface_query)              \
   EXT_TO_CHECK(43, 31, ARB_shader_storage_buffer_object)         \
   EXT_TO_CHECK(43, 31, ARB_stencil_texturing)                    \
-  EXT_TO_CHECK(43, 32, ARB_texture_storage_multisample)          \
+  /* See above with ARB_texture_multisample_no_array          */ \
+  EXT_TO_CHECK(43, 32, ARB_texture_storage_multisample_no_array) \
+  EXT_TO_CHECK(43, 99, ARB_texture_storage_multisample)          \
   EXT_TO_CHECK(43, 99, ARB_texture_view)                         \
   EXT_TO_CHECK(43, 31, ARB_vertex_attrib_binding)                \
   EXT_TO_CHECK(43, 32, KHR_debug)                                \
   EXT_TO_CHECK(44, 99, ARB_enhanced_layouts)                     \
   EXT_TO_CHECK(44, 99, ARB_query_buffer_object)                  \
   EXT_TO_CHECK(45, 99, ARB_clip_control)                         \
+  EXT_TO_CHECK(45, 99, ARB_direct_state_access)                  \
   EXT_TO_CHECK(46, 99, ARB_polygon_offset_clamp)                 \
   EXT_TO_CHECK(46, 99, ARB_texture_filter_anisotropic)           \
   EXT_TO_CHECK(46, 99, ARB_pipeline_statistics_query)            \
@@ -757,6 +788,8 @@ struct DrawElementsIndirectCommand
   uint32_t baseInstance;
 };
 
+DECLARE_REFLECTION_STRUCT(DrawElementsIndirectCommand);
+
 struct DrawArraysIndirectCommand
 {
   uint32_t count;
@@ -765,6 +798,8 @@ struct DrawArraysIndirectCommand
   uint32_t baseInstance;
 };
 
+DECLARE_REFLECTION_STRUCT(DrawArraysIndirectCommand);
+
 enum class GLChunk : uint32_t
 {
   DeviceInitialisation = (uint32_t)SystemChunk::FirstDriverChunk,
@@ -1973,6 +2008,65 @@ enum class GLChunk : uint32_t
   glSpecializeShader,
   glSpecializeShaderARB,
 
+  glUniform1fARB,
+  glUniform1iARB,
+  glUniform2fARB,
+  glUniform2iARB,
+  glUniform3fARB,
+  glUniform3iARB,
+  glUniform4fARB,
+  glUniform4iARB,
+  glUniform1fvARB,
+  glUniform1ivARB,
+  glUniform2fvARB,
+  glUniform2ivARB,
+  glUniform3fvARB,
+  glUniform3ivARB,
+  glUniform4fvARB,
+  glUniform4ivARB,
+  glUniformMatrix2fvARB,
+  glUniformMatrix3fvARB,
+  glUniformMatrix4fvARB,
+
+  glGetUnsignedBytevEXT,
+  glGetUnsignedBytei_vEXT,
+  glDeleteMemoryObjectsEXT,
+  glIsMemoryObjectEXT,
+  glCreateMemoryObjectsEXT,
+  glMemoryObjectParameterivEXT,
+  glGetMemoryObjectParameterivEXT,
+  glTexStorageMem2DEXT,
+  glTexStorageMem2DMultisampleEXT,
+  glTexStorageMem3DEXT,
+  glTexStorageMem3DMultisampleEXT,
+  glBufferStorageMemEXT,
+  glTextureStorageMem2DEXT,
+  glTextureStorageMem2DMultisampleEXT,
+  glTextureStorageMem3DEXT,
+  glTextureStorageMem3DMultisampleEXT,
+  glNamedBufferStorageMemEXT,
+  glTexStorageMem1DEXT,
+  glTextureStorageMem1DEXT,
+  glGenSemaphoresEXT,
+  glDeleteSemaphoresEXT,
+  glIsSemaphoreEXT,
+  glSemaphoreParameterui64vEXT,
+  glGetSemaphoreParameterui64vEXT,
+  glWaitSemaphoreEXT,
+  glSignalSemaphoreEXT,
+  glImportMemoryFdEXT,
+  glImportSemaphoreFdEXT,
+  glImportMemoryWin32HandleEXT,
+  glImportMemoryWin32NameEXT,
+  glImportSemaphoreWin32HandleEXT,
+  glImportSemaphoreWin32NameEXT,
+  glAcquireKeyedMutexWin32EXT,
+  glReleaseKeyedMutexWin32EXT,
+
+  ContextConfiguration,
+
+  glTextureFoveationParametersQCOM,
+
   Max,
 };
 
