@@ -244,6 +244,9 @@ void GLReplay::InitDebugData()
   {
     glslVersion = glslBaseVer = glslCSVer = 310;
     shaderType = eShaderGLSLES;
+
+    if(GLCoreVersion >= 32)
+      glslVersion = glslBaseVer = glslCSVer = 320;
   }
   else
   {
@@ -252,10 +255,6 @@ void GLReplay::InitDebugData()
     shaderType = eShaderGLSL;
   }
 
-  // TODO In case of GLES some currently unused shaders, which are guarded by HasExt[..] checks,
-  // still contain compile errors (e.g. array2ms.comp, ms2array.comp, quad*, etc.).
-  bool glesShadersAreComplete = !IsGLES;
-
   GenerateGLSLShader(vs, shaderType, "", GetEmbeddedResource(glsl_blit_vert), glslBaseVer);
 
   // used to combine with custom shaders.
@@ -300,8 +299,20 @@ void GLReplay::InitDebugData()
   GenerateGLSLShader(vs, shaderType, "", GetEmbeddedResource(glsl_blit_vert), glslBaseVer);
 
   DebugData.fixedcolFragShader = DebugData.quadoverdrawFragShader = 0;
+  DebugData.quadoverdrawResolveProg = 0;
 
-  if(glesShadersAreComplete && HasExt[ARB_shader_image_load_store] && HasExt[ARB_gpu_shader5])
+  if(IsGLES)
+  {
+    // quad overdraw not supported on GLES.
+    // 1.
+    //   dFdx doesn't support uints - potentially workaroundable with float casts, but highly
+    //   doubtful GLES compilers will do that properly without exploding.
+    // 2.
+    //   quad overdraw write shader must be linked with user shaders in program, which requires
+    //   matching ESSL version and features required for it aren't exposed as extensions to older
+    //   versions but only in core versions.
+  }
+  else if(HasExt[ARB_shader_image_load_store] && HasExt[ARB_gpu_shader5])
   {
     GenerateGLSLShader(fs, shaderType, "", GetEmbeddedResource(glsl_quadresolve_frag), glslBaseVer);
 
@@ -316,7 +327,6 @@ void GLReplay::InitDebugData()
                                MessageSource::RuntimeWarning,
                                "GL_ARB_shader_image_load_store/GL_ARB_gpu_shader5 not supported, "
                                "disabling quad overdraw feature.");
-    DebugData.quadoverdrawResolveProg = 0;
   }
 
   GenerateGLSLShader(fs, shaderType, "", GetEmbeddedResource(glsl_checkerboard_frag), glslBaseVer);
@@ -381,7 +391,6 @@ void GLReplay::InitDebugData()
     RDCCOMPILE_ASSERT(sizeof(TexDisplayUBOData) <= 2048, "UBO too small");
     RDCCOMPILE_ASSERT(sizeof(FontUBOData) <= 2048, "UBO too small");
     RDCCOMPILE_ASSERT(sizeof(HistogramUBOData) <= 2048, "UBO too small");
-    RDCCOMPILE_ASSERT(sizeof(overdrawRamp) <= 2048, "UBO too small");
   }
 
   DebugData.overlayTexWidth = DebugData.overlayTexHeight = DebugData.overlayTexSamples = 0;
@@ -424,67 +433,76 @@ void GLReplay::InitDebugData()
         ARRAY_COUNT(DebugData.minmaxTileProgram) >= (TEXDISPLAY_SINT_TEX | TEXDISPLAY_TYPEMASK) + 1,
         "not enough programs");
 
-    string extensions =
-        "#extension GL_ARB_compute_shader : require\n"
-        "#extension GL_ARB_shader_storage_buffer_object : require\n";
+    string extensions;
 
-    for(int t = 1; glesShadersAreComplete && HasExt[ARB_compute_shader] && t <= RESTYPE_TEXTYPEMAX;
-        t++)
+    if(!IsGLES)
     {
-      // float, uint, sint
-      for(int i = 0; i < 3; i++)
+      extensions =
+          "#extension GL_ARB_compute_shader : require\n"
+          "#extension GL_ARB_shader_storage_buffer_object : require\n";
+    }
+
+    if(HasExt[ARB_compute_shader] && HasExt[ARB_shader_storage_buffer_object])
+    {
+      for(int t = 1; t <= RESTYPE_TEXTYPEMAX; t++)
       {
-        int idx = t;
-        if(i == 1)
-          idx |= TEXDISPLAY_UINT_TEX;
-        if(i == 2)
-          idx |= TEXDISPLAY_SINT_TEX;
-
+        // float, uint, sint
+        for(int i = 0; i < 3; i++)
         {
-          string defines = extensions;
-          defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
-          defines += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
-          defines += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
+          int idx = t;
+          if(i == 1)
+            idx |= TEXDISPLAY_UINT_TEX;
+          if(i == 2)
+            idx |= TEXDISPLAY_SINT_TEX;
 
-          GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_minmaxtile_comp),
-                             glslCSVer);
+          {
+            string defines = extensions;
+            defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
+            defines += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
+            defines += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
 
-          DebugData.minmaxTileProgram[idx] = CreateCShaderProgram(cs);
-        }
+            GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_minmaxtile_comp),
+                               glslCSVer);
 
-        {
-          string defines = extensions;
-          defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
-          defines += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
-          defines += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
+            DebugData.minmaxTileProgram[idx] = CreateCShaderProgram(cs);
+          }
 
-          GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_histogram_comp),
-                             glslCSVer);
+          {
+            string defines = extensions;
+            defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
+            defines += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
+            defines += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
 
-          DebugData.histogramProgram[idx] = CreateCShaderProgram(cs);
-        }
+            GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_histogram_comp),
+                               glslCSVer);
 
-        if(t == 1)
-        {
-          string defines = extensions;
-          defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
-          defines += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
-          defines += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
+            DebugData.histogramProgram[idx] = CreateCShaderProgram(cs);
+          }
 
-          GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_minmaxresult_comp),
-                             glslCSVer);
+          if(t == 1)
+          {
+            string defines = extensions;
+            defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
+            defines += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
+            defines += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
 
-          DebugData.minmaxResultProgram[i] = CreateCShaderProgram(cs);
+            GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_minmaxresult_comp),
+                               glslCSVer);
+
+            DebugData.minmaxResultProgram[i] = CreateCShaderProgram(cs);
+          }
         }
       }
     }
-
-    if(!HasExt[ARB_compute_shader])
+    else
     {
-      RDCWARN("GL_ARB_compute_shader not supported, disabling min/max and histogram features.");
-      m_pDriver->AddDebugMessage(
-          MessageCategory::Portability, MessageSeverity::Medium, MessageSource::RuntimeWarning,
-          "GL_ARB_compute_shader not supported, disabling min/max and histogram features.");
+      RDCWARN(
+          "GL_ARB_compute_shader or ARB_shader_storage_buffer_object not supported, disabling "
+          "min/max and histogram features.");
+      m_pDriver->AddDebugMessage(MessageCategory::Portability, MessageSeverity::Medium,
+                                 MessageSource::RuntimeWarning,
+                                 "GL_ARB_compute_shader or ARB_shader_storage_buffer_object not "
+                                 "supported, disabling min/max and histogram features.");
     }
 
     drv.glGenBuffers(1, &DebugData.minmaxTileResult);
@@ -500,33 +518,68 @@ void GLReplay::InitDebugData()
 
     drv.glNamedBufferDataEXT(DebugData.minmaxTileResult, byteSize, NULL, eGL_DYNAMIC_DRAW);
     drv.glNamedBufferDataEXT(DebugData.minmaxResult, sizeof(Vec4f) * 2, NULL, eGL_DYNAMIC_READ);
-    drv.glNamedBufferDataEXT(DebugData.histogramBuf, sizeof(uint32_t) * 4 * HGRAM_NUM_BUCKETS, NULL,
+    drv.glNamedBufferDataEXT(DebugData.histogramBuf, sizeof(uint32_t) * HGRAM_NUM_BUCKETS, NULL,
                              eGL_DYNAMIC_READ);
   }
 
-  if(glesShadersAreComplete && HasExt[ARB_compute_shader])
+  if(HasExt[ARB_compute_shader] && HasExt[ARB_shader_image_load_store])
   {
     GenerateGLSLShader(cs, shaderType, "", GetEmbeddedResource(glsl_ms2array_comp), glslCSVer);
     DebugData.MS2Array = CreateCShaderProgram(cs);
 
-    GenerateGLSLShader(cs, shaderType, "", GetEmbeddedResource(glsl_array2ms_comp), glslCSVer);
-    DebugData.Array2MS = CreateCShaderProgram(cs);
+    // GLES doesn't have multisampled image load/store even with any extension
+    DebugData.Array2MS = 0;
+    if(!IsGLES)
+    {
+      GenerateGLSLShader(cs, shaderType, "", GetEmbeddedResource(glsl_array2ms_comp), glslCSVer);
+      DebugData.Array2MS = CreateCShaderProgram(cs);
+    }
   }
   else
   {
     DebugData.MS2Array = 0;
     DebugData.Array2MS = 0;
-    RDCWARN("GL_ARB_compute_shader not supported, disabling 2DMS save/load.");
+    RDCWARN(
+        "GL_ARB_compute_shader or ARB_shader_image_load_store not supported, disabling 2DMS "
+        "save/load.");
     m_pDriver->AddDebugMessage(MessageCategory::Portability, MessageSeverity::Medium,
                                MessageSource::RuntimeWarning,
-                               "GL_ARB_compute_shader not supported, disabling 2DMS save/load.");
+                               "GL_ARB_compute_shader or ARB_shader_image_load_store not "
+                               "supported, disabling 2DMS save/load.");
   }
 
-  if(glesShadersAreComplete && HasExt[ARB_compute_shader])
+  DebugData.DepthArray2MS = DebugData.DepthMS2Array = 0;
+
+  if(HasExt[ARB_texture_multisample])
   {
-    string defines =
-        "#extension GL_ARB_compute_shader : require\n"
-        "#extension GL_ARB_shader_storage_buffer_object : require";
+    GenerateGLSLShader(vs, shaderType, "", GetEmbeddedResource(glsl_blit_vert), glslBaseVer);
+
+    GenerateGLSLShader(fs, shaderType, "", GetEmbeddedResource(glsl_depthms2arr_frag), glslBaseVer);
+    DebugData.DepthMS2Array = CreateShaderProgram(vs, fs);
+
+    GenerateGLSLShader(fs, shaderType, "", GetEmbeddedResource(glsl_deptharr2ms_frag), glslBaseVer);
+    DebugData.DepthArray2MS = CreateShaderProgram(vs, fs);
+  }
+  else
+  {
+    DebugData.MS2Array = 0;
+    DebugData.Array2MS = 0;
+    RDCWARN("GL_ARB_texture_multisample not supported, disabling 2DMS depth-stencil save/load.");
+    m_pDriver->AddDebugMessage(
+        MessageCategory::Portability, MessageSeverity::Medium, MessageSource::RuntimeWarning,
+        "GL_ARB_texture_multisample not supported, disabling 2DMS depth-stencil save/load.");
+  }
+
+  if(HasExt[ARB_compute_shader])
+  {
+    string defines;
+
+    if(!IsGLES)
+    {
+      defines =
+          "#extension GL_ARB_compute_shader : require\n"
+          "#extension GL_ARB_shader_storage_buffer_object : require";
+    }
     GenerateGLSLShader(cs, shaderType, defines, GetEmbeddedResource(glsl_mesh_comp), glslCSVer);
     DebugData.meshPickProgram = CreateCShaderProgram(cs);
   }
@@ -619,8 +672,11 @@ void GLReplay::InitDebugData()
 
   // try to identify the GPU we're running on.
   {
+    RDCEraseEl(m_DriverInfo);
+
     const char *vendor = (const char *)drv.glGetString(eGL_VENDOR);
     const char *renderer = (const char *)drv.glGetString(eGL_RENDERER);
+    const char *version = (const char *)drv.glGetString(eGL_VERSION);
 
     // we're just doing substring searches, so combine both for ease.
     std::string combined = (vendor ? vendor : "");
@@ -665,24 +721,34 @@ void GLReplay::InitDebugData()
     {
       if(combined.find(p.search) != std::string::npos)
       {
-        if(m_Vendor == GPUVendor::Unknown)
+        if(m_DriverInfo.vendor == GPUVendor::Unknown)
         {
-          m_Vendor = p.vendor;
+          m_DriverInfo.vendor = p.vendor;
         }
         else
         {
           // either we already found this with another pattern, or we've identified two patterns and
           // it's ambiguous. Keep the first one we found, arbitrarily, but print a warning.
-          if(m_Vendor != p.vendor)
+          if(m_DriverInfo.vendor != p.vendor)
           {
             RDCWARN("Already identified '%s' as %s, but now identified as %s", combined.c_str(),
-                    ToStr(m_Vendor).c_str(), ToStr(p.vendor).c_str());
+                    ToStr(m_DriverInfo.vendor).c_str(), ToStr(p.vendor).c_str());
           }
         }
       }
     }
 
-    RDCDEBUG("Identified GPU vendor '%s'", ToStr(m_Vendor).c_str());
+    RDCDEBUG("Identified GPU vendor '%s'", ToStr(m_DriverInfo.vendor).c_str());
+
+    std::string versionString = version;
+
+    versionString += " / ";
+    versionString += renderer;
+    versionString += " / ";
+    versionString += vendor;
+
+    versionString.resize(RDCMIN(versionString.size(), ARRAY_COUNT(m_DriverInfo.version) - 1));
+    memcpy(m_DriverInfo.version, versionString.c_str(), versionString.size());
   }
 
   // these below need to be made on the replay context, as they are context-specific (not shared)
@@ -806,6 +872,9 @@ void GLReplay::DeleteDebugData()
   drv.glDeleteProgram(DebugData.Array2MS);
   drv.glDeleteProgram(DebugData.MS2Array);
 
+  drv.glDeleteProgram(DebugData.DepthArray2MS);
+  drv.glDeleteProgram(DebugData.DepthMS2Array);
+
   drv.glDeleteBuffers(1, &DebugData.minmaxTileResult);
   drv.glDeleteBuffers(1, &DebugData.minmaxResult);
   drv.glDeleteBuffers(1, &DebugData.histogramBuf);
@@ -1152,17 +1221,10 @@ bool GLReplay::GetHistogram(ResourceId texid, uint32_t sliceFace, uint32_t mip,
   GL.glMemoryBarrier(GL_SHADER_STORAGE_BARRIER_BIT);
 
   histogram.clear();
-  histogram.resize(HGRAM_NUM_BUCKETS * 4);
+  histogram.resize(HGRAM_NUM_BUCKETS);
 
   GL.glBindBuffer(eGL_COPY_READ_BUFFER, DebugData.histogramBuf);
-  GL.glGetBufferSubData(eGL_COPY_READ_BUFFER, 0, sizeof(uint32_t) * 4 * HGRAM_NUM_BUCKETS,
-                        &histogram[0]);
-
-  // compress down from uvec4, then resize down
-  for(size_t i = 1; i < HGRAM_NUM_BUCKETS; i++)
-    histogram[i] = histogram[i * 4];
-
-  histogram.resize(HGRAM_NUM_BUCKETS);
+  GL.glGetBufferSubData(eGL_COPY_READ_BUFFER, 0, sizeof(uint32_t) * HGRAM_NUM_BUCKETS, &histogram[0]);
 
   if(maxlevel >= 0)
     GL.glTextureParameterivEXT(texname, target, eGL_TEXTURE_MAX_LEVEL, (GLint *)&maxlevel);
@@ -1649,6 +1711,7 @@ void GLReplay::PickPixel(ResourceId texture, uint32_t x, uint32_t y, uint32_t sl
     // at the same time.
     if(texDetails.internalFormat == eGL_DEPTH24_STENCIL8 ||
        texDetails.internalFormat == eGL_DEPTH32F_STENCIL8 ||
+       texDetails.internalFormat == eGL_DEPTH_STENCIL ||
        texDetails.internalFormat == eGL_STENCIL_INDEX8)
     {
       texDisplay.red = texDisplay.blue = texDisplay.alpha = false;
