@@ -58,6 +58,7 @@ public:
 
   void *handle = NULL;
   WrappedOpenGL *driver = NULL;
+  bool enabled = false;
 } glhook;
 
 #if ENABLED(RDOC_DEVEL)
@@ -98,14 +99,36 @@ int ScopedPrinter::depth = 0;
 
 #endif
 
-DefineSupportedHooks();
-DefineUnsupportedHooks();
-
 void SetDriverForHooks(WrappedOpenGL *driver)
 {
   glhook.driver = driver;
 }
 
+#if ENABLED(RDOC_WIN32)
+void EnableGLHooks()
+{
+  glhook.enabled = true;
+}
+
+// if we were injected and aren't ready to capture, skip out and call the real function
+#define UNINIT_CALL(function, ...)                                                      \
+  if(!glhook.enabled)                                                                   \
+  {                                                                                     \
+    if(GL.function == NULL)                                                             \
+      RDCERR("No function pointer for '%s' while uninitialised!", STRINGIZE(function)); \
+    return GL.function(__VA_ARGS__);                                                    \
+  }
+
+#else
+
+// nothing to do - we always assume we are ready to capture
+#define UNINIT_CALL(function, ...)
+
+#endif
+
+DefineSupportedHooks();
+DefineUnsupportedHooks();
+
 void *HookedGetProcAddress(const char *func, void *realFunc)
 {
 #define CheckFunction(function, name)                    \
@@ -174,8 +197,10 @@ void GLHook::RegisterHooks()
   const char *libraryName = "opengl32.dll";
 #elif ENABLED(RDOC_ANDROID)
   const char *libraryName = "libEGL.so";
+#elif ENABLED(RDOC_APPLE)
+  const char *libraryName = "/System/Library/Frameworks/OpenGL.framework/Versions/Current/OpenGL";
 #else
-  const char *libraryName = "libGL.so";
+  const char *libraryName = "libGL.so.1";
 #endif
 
   LibraryHooks::RegisterLibraryHook(libraryName, &GLHooked);
@@ -198,3 +223,7 @@ void GLHook::RegisterHooks()
   }
 #endif
 }
+
+#if ENABLED(RDOC_APPLE)
+#include "apple_gl_hook_defs.h"
+#endif
\ No newline at end of file
