@@ -248,8 +248,16 @@ void GLResourceManager::ContextPrepare_InitialState(GLResource res)
       }
     }
 
-    for(int i = 0; i < (int)ARRAY_COUNT(data.DrawBuffers); i++)
-      GL.glGetIntegerv(GLenum(eGL_DRAW_BUFFER0 + i), (GLint *)&data.DrawBuffers[i]);
+    GLuint maxDraws = 0;
+    GL.glGetIntegerv(eGL_MAX_DRAW_BUFFERS, (GLint *)&maxDraws);
+
+    for(GLuint i = 0; i < (GLuint)ARRAY_COUNT(data.DrawBuffers); i++)
+    {
+      if(i < maxDraws)
+        GL.glGetIntegerv(GLenum(eGL_DRAW_BUFFER0 + i), (GLint *)&data.DrawBuffers[i]);
+      else
+        data.DrawBuffers[i] = eGL_COLOR_ATTACHMENT0;
+    }
 
     GL.glGetIntegerv(eGL_READ_BUFFER, (GLint *)&data.ReadBuffer);
 
@@ -321,6 +329,13 @@ void GLResourceManager::ContextPrepare_InitialState(GLResource res)
 
     GL.glBindVertexArray(res.name);
 
+    for(GLuint i = 0; i < 16; i++)
+    {
+      GLuint buffer = GetBoundVertexBuffer(i);
+
+      data.VertexBuffers[i].Buffer = BufferRes(ctx, buffer);
+    }
+
     for(GLuint i = 0; i < 16; i++)
     {
       GL.glGetVertexAttribiv(i, eGL_VERTEX_ATTRIB_ARRAY_ENABLED,
@@ -338,10 +353,6 @@ void GLResourceManager::ContextPrepare_InitialState(GLResource res)
 
       GL.glGetVertexAttribiv(i, eGL_VERTEX_ATTRIB_ARRAY_SIZE, (GLint *)&data.VertexAttribs[i].size);
 
-      GLuint buffer = GetBoundVertexBuffer(i);
-
-      data.VertexBuffers[i].Buffer = BufferRes(ctx, buffer);
-
       if(HasExt[ARB_vertex_attrib_binding])
       {
         GL.glGetIntegeri_v(eGL_VERTEX_BINDING_STRIDE, i, (GLint *)&data.VertexBuffers[i].Stride);
@@ -358,6 +369,7 @@ void GLResourceManager::ContextPrepare_InitialState(GLResource res)
                                (GLint *)&data.VertexBuffers[i].Stride);
         GL.glGetVertexAttribiv(i, eGL_VERTEX_ATTRIB_ARRAY_DIVISOR,
                                (GLint *)&data.VertexBuffers[i].Divisor);
+        data.VertexAttribs[i].vbslot = i;
         data.VertexBuffers[i].Offset = 0;
 
         void *ptr = NULL;
@@ -365,6 +377,16 @@ void GLResourceManager::ContextPrepare_InitialState(GLResource res)
 
         data.VertexAttribs[i].offset = (uint32_t)(uintptr_t)ptr;
       }
+
+      // if no buffer is bound, replace any non-zero offset with a marker value. This makes captures
+      // more deterministic and ensures that if we ever try to use the invalid offset/pointer then
+      // we crash with a known value.
+      if(data.VertexBuffers[data.VertexAttribs[i].vbslot].Buffer.name == 0 &&
+         data.VertexAttribs[i].offset > 0)
+        data.VertexAttribs[i].offset = 0xDEADBEEF;
+
+      if(data.VertexBuffers[i].Buffer.name == 0 && data.VertexBuffers[i].Offset > 0)
+        data.VertexBuffers[i].Offset = 0xDEADBEEF;
     }
 
     GLuint buffer = 0;
@@ -552,7 +574,7 @@ void GLResourceManager::CreateTextureImage(GLuint tex, GLenum internalFormat,
     {
       baseFormat = GetBaseFormat(internalFormat);
 
-      if(dataType == eGL_NONE)
+      if(internalFormatHint == eGL_NONE)
         dataType = GetDataType(internalFormat);
     }
 
@@ -1548,7 +1570,6 @@ bool GLResourceManager::Serialise_InitialState(SerialiserType &ser, ResourceId r
                 else
                 {
                   // we avoid glGetTextureImageEXT as it seems buggy for cubemap faces
-                  RDCLOG("Getting tex image for %llu", resid);
                   GL.glGetTexImage(targets[trg], i, fmt, type, scratchBuf);
                 }
               }
