@@ -92,6 +92,7 @@ public:
   void Shutdown();
 
   void SetDriver(WrappedOpenGL *d) { m_pDriver = d; }
+  DriverInformation GetDriverInfo() { return m_DriverInfo; }
   APIProperties GetAPIProperties();
 
   ResourceDescription &GetResourceDesc(ResourceId id);
@@ -178,8 +179,10 @@ public:
 
   void RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryDraws, const MeshDisplay &cfg);
 
-  void BuildTargetShader(string source, string entry, const ShaderCompileFlags &compileFlags,
-                         ShaderStage type, ResourceId *id, string *errors);
+  rdcarray<ShaderEncoding> GetTargetShaderEncodings() { return {ShaderEncoding::GLSL}; }
+  void BuildTargetShader(ShaderEncoding sourceEncoding, bytebuf source, string entry,
+                         const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId *id,
+                         string *errors);
   void BuildCustomShader(string source, string entry, const ShaderCompileFlags &compileFlags,
                          ShaderStage type, ResourceId *id, string *errors);
   void FreeCustomShader(ResourceId id);
@@ -245,9 +248,14 @@ private:
   void CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint fragShader);
 
   void CopyArrayToTex2DMS(GLuint destMS, GLuint srcArray, GLint width, GLint height,
-                          GLint arraySize, GLint samples, GLenum intFormat);
+                          GLint arraySize, GLint samples, GLenum intFormat, uint32_t selectedSlice);
   void CopyTex2DMSToArray(GLuint &destArray, GLuint srcMS, GLint width, GLint height,
                           GLint arraySize, GLint samples, GLenum intFormat);
+  void CopyDepthArrayToTex2DMS(GLuint destMS, GLuint srcArray, GLint width, GLint height,
+                               GLint arraySize, GLint samples, GLenum intFormat,
+                               uint32_t selectedSlice);
+  void CopyDepthTex2DMSToArray(GLuint &destArray, GLuint srcMS, GLint width, GLint height,
+                               GLint arraySize, GLint samples, GLenum intFormat);
 
   struct OutputWindow : public GLWindowingData
   {
@@ -317,6 +325,7 @@ private:
     GLuint pickResultBuf;
 
     GLuint MS2Array, Array2MS;
+    GLuint DepthMS2Array, DepthArray2MS;
 
     GLuint pointSampler;
     GLuint pointNoMipSampler;
@@ -351,6 +360,7 @@ private:
     GLuint quadoverdrawFragShader;
     GLuint quadoverdrawResolveProg;
 
+    ResourceId overlayTexId;
     GLuint overlayTex;
     GLuint overlayFBO;
     GLuint overlayProg;
@@ -363,8 +373,6 @@ private:
 
   bool m_Degraded;
 
-  GPUVendor m_Vendor = GPUVendor::Unknown;
-
   HighlightCache m_HighlightCache;
 
   // eventId -> data
@@ -417,6 +425,8 @@ private:
 
   GLPipe::State m_CurPipelineState;
 
+  DriverInformation m_DriverInfo;
+
   // AMD counter instance
   AMDCounters *m_pAMDCounters = NULL;
 
