@@ -47,6 +47,7 @@ public:
 
   std::set<HGLRC> contexts;
 
+  void RefreshWindowParameters(const GLWindowingData &data);
   void ProcessSwapBuffers(HDC dc);
   void PopulateFromContext(HDC dc, HGLRC rc);
   GLInitParams GetInitParamsForDC(HDC dc);
@@ -54,6 +55,9 @@ public:
 
 void WGLHook::PopulateFromContext(HDC dc, HGLRC rc)
 {
+  SetDriverForHooks(&driver);
+  EnableGLHooks();
+
   // called from wglCreate*Context*, to populate GL functions as soon as possible by making a new
   // context current and fetching our function pointers
   {
@@ -134,6 +138,19 @@ GLInitParams WGLHook::GetInitParamsForDC(HDC dc)
   return ret;
 }
 
+void WGLHook::RefreshWindowParameters(const GLWindowingData &data)
+{
+  if(haveContextCreation && data.ctx && data.wnd)
+  {
+    RECT r;
+    GetClientRect(data.wnd, &r);
+
+    GLInitParams &params = driver.GetInitParams(data);
+    params.width = r.right - r.left;
+    params.height = r.bottom - r.top;
+  }
+}
+
 void WGLHook::ProcessSwapBuffers(HDC dc)
 {
   HWND w = WindowFromDC(dc);
@@ -142,10 +159,12 @@ void WGLHook::ProcessSwapBuffers(HDC dc)
 
   if(w != NULL && haveContextCreation && !swapRecurse)
   {
-    RECT r;
-    GetClientRect(w, &r);
+    GLWindowingData data;
+    data.DC = dc;
+    data.wnd = w;
+    data.ctx = WGL.wglGetCurrentContext();
 
-    driver.WindowSize(w, r.right - r.left, r.bottom - r.top);
+    RefreshWindowParameters(data);
 
     {
       SCOPED_LOCK(glLock);
@@ -169,8 +188,6 @@ static HGLRC WINAPI wglCreateContext_hooked(HDC dc)
   {
     DWORD err = GetLastError();
 
-    SetDriverForHooks(&wglhook.driver);
-
     wglhook.PopulateFromContext(dc, ret);
 
     GLWindowingData data;
@@ -326,8 +343,6 @@ static HGLRC WINAPI wglCreateContextAttribsARB_hooked(HDC dc, HGLRC hShareContex
     return ret;
   }
 
-  SetDriverForHooks(&wglhook.driver);
-
   wglhook.PopulateFromContext(dc, ret);
 
   GLWindowingData data;
@@ -375,7 +390,9 @@ static BOOL WINAPI wglMakeCurrent_hooked(HDC dc, HGLRC rc)
     data.wnd = WindowFromDC(dc);
     data.ctx = rc;
 
-    if(wglhook.haveContextCreation)
+    wglhook.RefreshWindowParameters(data);
+
+    if(wglhook.haveContextCreation && data.ctx && data.wnd)
       wglhook.driver.ActivateContext(data);
   }
 
@@ -508,8 +525,9 @@ static PROC WINAPI wglGetProcAddress_hooked(const char *func)
   if(!strcmp(func, "wglGetProcAddress"))
     return (PROC)&wglGetProcAddress_hooked;
 
-  // assume wgl functions are safe to just pass straight through
-  if(!strncmp(func, "wgl", 3))
+  // assume wgl functions are safe to just pass straight through, but don't pass through the wgl DX
+  // interop functions
+  if(!strncmp(func, "wgl", 3) && strncmp(func, "wglDX", 5))
     return realFunc;
 
   // otherwise, consult our database of hooks
