@@ -274,6 +274,8 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
 
       reflection.stage = MakeShaderStage(type);
 
+      reflection.debugInfo.encoding = ShaderEncoding::GLSL;
+
       reflection.debugInfo.files.resize(1);
       reflection.debugInfo.files[0].filename = "main.glsl";
       reflection.debugInfo.files[0].contents = concatenated;
@@ -818,13 +820,13 @@ GLuint WrappedOpenGL::glCreateProgram()
 
     record->AddChunk(chunk);
   }
-
+  else
   {
     GetResourceManager()->AddLiveResource(id, res);
-
-    m_Programs[id].linked = false;
   }
 
+  m_Programs[id].linked = false;
+
   return real;
 }
 
@@ -930,6 +932,8 @@ bool WrappedOpenGL::Serialise_glUniformBlockBinding(SerialiserType &ser, GLuint
   if(IsReplayingAndReading())
   {
     GL.glUniformBlockBinding(program.name, uniformBlockIndex, uniformBlockBinding);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -970,6 +974,8 @@ bool WrappedOpenGL::Serialise_glShaderStorageBlockBinding(SerialiserType &ser, G
   if(IsReplayingAndReading())
   {
     GL.glShaderStorageBlockBinding(program.name, storageBlockIndex, storageBlockBinding);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -1009,6 +1015,8 @@ bool WrappedOpenGL::Serialise_glBindAttribLocation(SerialiserType &ser, GLuint p
   if(IsReplayingAndReading())
   {
     GL.glBindAttribLocation(program.name, index, name);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -1047,6 +1055,8 @@ bool WrappedOpenGL::Serialise_glBindFragDataLocation(SerialiserType &ser, GLuint
   if(IsReplayingAndReading())
   {
     GL.glBindFragDataLocation(program.name, color, name);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -1121,6 +1131,8 @@ bool WrappedOpenGL::Serialise_glBindFragDataLocationIndexed(SerialiserType &ser,
   if(IsReplayingAndReading())
   {
     GL.glBindFragDataLocationIndexed(program.name, colorNumber, index, name);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -1163,6 +1175,8 @@ bool WrappedOpenGL::Serialise_glTransformFeedbackVaryings(SerialiserType &ser, G
   if(IsReplayingAndReading())
   {
     GL.glTransformFeedbackVaryings(program.name, count, varyings, bufferMode);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -1202,6 +1216,8 @@ bool WrappedOpenGL::Serialise_glProgramParameteri(SerialiserType &ser, GLuint pr
   if(IsReplayingAndReading())
   {
     GL.glProgramParameteri(program.name, pname, value);
+
+    AddResourceInitChunk(program);
   }
 
   return true;
@@ -1410,6 +1426,8 @@ bool WrappedOpenGL::Serialise_glUseProgramStages(SerialiserType &ser, GLuint pip
 
       GL.glUseProgramStages(pipeline.name, stages, 0);
     }
+
+    AddResourceInitChunk(pipeline);
   }
 
   return true;
