@@ -55,12 +55,44 @@ struct VkInitParams
   uint32_t GetSerialiseSize();
 
   // check if a frame capture section version is supported
-  static const uint64_t CurrentVersion = 0xD;
+  static const uint64_t CurrentVersion = 0xE;
   static bool IsSupportedVersion(uint64_t ver);
 };
 
 DECLARE_REFLECTION_STRUCT(VkInitParams);
 
+enum class VkIndirectPatchType
+{
+  NoPatch,
+  DispatchIndirect,
+  DrawIndirect,
+  DrawIndexedIndirect,
+  DrawIndirectCount,
+  DrawIndexedIndirectCount,
+  DrawIndirectByteCount,
+};
+
+struct VkIndirectRecordData
+{
+  VkBufferMemoryBarrier paramsBarrier, countBarrier;
+
+  struct
+  {
+    VkBuffer src, dst;
+    VkBufferCopy copy;
+  } paramsCopy, countCopy;
+};
+
+struct VkIndirectPatchData
+{
+  VkIndirectPatchType type = VkIndirectPatchType::NoPatch;
+  MemoryAllocation alloc;
+  VkBuffer buf;
+  uint32_t count;
+  uint32_t stride;
+  uint32_t vertexoffset;
+};
+
 struct VulkanDrawcallTreeNode
 {
   VulkanDrawcallTreeNode() {}
@@ -68,6 +100,8 @@ struct VulkanDrawcallTreeNode
   DrawcallDescription draw;
   vector<VulkanDrawcallTreeNode> children;
 
+  VkIndirectPatchData indirectPatch;
+
   vector<pair<ResourceId, EventUsage>> resourceUsage;
 
   vector<ResourceId> executedCmds;
@@ -333,6 +367,13 @@ private:
 
   PFN_vkSetDeviceLoaderData m_SetDeviceLoaderData;
 
+  InstanceDeviceInfo m_EnabledExtensions;
+
+  const InstanceDeviceInfo &GetExtensions(VkResourceRecord *record)
+  {
+    return record ? *record->instDevInfo : m_EnabledExtensions;
+  }
+
   // the instance corresponding to this WrappedVulkan
   VkInstance m_Instance;
   // the instance's dbg msg callback handle
@@ -479,6 +520,8 @@ private:
     vector<DebugMessage> debugMessages;
     std::list<VulkanDrawcallTreeNode *> drawStack;
 
+    std::vector<VkIndirectRecordData> indirectCopies;
+
     uint32_t beginChunk = 0;
     uint32_t endChunk = 0;
 
@@ -503,6 +546,9 @@ private:
       uint32_t idxWidth = 0;
       ResourceId ibuffer;
       std::vector<ResourceId> vbuffers;
+      std::vector<ResourceId> xfbbuffers;
+      uint32_t xfbfirst = 0;
+      uint32_t xfbcount = 0;
 
       ResourceId renderPass;
       ResourceId framebuffer;
@@ -511,7 +557,7 @@ private:
 
     std::vector<std::pair<ResourceId, ImageRegionState>> imgbarriers;
 
-    ResourceId pushDescriptorID[64];
+    ResourceId pushDescriptorID[2][64];
 
     VulkanDrawcallTreeNode *draw;    // the root draw to copy from when submitting
     uint32_t eventCount;             // how many events are in this cmd buffer, for quick skipping
@@ -519,6 +565,8 @@ private:
     uint32_t drawCount;              // similar to above
   };
 
+  bool HasNonMarkerEvents(ResourceId cmdBuffer);
+
   // on replay, the current command buffer for the last chunk we
   // handled.
   ResourceId m_LastCmdBufferID;
@@ -619,16 +667,18 @@ private:
 
   bool InRerecordRange(ResourceId cmdid);
   bool HasRerecordCmdBuf(ResourceId cmdid);
-  bool IsPartialCmdBuf(ResourceId cmdid);
+  bool ShouldUpdateRenderState(ResourceId cmdid, bool forcePrimary = false);
   VkCommandBuffer RerecordCmdBuf(ResourceId cmdid, PartialReplayIndex partialType = ePartialNum);
 
   // this info is stored in the record on capture, but we
   // need it on replay too
   struct DescriptorSetInfo
   {
+    DescriptorSetInfo(bool p = false) : push(p) {}
     ~DescriptorSetInfo() { clear(); }
     ResourceId layout;
     vector<DescriptorSetSlot *> currentBindings;
+    bool push;
 
     void clear()
     {
@@ -681,7 +731,7 @@ private:
   {
     T *ret = GetTempArray<T>(count);
     for(uint32_t i = 0; i < count; i++)
-      ret[i] = Unwrap(wrapped[i]);
+      ret[i] = wrapped ? Unwrap(wrapped[i]) : VK_NULL_HANDLE;
     return ret;
   }
 
@@ -691,6 +741,13 @@ private:
   template <class T>
   T *UnwrapInfos(const T *infos, uint32_t count);
 
+  VkIndirectPatchData FetchIndirectData(VkIndirectPatchType type, VkCommandBuffer commandBuffer,
+                                        VkBuffer dataBuffer, VkDeviceSize dataOffset, uint32_t count,
+                                        uint32_t stride = 0, VkBuffer counterBuffer = VK_NULL_HANDLE,
+                                        VkDeviceSize counterOffset = 0);
+  void ExecuteIndirectReadback(VkCommandBuffer commandBuffer,
+                               const VkIndirectRecordData &indirectcopy);
+
   WriteSerialiser &GetThreadSerialiser();
   template <typename SerialiserType>
   bool Serialise_CaptureScope(SerialiserType &ser);
@@ -707,14 +764,17 @@ private:
 
   std::vector<VkImageMemoryBarrier> GetImplicitRenderPassBarriers(uint32_t subpass = 0);
   string MakeRenderPassOpString(bool store);
-  void MakeSubpassLoadRP(VkRenderPassCreateInfo &info, const VkRenderPassCreateInfo *origInfo,
-                         uint32_t s);
 
   bool IsDrawInRenderPass();
 
   void StartFrameCapture(void *dev, void *wnd);
   bool EndFrameCapture(void *dev, void *wnd);
 
+  void AdvanceFrame();
+  void Present(void *dev, void *wnd);
+
+  void HandleVRFrameMarkers(const char *marker, VkCommandBuffer commandBuffer);
+
   template <typename SerialiserType>
   bool Serialise_SetShaderDebugPath(SerialiserType &ser, VkShaderModule ShaderObject,
                                     std::string DebugPath);
@@ -752,7 +812,9 @@ private:
   void AddRequiredExtensions(bool instance, vector<string> &extensionList,
                              const std::set<string> &supportedExtensions);
 
-  void InsertDrawsAndRefreshIDs(vector<VulkanDrawcallTreeNode> &cmdBufNodes);
+  bool PatchIndirectDraw(VkIndirectPatchType type, DrawcallDescription &draw, byte *&argptr,
+                         byte *argend);
+  void InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo);
 
   list<VulkanDrawcallTreeNode *> m_DrawcallStack;
 
@@ -802,7 +864,6 @@ public:
 
   APIProperties APIProps;
 
-  ResourceId GetContextResourceID() { return m_FrameCaptureRecord->GetResourceID(); }
   static std::string GetChunkName(uint32_t idx);
   VulkanResourceManager *GetResourceManager() { return m_ResourceManager; }
   VulkanDebugManager *GetDebugManager() { return m_DebugManager; }
@@ -872,7 +933,10 @@ public:
     return m_PhysicalDevice;
   }
   VkCommandBuffer GetNextCmd();
-  void SubmitCmds();
+  void RemovePendingCommandBuffer(VkCommandBuffer cmd);
+  void AddPendingCommandBuffer(VkCommandBuffer cmd);
+  void SubmitCmds(VkSemaphore *unwrappedWaitSemaphores = NULL,
+                  VkPipelineStageFlags *waitStageMask = NULL, uint32_t waitSemaphoreCount = 0);
   VkSemaphore GetNextSemaphore();
   void SubmitSemaphores();
   void FlushQ();
@@ -1537,6 +1601,13 @@ public:
                                      const VkAllocationCallbacks *pAllocator, VkSurfaceKHR *pSurface);
 #endif
 
+#if defined(VK_USE_PLATFORM_MACOS_MVK)
+  // VK_MVK_macos_surface
+  VkResult vkCreateMacOSSurfaceMVK(VkInstance instance,
+                                   const VkMacOSSurfaceCreateInfoMVK *pCreateInfo,
+                                   const VkAllocationCallbacks *pAllocator, VkSurfaceKHR *pSurface);
+#endif
+
 #if defined(VK_USE_PLATFORM_XCB_KHR)
   // VK_KHR_xcb_surface
   VkResult vkCreateXcbSurfaceKHR(VkInstance instance, const VkXcbSurfaceCreateInfoKHR *pCreateInfo,
@@ -1693,7 +1764,8 @@ public:
                               VkShaderInfoTypeAMD infoType, size_t *pInfoSize, void *pInfo);
 
   // VK_KHR_push_descriptor
-  void ApplyPushDescriptorWrites(VkPipelineLayout layout, uint32_t set, uint32_t descriptorWriteCount,
+  void ApplyPushDescriptorWrites(VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout,
+                                 uint32_t set, uint32_t descriptorWriteCount,
                                  const VkWriteDescriptorSet *pDescriptorWrites);
 
   IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdPushDescriptorSetKHR, VkCommandBuffer commandBuffer,
@@ -1825,4 +1897,87 @@ public:
                                                  const VkPhysicalDeviceSurfaceInfo2KHR *pSurfaceInfo,
                                                  uint32_t *pSurfaceFormatCount,
                                                  VkSurfaceFormat2KHR *pSurfaceFormats);
+
+  // VK_KHR_get_display_properties2
+  VkResult vkGetPhysicalDeviceDisplayProperties2KHR(VkPhysicalDevice physicalDevice,
+                                                    uint32_t *pPropertyCount,
+                                                    VkDisplayProperties2KHR *pProperties);
+
+  VkResult vkGetPhysicalDeviceDisplayPlaneProperties2KHR(VkPhysicalDevice physicalDevice,
+                                                         uint32_t *pPropertyCount,
+                                                         VkDisplayPlaneProperties2KHR *pProperties);
+
+  VkResult vkGetDisplayModeProperties2KHR(VkPhysicalDevice physicalDevice, VkDisplayKHR display,
+                                          uint32_t *pPropertyCount,
+                                          VkDisplayModeProperties2KHR *pProperties);
+
+  VkResult vkGetDisplayPlaneCapabilities2KHR(VkPhysicalDevice physicalDevice,
+                                             const VkDisplayPlaneInfo2KHR *pDisplayPlaneInfo,
+                                             VkDisplayPlaneCapabilities2KHR *pCapabilities);
+
+  // VK_KHR_draw_indirect_count
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdDrawIndirectCountKHR, VkCommandBuffer commandBuffer,
+                                VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer,
+                                VkDeviceSize countBufferOffset, uint32_t maxDrawCount,
+                                uint32_t stride);
+
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdDrawIndexedIndirectCountKHR,
+                                VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset,
+                                VkBuffer countBuffer, VkDeviceSize countBufferOffset,
+                                uint32_t maxDrawCount, uint32_t stride);
+
+  // VK_EXT_validation_cache
+  VkResult vkCreateValidationCacheEXT(VkDevice device,
+                                      const VkValidationCacheCreateInfoEXT *pCreateInfo,
+                                      const VkAllocationCallbacks *pAllocator,
+                                      VkValidationCacheEXT *pValidationCache);
+
+  void vkDestroyValidationCacheEXT(VkDevice device, VkValidationCacheEXT validationCache,
+                                   const VkAllocationCallbacks *pAllocator);
+
+  VkResult vkMergeValidationCachesEXT(VkDevice device, VkValidationCacheEXT dstCache,
+                                      uint32_t srcCacheCount, const VkValidationCacheEXT *pSrcCaches);
+
+  VkResult vkGetValidationCacheDataEXT(VkDevice device, VkValidationCacheEXT validationCache,
+                                       size_t *pDataSize, void *pData);
+
+  // VK_KHR_shared_presentable_image
+  VkResult vkGetSwapchainStatusKHR(VkDevice device, VkSwapchainKHR swapchain);
+
+  // VK_KHR_create_renderpass2
+  IMPLEMENT_FUNCTION_SERIALISED(VkResult, vkCreateRenderPass2KHR, VkDevice device,
+                                const VkRenderPassCreateInfo2KHR *pCreateInfo,
+                                const VkAllocationCallbacks *pAllocator, VkRenderPass *pRenderPass);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdBeginRenderPass2KHR, VkCommandBuffer commandBuffer,
+                                const VkRenderPassBeginInfo *pRenderPassBegin,
+                                const VkSubpassBeginInfoKHR *pSubpassBeginInfo);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdNextSubpass2KHR, VkCommandBuffer commandBuffer,
+                                const VkSubpassBeginInfoKHR *pSubpassBeginInfo,
+                                const VkSubpassEndInfoKHR *pSubpassEndInfo);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdEndRenderPass2KHR, VkCommandBuffer commandBuffer,
+                                const VkSubpassEndInfoKHR *pSubpassEndInfo);
+
+  // VK_EXT_transform_feedback
+
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdBindTransformFeedbackBuffersEXT,
+                                VkCommandBuffer commandBuffer, uint32_t firstBinding,
+                                uint32_t bindingCount, const VkBuffer *pBuffers,
+                                const VkDeviceSize *pOffsets, const VkDeviceSize *pSizes);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdBeginTransformFeedbackEXT, VkCommandBuffer commandBuffer,
+                                uint32_t firstBuffer, uint32_t bufferCount,
+                                const VkBuffer *pCounterBuffers,
+                                const VkDeviceSize *pCounterBufferOffsets);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdEndTransformFeedbackEXT, VkCommandBuffer commandBuffer,
+                                uint32_t firstBuffer, uint32_t bufferCount,
+                                const VkBuffer *pCounterBuffers,
+                                const VkDeviceSize *pCounterBufferOffsets);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdBeginQueryIndexedEXT, VkCommandBuffer commandBuffer,
+                                VkQueryPool queryPool, uint32_t query, VkQueryControlFlags flags,
+                                uint32_t index);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdEndQueryIndexedEXT, VkCommandBuffer commandBuffer,
+                                VkQueryPool queryPool, uint32_t query, uint32_t index);
+  IMPLEMENT_FUNCTION_SERIALISED(void, vkCmdDrawIndirectByteCountEXT, VkCommandBuffer commandBuffer,
+                                uint32_t instanceCount, uint32_t firstInstance,
+                                VkBuffer counterBuffer, VkDeviceSize counterBufferOffset,
+                                uint32_t counterOffset, uint32_t vertexStride);
 };
