@@ -25,6 +25,7 @@
 #pragma once
 
 #include "official/vk_layer.h"
+#include "official/vk_layer_dispatch_table.h"
 
 // extensions made core in 1.1
 #define VK11 VK_MAKE_VERSION(1, 1, 0)
@@ -69,6 +70,18 @@
   HookDefine3(VkResult, vkGetFenceWin32HandleKHR, VkDevice, device,                              \
               const VkFenceGetWin32HandleInfoKHR *, pGetWin32HandleInfo, HANDLE *, pHandle);
 
+#elif defined(VK_USE_PLATFORM_MACOS_MVK)
+
+#define HookInitInstance_PlatformSpecific() \
+  HookInitExtension(VK_MVK_macos_surface, CreateMacOSSurfaceMVK);
+
+#define HookInitDevice_PlatformSpecific()
+
+#define HookDefine_PlatformSpecific()                                                          \
+  HookDefine4(VkResult, vkCreateMacOSSurfaceMVK, VkInstance, instance,                         \
+              const VkMacOSSurfaceCreateInfoMVK *, pCreateInfo, const VkAllocationCallbacks *, \
+              pAllocator, VkSurfaceKHR *, pSurface);
+
 #elif defined(VK_USE_PLATFORM_ANDROID_KHR)
 
 #define HookInitInstance_PlatformSpecific() \
@@ -285,6 +298,7 @@
   CheckExt(KHR_xcb_surface, VKXX);                     \
   CheckExt(KHR_win32_surface, VKXX);                   \
   CheckExt(KHR_android_surface, VKXX);                 \
+  CheckExt(MVK_macos_surface, VKXX);                   \
   CheckExt(KHR_surface, VKXX);                         \
   CheckExt(EXT_debug_report, VKXX);                    \
   CheckExt(KHR_display, VKXX);                         \
@@ -299,7 +313,8 @@
   CheckExt(EXT_debug_utils, VKXX);                     \
   CheckExt(KHR_device_group_creation, VK11);           \
   CheckExt(protected_memory, VK11);                    \
-  CheckExt(KHR_get_surface_capabilities2, VKXX);
+  CheckExt(KHR_get_surface_capabilities2, VKXX);       \
+  CheckExt(KHR_get_display_properties2, VKXX);
 
 #define CheckDeviceExts()                         \
   CheckExt(EXT_debug_marker, VKXX);               \
@@ -332,7 +347,13 @@
   CheckExt(EXT_vertex_attribute_divisor, VKXX);   \
   CheckExt(EXT_sampler_filter_minmax, VKXX);      \
   CheckExt(KHR_sampler_ycbcr_conversion, VK11);   \
-  CheckExt(KHR_device_group, VK11);
+  CheckExt(KHR_device_group, VK11);               \
+  CheckExt(MVK_moltenvk, VKXX);                   \
+  CheckExt(KHR_draw_indirect_count, VKXX);        \
+  CheckExt(EXT_validation_cache, VKXX);           \
+  CheckExt(KHR_shared_presentable_image, VKXX);   \
+  CheckExt(KHR_create_renderpass2, VKXX);         \
+  CheckExt(EXT_transform_feedback, VKXX);
 
 #define HookInitVulkanInstanceExts()                                                                 \
   HookInitExtension(KHR_surface, DestroySurfaceKHR);                                                 \
@@ -382,6 +403,10 @@
   HookInitExtension(KHR_device_group_creation &&KHR_surface, GetPhysicalDevicePresentRectanglesKHR); \
   HookInitExtension(KHR_get_surface_capabilities2, GetPhysicalDeviceSurfaceFormats2KHR);             \
   HookInitExtension(KHR_get_surface_capabilities2, GetPhysicalDeviceSurfaceCapabilities2KHR);        \
+  HookInitExtension(KHR_get_display_properties2, GetPhysicalDeviceDisplayProperties2KHR);            \
+  HookInitExtension(KHR_get_display_properties2, GetPhysicalDeviceDisplayPlaneProperties2KHR);       \
+  HookInitExtension(KHR_get_display_properties2, GetDisplayModeProperties2KHR);                      \
+  HookInitExtension(KHR_get_display_properties2, GetDisplayPlaneCapabilities2KHR);                   \
   HookInitInstance_PlatformSpecific()
 
 #define HookInitVulkanDeviceExts()                                                                 \
@@ -438,6 +463,23 @@
   HookInitExtension(KHR_device_group &&KHR_surface, GetDeviceGroupSurfacePresentModesKHR);         \
   HookInitExtension(KHR_device_group &&KHR_swapchain, AcquireNextImage2KHR);                       \
   HookInitExtension(protected_memory, GetDeviceQueue2);                                            \
+  HookInitExtension(KHR_draw_indirect_count, CmdDrawIndirectCountKHR);                             \
+  HookInitExtension(KHR_draw_indirect_count, CmdDrawIndexedIndirectCountKHR);                      \
+  HookInitExtension(EXT_validation_cache, CreateValidationCacheEXT);                               \
+  HookInitExtension(EXT_validation_cache, DestroyValidationCacheEXT);                              \
+  HookInitExtension(EXT_validation_cache, MergeValidationCachesEXT);                               \
+  HookInitExtension(EXT_validation_cache, GetValidationCacheDataEXT);                              \
+  HookInitExtension(KHR_shared_presentable_image, GetSwapchainStatusKHR);                          \
+  HookInitExtension(KHR_create_renderpass2, CreateRenderPass2KHR);                                 \
+  HookInitExtension(KHR_create_renderpass2, CmdBeginRenderPass2KHR);                               \
+  HookInitExtension(KHR_create_renderpass2, CmdNextSubpass2KHR);                                   \
+  HookInitExtension(KHR_create_renderpass2, CmdEndRenderPass2KHR);                                 \
+  HookInitExtension(EXT_transform_feedback, CmdBindTransformFeedbackBuffersEXT);                   \
+  HookInitExtension(EXT_transform_feedback, CmdBeginTransformFeedbackEXT);                         \
+  HookInitExtension(EXT_transform_feedback, CmdEndTransformFeedbackEXT);                           \
+  HookInitExtension(EXT_transform_feedback, CmdBeginQueryIndexedEXT);                              \
+  HookInitExtension(EXT_transform_feedback, CmdEndQueryIndexedEXT);                                \
+  HookInitExtension(EXT_transform_feedback, CmdDrawIndirectByteCountEXT);                          \
   HookInitDevice_PlatformSpecific()
 
 #define DefineHooks()                                                                                \
@@ -937,6 +979,60 @@
   HookDefine4(VkResult, vkGetPhysicalDeviceSurfaceFormats2KHR, VkPhysicalDevice, physicalDevice,     \
               const VkPhysicalDeviceSurfaceInfo2KHR *, pSurfaceInfo, uint32_t *,                     \
               pSurfaceFormatCount, VkSurfaceFormat2KHR *, pSurfaceFormats);                          \
+  HookDefine3(VkResult, vkGetPhysicalDeviceDisplayProperties2KHR, VkPhysicalDevice,                  \
+              physicalDevice, uint32_t *, pPropertyCount, VkDisplayProperties2KHR *, pProperties);   \
+  HookDefine3(VkResult, vkGetPhysicalDeviceDisplayPlaneProperties2KHR, VkPhysicalDevice,             \
+              physicalDevice, uint32_t *, pPropertyCount, VkDisplayPlaneProperties2KHR *,            \
+              pProperties);                                                                          \
+  HookDefine4(VkResult, vkGetDisplayModeProperties2KHR, VkPhysicalDevice, physicalDevice,            \
+              VkDisplayKHR, display, uint32_t *, pPropertyCount, VkDisplayModeProperties2KHR *,      \
+              pProperties);                                                                          \
+  HookDefine3(VkResult, vkGetDisplayPlaneCapabilities2KHR, VkPhysicalDevice, physicalDevice,         \
+              const VkDisplayPlaneInfo2KHR *, pDisplayPlaneInfo, VkDisplayPlaneCapabilities2KHR *,   \
+              pCapabilities);                                                                        \
+  HookDefine7(void, vkCmdDrawIndirectCountKHR, VkCommandBuffer, commandBuffer, VkBuffer, buffer,     \
+              VkDeviceSize, offset, VkBuffer, countBuffer, VkDeviceSize, countBufferOffset,          \
+              uint32_t, maxDrawCount, uint32_t, stride);                                             \
+  HookDefine7(void, vkCmdDrawIndexedIndirectCountKHR, VkCommandBuffer, commandBuffer, VkBuffer,      \
+              buffer, VkDeviceSize, offset, VkBuffer, countBuffer, VkDeviceSize,                     \
+              countBufferOffset, uint32_t, maxDrawCount, uint32_t, stride);                          \
+  HookDefine4(VkResult, vkCreateValidationCacheEXT, VkDevice, device,                                \
+              const VkValidationCacheCreateInfoEXT *, pCreateInfo, const VkAllocationCallbacks *,    \
+              pAllocator, VkValidationCacheEXT *, pValidationCache);                                 \
+  HookDefine3(void, vkDestroyValidationCacheEXT, VkDevice, device, VkValidationCacheEXT,             \
+              validationCache, const VkAllocationCallbacks *, pAllocator);                           \
+  HookDefine4(VkResult, vkMergeValidationCachesEXT, VkDevice, device, VkValidationCacheEXT,          \
+              dstCache, uint32_t, srcCacheCount, const VkValidationCacheEXT *, pSrcCaches);          \
+  HookDefine4(VkResult, vkGetValidationCacheDataEXT, VkDevice, device, VkValidationCacheEXT,         \
+              validationCache, size_t *, pDataSize, void *, pData);                                  \
+  HookDefine2(VkResult, vkGetSwapchainStatusKHR, VkDevice, device, VkSwapchainKHR, swapchain);       \
+  HookDefine4(VkResult, vkCreateRenderPass2KHR, VkDevice, device,                                    \
+              const VkRenderPassCreateInfo2KHR *, pCreateInfo, const VkAllocationCallbacks *,        \
+              pAllocator, VkRenderPass *, pRenderPass);                                              \
+  HookDefine3(void, vkCmdBeginRenderPass2KHR, VkCommandBuffer, commandBuffer,                        \
+              const VkRenderPassBeginInfo *, pRenderPassBegin, const VkSubpassBeginInfoKHR *,        \
+              pSubpassBeginInfo);                                                                    \
+  HookDefine3(void, vkCmdNextSubpass2KHR, VkCommandBuffer, commandBuffer,                            \
+              const VkSubpassBeginInfoKHR *, pSubpassBeginInfo, const VkSubpassEndInfoKHR *,         \
+              pSubpassEndInfo);                                                                      \
+  HookDefine2(void, vkCmdEndRenderPass2KHR, VkCommandBuffer, commandBuffer,                          \
+              const VkSubpassEndInfoKHR *, pSubpassEndInfo);                                         \
+  HookDefine6(void, vkCmdBindTransformFeedbackBuffersEXT, VkCommandBuffer, commandBuffer,            \
+              uint32_t, firstBinding, uint32_t, bindingCount, const VkBuffer *, pBuffers,            \
+              const VkDeviceSize *, pOffsets, const VkDeviceSize *, pSizes);                         \
+  HookDefine5(void, vkCmdBeginTransformFeedbackEXT, VkCommandBuffer, commandBuffer, uint32_t,        \
+              firstBuffer, uint32_t, bufferCount, const VkBuffer *, pCounterBuffers,                 \
+              const VkDeviceSize *, pCounterBufferOffsets);                                          \
+  HookDefine5(void, vkCmdEndTransformFeedbackEXT, VkCommandBuffer, commandBuffer, uint32_t,          \
+              firstBuffer, uint32_t, bufferCount, const VkBuffer *, pCounterBuffers,                 \
+              const VkDeviceSize *, pCounterBufferOffsets);                                          \
+  HookDefine5(void, vkCmdBeginQueryIndexedEXT, VkCommandBuffer, commandBuffer, VkQueryPool,          \
+              queryPool, uint32_t, query, VkQueryControlFlags, flags, uint32_t, index);              \
+  HookDefine4(void, vkCmdEndQueryIndexedEXT, VkCommandBuffer, commandBuffer, VkQueryPool,            \
+              queryPool, uint32_t, query, uint32_t, index);                                          \
+  HookDefine7(void, vkCmdDrawIndirectByteCountEXT, VkCommandBuffer, commandBuffer, uint32_t,         \
+              instanceCount, uint32_t, firstInstance, VkBuffer, counterBuffer, VkDeviceSize,         \
+              counterBufferOffset, uint32_t, counterOffset, uint32_t, vertexStride);                 \
   HookDefine_PlatformSpecific()
 
 struct VkLayerInstanceDispatchTableExtended : VkLayerInstanceDispatchTable
