@@ -190,6 +190,9 @@ struct VulkanCreationInfo
     float depthBiasSlopeFactor;
     float lineWidth;
 
+    // VkPipelineRasterizationStateStreamCreateInfoEXT
+    uint32_t rasterizationStream;
+
     // VkPipelineRasterizationConservativeStateCreateInfoEXT
     VkConservativeRasterizationModeEXT conservativeRasterizationMode;
     float extraPrimitiveOverestimationSize;
@@ -252,8 +255,23 @@ struct VulkanCreationInfo
   {
     void Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
               const VkRenderPassCreateInfo *pCreateInfo);
+    void Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
+              const VkRenderPassCreateInfo2KHR *pCreateInfo);
 
-    vector<VkAttachmentDescription> attachments;
+    struct Attachment
+    {
+      VkAttachmentDescriptionFlags flags;
+      VkFormat format;
+      VkSampleCountFlagBits samples;
+      VkAttachmentLoadOp loadOp;
+      VkAttachmentStoreOp storeOp;
+      VkAttachmentLoadOp stencilLoadOp;
+      VkAttachmentStoreOp stencilStoreOp;
+      VkImageLayout initialLayout;
+      VkImageLayout finalLayout;
+    };
+
+    vector<Attachment> attachments;
 
     struct Subpass
     {
