@@ -88,7 +88,7 @@ bool WrappedVulkan::Prepare_InitialState(WrappedVkRes *res)
     WrappedVkBuffer *buffer = (WrappedVkBuffer *)res;
 
     // buffers are only dirty if they are sparse
-    RDCASSERT(buffer->record->sparseInfo);
+    RDCASSERT(buffer->record->resInfo && buffer->record->resInfo->IsSparse());
 
     return Prepare_SparseInitialState(buffer);
   }
@@ -98,7 +98,7 @@ bool WrappedVulkan::Prepare_InitialState(WrappedVkRes *res)
 
     WrappedVkImage *im = (WrappedVkImage *)res;
 
-    if(im->record->sparseInfo)
+    if(im->record->resInfo && im->record->resInfo->IsSparse())
     {
       // if the image is sparse we have to do a different kind of initial state prepare,
       // to serialise out the page mapping. The fetching of memory is also different
@@ -458,6 +458,11 @@ bool WrappedVulkan::Prepare_InitialState(WrappedVkRes *res)
       queues.push_back(i);
     bufInfo.pQueueFamilyIndices = queues.data();
 
+    // spec requires that CONCURRENT must specify more than one queue family. If there is only one
+    // queue family, we can safely use exclusive.
+    if(bufInfo.queueFamilyIndexCount == 1)
+      bufInfo.sharingMode = VK_SHARING_MODE_EXCLUSIVE;
+
     // since this happens during capture, we don't want to start serialising extra buffer creates,
     // so we manually create & then just wrap.
     VkBuffer srcBuf, dstBuf;
@@ -708,6 +713,8 @@ bool WrappedVulkan::Serialise_InitialState(SerialiserType &ser, ResourceId id, W
 
         uint32_t descriptorCount = layout.bindings[j].descriptorCount;
 
+        ResourceId *immutableSamplers = layout.bindings[j].immutableSampler;
+
         DescriptorSetSlot *src = srcData;
         srcData += descriptorCount;
 
@@ -753,6 +760,13 @@ bool WrappedVulkan::Serialise_InitialState(SerialiserType &ser, ResourceId id, W
               for(uint32_t d = 0; d < descriptorCount; d++)
                 dstImage[d] = src[d].imageInfo;
 
+              if(immutableSamplers)
+              {
+                for(uint32_t d = 0; d < descriptorCount; d++)
+                  dstImage[d].sampler =
+                      GetResourceManager()->GetCurrentHandle<VkSampler>(immutableSamplers[d]);
+              }
+
               writes[bind].pImageInfo = dstImage;
               // NULL the others
               dstBuffer = NULL;
@@ -900,6 +914,18 @@ bool WrappedVulkan::Serialise_InitialState(SerialiserType &ser, ResourceId id, W
       vkr = ObjDisp(d)->MapMemory(Unwrap(d), Unwrap(mappedMem.mem), initContents.mem.offs,
                                   initContents.mem.size, 0, (void **)&Contents);
       RDCASSERTEQUAL(vkr, VK_SUCCESS);
+
+      // invalidate the cpu cache for this memory range to avoid reading stale data
+      VkMappedMemoryRange range = {
+          VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE,
+          NULL,
+          Unwrap(mappedMem.mem),
+          mappedMem.offs,
+          mappedMem.size,
+      };
+
+      vkr = ObjDisp(d)->InvalidateMappedMemoryRanges(Unwrap(d), 1, &range);
+      RDCASSERTEQUAL(vkr, VK_SUCCESS);
     }
     else if(IsReplayingAndReading() && !ser.IsErrored())
     {
@@ -933,7 +959,24 @@ bool WrappedVulkan::Serialise_InitialState(SerialiserType &ser, ResourceId id, W
 
     // unmap the resource we mapped before - we need to do this on read and on write.
     if(!IsStructuredExporting(m_State) && mappedMem.mem != VK_NULL_HANDLE)
+    {
+      if(IsReplayingAndReading())
+      {
+        // first ensure we flush the writes from the cpu to gpu memory
+        VkMappedMemoryRange range = {
+            VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE,
+            NULL,
+            Unwrap(mappedMem.mem),
+            mappedMem.offs,
+            mappedMem.size,
+        };
+
+        vkr = ObjDisp(d)->FlushMappedMemoryRanges(Unwrap(d), 1, &range);
+        RDCASSERTEQUAL(vkr, VK_SUCCESS);
+      }
+
       ObjDisp(d)->UnmapMemory(Unwrap(d), Unwrap(mappedMem.mem));
+    }
 
     SERIALISE_CHECK_READ_ERRORS();
 
