@@ -25,13 +25,6 @@
 #include "vk_manager.h"
 #include "vk_core.h"
 
-bool VulkanResourceManager::SerialisableResource(ResourceId id, VkResourceRecord *record)
-{
-  if(record->SpecialResource || id == m_Core->GetContextResourceID())
-    return false;
-  return true;
-}
-
 // debugging logging for barriers
 #if 0
 #define TRDBG(...) RDCLOG(__VA_ARGS__)
@@ -361,7 +354,7 @@ template void VulkanResourceManager::SerialiseImageStates(WriteSerialiser &ser,
                                                           std::map<ResourceId, ImageLayouts> &states,
                                                           std::vector<VkImageMemoryBarrier> &barriers);
 
-void VulkanResourceManager::MarkSparseMapReferenced(SparseMapping *sparse)
+void VulkanResourceManager::MarkSparseMapReferenced(ResourceInfo *sparse)
 {
   if(sparse == NULL)
   {
@@ -381,6 +374,16 @@ void VulkanResourceManager::MarkSparseMapReferenced(SparseMapping *sparse)
   }
 }
 
+void VulkanResourceManager::SetInternalResource(ResourceId id)
+{
+  if(!RenderDoc::Inst().IsReplayApp())
+  {
+    VkResourceRecord *record = GetResourceRecord(id);
+    if(record)
+      record->InternalResource = true;
+  }
+}
+
 void VulkanResourceManager::ApplyBarriers(uint32_t queueFamilyIndex,
                                           vector<pair<ResourceId, ImageRegionState> > &states,
                                           map<ResourceId, ImageLayouts> &layouts)
@@ -609,6 +612,16 @@ void VulkanResourceManager::Apply_InitialState(WrappedVkRes *live, VkInitialCont
   return m_Core->Apply_InitialState(live, initial);
 }
 
+std::vector<ResourceId> VulkanResourceManager::InitialContentResources()
+{
+  std::vector<ResourceId> resources =
+      ResourceManager<VulkanResourceManagerConfiguration>::InitialContentResources();
+  std::sort(resources.begin(), resources.end(), [this](ResourceId a, ResourceId b) {
+    return m_InitialContents[a].type < m_InitialContents[b].type;
+  });
+  return resources;
+}
+
 bool VulkanResourceManager::ResourceTypeRelease(WrappedVkRes *res)
 {
   return m_Core->ReleaseResource(res);
