@@ -310,6 +310,7 @@ public:
     if(IsReplayMode(m_State))
       ResourceManager::RemoveWrapper(ToTypedHandle(Unwrap(obj)));
 
+    ResourceManager::MarkCleanResource(id);
     ResourceManager::ReleaseCurrentResource(id);
     VkResourceRecord *record = GetRecord(obj);
     if(record)
@@ -403,11 +404,11 @@ public:
   }
 
   // helper for sparse mappings
-  void MarkSparseMapReferenced(SparseMapping *sparse);
+  void MarkSparseMapReferenced(ResourceInfo *sparse);
+
+  void SetInternalResource(ResourceId id);
 
 private:
-  bool SerialisableResource(ResourceId id, VkResourceRecord *record);
-
   bool ResourceTypeRelease(WrappedVkRes *res);
 
   bool Force_InitialState(WrappedVkRes *res, bool prepare);
@@ -418,6 +419,7 @@ private:
   bool Serialise_InitialState(WriteSerialiser &ser, ResourceId resid, WrappedVkRes *res);
   void Create_InitialState(ResourceId id, WrappedVkRes *live, bool hasData);
   void Apply_InitialState(WrappedVkRes *live, VkInitialContents initial);
+  std::vector<ResourceId> InitialContentResources();
 
   CaptureState m_State;
   WrappedVulkan *m_Core;
