@@ -126,15 +126,10 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
         att->colorWriteMask = 0x0;
       }
 
-      // disable depth/stencil writes
+      // disable depth/stencil writes but keep any tests enabled
       VkPipelineDepthStencilStateCreateInfo *ds =
           (VkPipelineDepthStencilStateCreateInfo *)pipeCreateInfo.pDepthStencilState;
       ds->depthWriteEnable = false;
-      ds->stencilTestEnable = false;
-      ds->depthBoundsTestEnable = false;
-      ds->front.compareOp = ds->back.compareOp = VK_COMPARE_OP_ALWAYS;
-      ds->front.compareMask = ds->back.compareMask = ds->front.writeMask = ds->back.writeMask = 0xff;
-      ds->front.reference = ds->back.reference = 0;
       ds->front.passOp = ds->front.failOp = ds->front.depthFailOp = VK_STENCIL_OP_KEEP;
       ds->back.passOp = ds->back.failOp = ds->back.depthFailOp = VK_STENCIL_OP_KEEP;
 
@@ -288,6 +283,8 @@ void VulkanDebugManager::PatchFixedColShader(VkShaderModule &mod, float col[4])
   alias.spirv = &spv[0];
   size_t spirvLength = spv.size();
 
+  int patched = 0;
+
   size_t it = 5;
   while(it < spirvLength)
   {
@@ -296,21 +293,26 @@ void VulkanDebugManager::PatchFixedColShader(VkShaderModule &mod, float col[4])
 
     if(opcode == spv::OpConstant)
     {
-      if(alias.data[it + 3] == 1.1f)
+      if(alias.data[it + 3] >= 1.0f && alias.data[it + 3] <= 1.5f)
         alias.data[it + 3] = col[0];
-      else if(alias.data[it + 3] == 2.2f)
+      else if(alias.data[it + 3] >= 2.0f && alias.data[it + 3] <= 2.5f)
         alias.data[it + 3] = col[1];
-      else if(alias.data[it + 3] == 3.3f)
+      else if(alias.data[it + 3] >= 3.0f && alias.data[it + 3] <= 3.5f)
         alias.data[it + 3] = col[2];
-      else if(alias.data[it + 3] == 4.4f)
+      else if(alias.data[it + 3] >= 4.0f && alias.data[it + 3] <= 4.5f)
         alias.data[it + 3] = col[3];
       else
         RDCERR("Unexpected constant value");
+
+      patched++;
     }
 
     it += WordCount;
   }
 
+  if(patched != 4)
+    RDCERR("Didn't patch all constants");
+
   VkShaderModuleCreateInfo modinfo = {
       VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO,
       NULL,
@@ -417,6 +419,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
   VkResult vkr = vt->BeginCommandBuffer(Unwrap(cmd), &beginInfo);
   RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
+  VkMarkerRegion::Begin(StringFormat::Fmt("RenderOverlay %d", overlay), cmd);
+
   // if the overlay image is the wrong size, free it
   if(m_Overlay.Image != VK_NULL_HANDLE && (iminfo.extent.width != m_Overlay.ImageDim.width ||
                                            iminfo.extent.height != m_Overlay.ImageDim.height))
@@ -1230,13 +1234,35 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       };
 
       ResourceId depthView = createinfo.m_Framebuffer[state.framebuffer].attachments[dsIdx].view;
-      ResourceId depthIm = createinfo.m_ImageView[depthView].image;
+      VulkanCreationInfo::ImageView &depthViewInfo = createinfo.m_ImageView[depthView];
 
-      attDescs[1].format = createinfo.m_Image[depthIm].format;
+      ResourceId depthIm = depthViewInfo.image;
+      VulkanCreationInfo::Image &depthImageInfo = createinfo.m_Image[depthIm];
+
+      attDescs[1].format = depthImageInfo.format;
       attDescs[0].samples = attDescs[1].samples = iminfo.samples;
 
+      std::vector<ImageRegionState> &depthStates =
+          m_pDriver->m_ImageLayouts[depthIm].subresourceStates;
+
+      for(ImageRegionState &ds : depthStates)
+      {
+        // find the state that overlaps the view's subresource range start. We assume all
+        // subresources are correctly in the same state (as they should be) so we just need to find
+        // the first match.
+        if(ds.subresourceRange.baseArrayLayer <= depthViewInfo.range.baseArrayLayer &&
+           ds.subresourceRange.baseArrayLayer + 1 > depthViewInfo.range.baseArrayLayer &&
+           ds.subresourceRange.baseMipLevel <= depthViewInfo.range.baseMipLevel &&
+           ds.subresourceRange.baseMipLevel + ds.subresourceRange.levelCount + 1 >
+               depthViewInfo.range.baseMipLevel)
+        {
+          attDescs[1].initialLayout = attDescs[1].finalLayout = ds.newLayout;
+          break;
+        }
+      }
+
       VkAttachmentReference colRef = {0, VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL};
-      VkAttachmentReference dsRef = {1, VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL};
+      VkAttachmentReference dsRef = {1, attDescs[1].initialLayout};
 
       VkSubpassDescription sub = {
           0,      VK_PIPELINE_BIND_POINT_GRAPHICS,
@@ -1620,6 +1646,16 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
       events.push_back(eventId);
 
+      // if we're rendering the whole pass, and the first draw is a BeginRenderPass, don't include
+      // it in the list. We want to start by replaying into the renderpass so that we have the
+      // correct state being applied.
+      if(overlay == DebugOverlay::QuadOverdrawPass)
+      {
+        const DrawcallDescription *draw = m_pDriver->GetDrawcall(events[0]);
+        if(draw->flags & DrawFlags::BeginPass)
+          events.erase(events.begin());
+      }
+
       VkImage quadImg;
       VkDeviceMemory quadImgMem;
       VkImageView quadImgView;
@@ -1937,13 +1973,35 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
         };
 
         ResourceId depthView = createinfo.m_Framebuffer[state.framebuffer].attachments[dsIdx].view;
-        ResourceId depthIm = createinfo.m_ImageView[depthView].image;
+        VulkanCreationInfo::ImageView &depthViewInfo = createinfo.m_ImageView[depthView];
 
-        attDescs[1].format = createinfo.m_Image[depthIm].format;
+        ResourceId depthIm = depthViewInfo.image;
+        VulkanCreationInfo::Image &depthImageInfo = createinfo.m_Image[depthIm];
+
+        attDescs[1].format = depthImageInfo.format;
         attDescs[0].samples = attDescs[1].samples = iminfo.samples;
 
+        std::vector<ImageRegionState> &depthStates =
+            m_pDriver->m_ImageLayouts[depthIm].subresourceStates;
+
+        for(ImageRegionState &ds : depthStates)
+        {
+          // find the state that overlaps the view's subresource range start. We assume all
+          // subresources are correctly in the same state (as they should be) so we just need to
+          // find the first match.
+          if(ds.subresourceRange.baseArrayLayer <= depthViewInfo.range.baseArrayLayer &&
+             ds.subresourceRange.baseArrayLayer + 1 > depthViewInfo.range.baseArrayLayer &&
+             ds.subresourceRange.baseMipLevel <= depthViewInfo.range.baseMipLevel &&
+             ds.subresourceRange.baseMipLevel + ds.subresourceRange.levelCount + 1 >
+                 depthViewInfo.range.baseMipLevel)
+          {
+            attDescs[1].initialLayout = attDescs[1].finalLayout = ds.newLayout;
+            break;
+          }
+        }
+
         VkAttachmentReference colRef = {0, VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL};
-        VkAttachmentReference dsRef = {1, VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL};
+        VkAttachmentReference dsRef = {1, attDescs[1].initialLayout};
 
         VkSubpassDescription sub = {
             0,      VK_PIPELINE_BIND_POINT_GRAPHICS,
@@ -2203,6 +2261,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
         }
       }
 
+      vt->CmdEndRenderPass(Unwrap(cmd));
+
       vkr = vt->EndCommandBuffer(Unwrap(cmd));
       RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
@@ -2231,6 +2291,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     RDCASSERTEQUAL(vkr, VK_SUCCESS);
   }
 
+  VkMarkerRegion::End(cmd);
+
   vkr = vt->EndCommandBuffer(Unwrap(cmd));
   RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
