@@ -33,6 +33,8 @@ enum class FeatureCheck
   NoCheck = 0x0,
   ShaderMSAAStorage = 0x1,
   FragmentStores = 0x2,
+  // only unsupported on MoltenVK
+  MSAAArrays = 0x4,
 };
 
 BITMASK_OPERATORS(FeatureCheck);
@@ -82,7 +84,7 @@ static const BuiltinShaderConfig builtinShaders[] = {
     {BuiltinShader::Array2MSCS, EmbeddedResource(glsl_array2ms_comp), SPIRVShaderStage::Compute,
      FeatureCheck::ShaderMSAAStorage, true},
     {BuiltinShader::DepthMS2ArrayFS, EmbeddedResource(glsl_depthms2arr_frag),
-     SPIRVShaderStage::Fragment, FeatureCheck::NoCheck, true},
+     SPIRVShaderStage::Fragment, FeatureCheck::MSAAArrays, true},
     {BuiltinShader::DepthArray2MSFS, EmbeddedResource(glsl_deptharr2ms_frag),
      SPIRVShaderStage::Fragment, FeatureCheck::NoCheck, true},
 };
@@ -154,6 +156,13 @@ VulkanShaderCache::VulkanShaderCache(WrappedVulkan *driver)
         continue;
     }
 
+    if(config.checks & FeatureCheck::MSAAArrays)
+    {
+      // for now we don't allow it at all - in future we could check on whether it's been enabled
+      if(driver->GetExtensions(GetRecord(m_Device)).ext_MVK_moltenvk)
+        continue;
+    }
+
     if(config.stage == SPIRVShaderStage::Geometry && !features.geometryShader)
       continue;
 
@@ -187,6 +196,8 @@ VulkanShaderCache::VulkanShaderCache(WrappedVulkan *driver)
       VkResult vkr =
           driver->vkCreateShaderModule(m_Device, &modinfo, NULL, &m_BuiltinShaderModules[i]);
       RDCASSERTEQUAL(vkr, VK_SUCCESS);
+
+      driver->GetResourceManager()->SetInternalResource(GetResID(m_BuiltinShaderModules[i]));
     }
   }
 
@@ -283,7 +294,7 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   specMapEntries.resize(specEntries);
   specdata.resize(specSize);
 
-  VkSpecializationMapEntry *entry = &specMapEntries[0];
+  VkSpecializationMapEntry *entry = specMapEntries.data();
 
   uint32_t stageCount = 0;
   specSize = 0;
