@@ -82,10 +82,10 @@ bool WrappedVulkan::Prepare_SparseInitialState(WrappedVkBuffer *buf)
   map<VkDeviceMemory, VkDeviceSize> boundMems;
 
   // value will be filled out later once all memories are added
-  for(size_t i = 0; i < buf->record->sparseInfo->opaquemappings.size(); i++)
-    boundMems[buf->record->sparseInfo->opaquemappings[i].memory] = 0;
+  for(size_t i = 0; i < buf->record->resInfo->opaquemappings.size(); i++)
+    boundMems[buf->record->resInfo->opaquemappings[i].memory] = 0;
 
-  uint32_t numElems = (uint32_t)buf->record->sparseInfo->opaquemappings.size();
+  uint32_t numElems = (uint32_t)buf->record->resInfo->opaquemappings.size();
 
   VkInitialContents initContents;
 
@@ -97,7 +97,7 @@ bool WrappedVulkan::Prepare_SparseInitialState(WrappedVkBuffer *buf)
   initContents.sparseBuffer.numUniqueMems = (uint32_t)boundMems.size();
   initContents.sparseBuffer.memDataOffs = new MemIDOffset[boundMems.size()];
 
-  memcpy(initContents.sparseBuffer.binds, &buf->record->sparseInfo->opaquemappings[0],
+  memcpy(initContents.sparseBuffer.binds, &buf->record->resInfo->opaquemappings[0],
          sizeof(VkSparseMemoryBind) * numElems);
 
   VkDevice d = GetDev();
@@ -201,7 +201,7 @@ bool WrappedVulkan::Prepare_SparseInitialState(WrappedVkImage *im)
 {
   ResourceId id = im->id;
 
-  SparseMapping *sparse = im->record->sparseInfo;
+  ResourceInfo *sparse = im->record->resInfo;
 
   // VKTODOLOW this is a bit conservative, as we save the whole memory object rather than just the
   // bound range.
@@ -441,6 +441,18 @@ bool WrappedVulkan::Serialise_SparseBufferInitialState(SerialiserType &ser, Reso
     vkr = ObjDisp(d)->MapMemory(Unwrap(d), Unwrap(mappedMem.mem), mappedMem.offs, mappedMem.size, 0,
                                 (void **)&Contents);
     RDCASSERTEQUAL(vkr, VK_SUCCESS);
+
+    // invalidate the cpu cache for this memory range to avoid reading stale data
+    VkMappedMemoryRange range = {
+        VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE,
+        NULL,
+        Unwrap(mappedMem.mem),
+        mappedMem.offs,
+        mappedMem.size,
+    };
+
+    vkr = ObjDisp(d)->InvalidateMappedMemoryRanges(Unwrap(d), 1, &range);
+    RDCASSERTEQUAL(vkr, VK_SUCCESS);
   }
   else if(IsReplayingAndReading() && !ser.IsErrored())
   {
@@ -477,7 +489,24 @@ bool WrappedVulkan::Serialise_SparseBufferInitialState(SerialiserType &ser, Reso
 
   // unmap the resource we mapped before - we need to do this on read and on write.
   if(!IsStructuredExporting(m_State) && mappedMem.mem != VK_NULL_HANDLE)
+  {
+    if(IsReplayingAndReading())
+    {
+      // first ensure we flush the writes from the cpu to gpu memory
+      VkMappedMemoryRange range = {
+          VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE,
+          NULL,
+          Unwrap(mappedMem.mem),
+          mappedMem.offs,
+          mappedMem.size,
+      };
+
+      vkr = ObjDisp(d)->FlushMappedMemoryRanges(Unwrap(d), 1, &range);
+      RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    }
+
     ObjDisp(d)->UnmapMemory(Unwrap(d), Unwrap(mappedMem.mem));
+  }
 
   SERIALISE_CHECK_READ_ERRORS();
 
