@@ -152,9 +152,10 @@ ReplayStatus WrappedVulkan::Initialise(VkInitParams &params, uint64_t sectionVer
   {
     if(*it == "VK_KHR_xlib_surface" || *it == "VK_KHR_xcb_surface" ||
        *it == "VK_KHR_wayland_surface" || *it == "VK_KHR_mir_surface" ||
-       *it == "VK_KHR_android_surface" || *it == "VK_KHR_win32_surface" ||
-       *it == "VK_KHR_display" || *it == "VK_EXT_direct_mode_display" ||
-       *it == "VK_EXT_acquire_xlib_display" || *it == "VK_EXT_display_surface_counter")
+       *it == "VK_MVK_macos_surface " || *it == "VK_KHR_android_surface" ||
+       *it == "VK_KHR_win32_surface" || *it == "VK_KHR_display" ||
+       *it == "VK_EXT_direct_mode_display" || *it == "VK_EXT_acquire_xlib_display" ||
+       *it == "VK_EXT_display_surface_counter")
     {
       it = params.Extensions.erase(it);
     }
@@ -283,14 +284,12 @@ ReplayStatus WrappedVulkan::Initialise(VkInitParams &params, uint64_t sectionVer
 
   VkResult ret = GetInstanceDispatchTable(NULL)->CreateInstance(&instinfo, NULL, &m_Instance);
 
-  InstanceDeviceInfo extInfo;
-
 #undef CheckExt
 #define CheckExt(name, ver)                                       \
   if(!strcmp(instinfo.ppEnabledExtensionNames[i], "VK_" #name) || \
      (int)renderdocAppInfo.apiVersion >= ver)                     \
   {                                                               \
-    extInfo.ext_##name = true;                                    \
+    m_EnabledExtensions.ext_##name = true;                        \
   }
 
   for(uint32_t i = 0; i < instinfo.enabledExtensionCount; i++)
@@ -310,10 +309,13 @@ ReplayStatus WrappedVulkan::Initialise(VkInitParams &params, uint64_t sectionVer
   GetResourceManager()->AddLiveResource(params.InstanceID, m_Instance);
 
   // we'll add the chunk later when we re-process it.
-  AddResource(params.InstanceID, ResourceType::Device, "Instance");
-  GetReplay()->GetResourceDesc(params.InstanceID).initialisationChunks.clear();
+  if(params.InstanceID != ResourceId())
+  {
+    AddResource(params.InstanceID, ResourceType::Device, "Instance");
+    GetReplay()->GetResourceDesc(params.InstanceID).initialisationChunks.clear();
+  }
 
-  InitInstanceExtensionTables(m_Instance, &extInfo);
+  InitInstanceExtensionTables(m_Instance, &m_EnabledExtensions);
 
   m_DbgMsgCallback = VK_NULL_HANDLE;
   m_PhysicalDevice = VK_NULL_HANDLE;
@@ -341,6 +343,9 @@ ReplayStatus WrappedVulkan::Initialise(VkInitParams &params, uint64_t sectionVer
   VkResult vkr = ObjDisp(m_Instance)->EnumeratePhysicalDevices(Unwrap(m_Instance), &count, NULL);
   RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
+  if(count == 0)
+    return ReplayStatus::APIHardwareUnsupported;
+
   m_ReplayPhysicalDevices.resize(count);
   m_ReplayPhysicalDevicesUsed.resize(count);
   m_OriginalPhysicalDevices.resize(count);
@@ -428,17 +433,30 @@ VkResult WrappedVulkan::vkCreateInstance(const VkInstanceCreateInfo *pCreateInfo
       hasDebugReport = true;
   }
 
+  std::vector<VkExtensionProperties> supportedExts;
+
   // enumerate what instance extensions are available
-  void *module = Process::LoadModule(VulkanLibraryName);
-  PFN_vkEnumerateInstanceExtensionProperties enumInstExts =
-      (PFN_vkEnumerateInstanceExtensionProperties)Process::GetFunctionAddress(
-          module, "vkEnumerateInstanceExtensionProperties");
+  void *module = LoadVulkanLibrary();
+  if(module)
+  {
+    PFN_vkEnumerateInstanceExtensionProperties enumInstExts =
+        (PFN_vkEnumerateInstanceExtensionProperties)Process::GetFunctionAddress(
+            module, "vkEnumerateInstanceExtensionProperties");
 
-  uint32_t numSupportedExts = 0;
-  enumInstExts(NULL, &numSupportedExts, NULL);
+    if(enumInstExts)
+    {
+      uint32_t numSupportedExts = 0;
+      enumInstExts(NULL, &numSupportedExts, NULL);
 
-  std::vector<VkExtensionProperties> supportedExts(numSupportedExts);
-  enumInstExts(NULL, &numSupportedExts, &supportedExts[0]);
+      supportedExts.resize(numSupportedExts);
+      enumInstExts(NULL, &numSupportedExts, &supportedExts[0]);
+    }
+  }
+
+  if(supportedExts.empty())
+    RDCWARN(
+        "Couldn't load vkEnumerateInstanceExtensionProperties in vkCreateInstance to enumerate "
+        "instance extensions");
 
   // always enable debug report, if it's available
   if(!hasDebugReport)
@@ -904,6 +922,16 @@ VkResult WrappedVulkan::vkEnumeratePhysicalDevices(VkInstance instance,
 
         ObjDisp(devices[i])->GetPhysicalDeviceMemoryProperties(Unwrap(devices[i]), record->memProps);
 
+        VkPhysicalDeviceProperties physProps;
+
+        ObjDisp(devices[i])->GetPhysicalDeviceProperties(Unwrap(devices[i]), &physProps);
+
+        VkDriverInfo capturedVersion(physProps);
+
+        RDCLOG("physical device %u: %s (ver %u.%u patch 0x%x) - %04x:%04x", i, physProps.deviceName,
+               capturedVersion.Major(), capturedVersion.Minor(), capturedVersion.Patch(),
+               physProps.vendorID, physProps.deviceID);
+
         m_PhysicalDevices[i] = devices[i];
 
         // we remap memory indices to discourage coherent maps as much as possible
@@ -969,7 +997,7 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
 
     // see above in Serialise_vkEnumeratePhysicalDevices where this is encoded
     uint32_t physicalDeviceIndex = GetPhysicalDeviceIndexFromHandle(Unwrap(physicalDevice));
-    physicalDevice = m_ReplayPhysicalDevices[physicalDeviceIndex];
+    physicalDevice = m_PhysicalDevices[physicalDeviceIndex];
 
     // we must make any modifications locally, so the free of pointers
     // in the serialised VkDeviceCreateInfo don't double-free
@@ -982,6 +1010,10 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
       if(!strcmp(createInfo.ppEnabledExtensionNames[i], VK_EXT_DEBUG_MARKER_EXTENSION_NAME))
         continue;
 
+      // don't include the validation cache extension
+      if(!strcmp(createInfo.ppEnabledExtensionNames[i], VK_EXT_VALIDATION_CACHE_EXTENSION_NAME))
+        continue;
+
       // don't include direct-display WSI extensions
       if(!strcmp(createInfo.ppEnabledExtensionNames[i], VK_KHR_DISPLAY_SWAPCHAIN_EXTENSION_NAME) ||
          !strcmp(createInfo.ppEnabledExtensionNames[i], VK_EXT_DISPLAY_CONTROL_EXTENSION_NAME))
@@ -1052,6 +1084,29 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
       RDCLOG("Enabling VK_AMD_gpa_interface");
     }
 
+    // enable VK_MVK_moltenvk if it's available, for detecting/controlling moltenvk.
+    // Currently this is used opaquely (extension present or not) rather than using anything the
+    // extension provides.
+    if(supportedExtensions.find("VK_MVK_moltenvk") != supportedExtensions.end())
+    {
+      Extensions.push_back("VK_MVK_moltenvk");
+      RDCLOG("Enabling VK_MVK_moltenvk");
+    }
+
+    // enable VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME if it's available, to fetch mesh output in
+    // tessellation/geometry stages
+    if(supportedExtensions.find(VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME) != supportedExtensions.end())
+    {
+      Extensions.push_back(VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME);
+      RDCLOG("Enabling VK_EXT_transform_feedback_extension");
+    }
+    else
+    {
+      RDCWARN(
+          "VK_EXT_transform_feedback_extension not available, mesh output from "
+          "geometry/tessellation stages will not be available");
+    }
+
     createInfo.enabledLayerCount = (uint32_t)Layers.size();
 
     const char **layerArray = NULL;
@@ -1408,7 +1463,15 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
     VkPhysicalDeviceFeatures enabledFeatures = {0};
     if(createInfo.pEnabledFeatures != NULL)
       enabledFeatures = *createInfo.pEnabledFeatures;
-    createInfo.pEnabledFeatures = &enabledFeatures;
+
+    VkPhysicalDeviceFeatures2 *enabledFeatures2 = (VkPhysicalDeviceFeatures2 *)FindNextStruct(
+        &createInfo, VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2);
+
+    // VkPhysicalDeviceFeatures2 takes priority
+    if(enabledFeatures2)
+      enabledFeatures = enabledFeatures2->features;
+    else if(createInfo.pEnabledFeatures)
+      enabledFeatures = *createInfo.pEnabledFeatures;
 
     VkPhysicalDeviceFeatures availFeatures = {0};
     ObjDisp(physicalDevice)->GetPhysicalDeviceFeatures(Unwrap(physicalDevice), &availFeatures);
@@ -1477,6 +1540,140 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
     CHECK_PHYS_FEATURE(variableMultisampleRate);
     CHECK_PHYS_FEATURE(inheritedQueries);
 
+#define BEGIN_PHYS_EXT_CHECK(struct, stype)                                                  \
+  if(struct *ext = (struct *)FindNextStruct(&createInfo, stype))                             \
+  {                                                                                          \
+    struct avail = {stype};                                                                  \
+    VkPhysicalDeviceFeatures2 availBase = {VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2};    \
+    availBase.pNext = &avail;                                                                \
+    ObjDisp(physicalDevice)->GetPhysicalDeviceFeatures2(Unwrap(physicalDevice), &availBase); \
+    const char *structName = #struct;
+
+#define END_PHYS_EXT_CHECK() }
+
+#define CHECK_PHYS_EXT_FEATURE(feature)                          \
+  if(ext->feature && !avail.feature)                             \
+  {                                                              \
+    m_FailedReplayStatus = ReplayStatus::APIHardwareUnsupported; \
+    RDCERR("Capture requires physical device feature '" #feature \
+           "' in struct '%s' which is not supported",            \
+           structName);                                          \
+    return false;                                                \
+  }
+
+    if(ObjDisp(physicalDevice)->GetPhysicalDeviceFeatures2)
+    {
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDevice8BitStorageFeaturesKHR,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_8BIT_STORAGE_FEATURES_KHR);
+      {
+        CHECK_PHYS_EXT_FEATURE(storageBuffer8BitAccess);
+        CHECK_PHYS_EXT_FEATURE(uniformAndStorageBuffer8BitAccess);
+        CHECK_PHYS_EXT_FEATURE(storagePushConstant8);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDevice16BitStorageFeatures,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_16BIT_STORAGE_FEATURES);
+      {
+        CHECK_PHYS_EXT_FEATURE(storageBuffer16BitAccess);
+        CHECK_PHYS_EXT_FEATURE(uniformAndStorageBuffer16BitAccess);
+        CHECK_PHYS_EXT_FEATURE(storagePushConstant16);
+        CHECK_PHYS_EXT_FEATURE(storageInputOutput16);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceASTCDecodeFeaturesEXT,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ASTC_DECODE_FEATURES_EXT);
+      {
+        CHECK_PHYS_EXT_FEATURE(decodeModeSharedExponent);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceFragmentShaderBarycentricFeaturesNV,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_BARYCENTRIC_FEATURES_NV);
+      {
+        CHECK_PHYS_EXT_FEATURE(fragmentShaderBarycentric);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceMultiviewFeatures,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_FEATURES);
+      {
+        CHECK_PHYS_EXT_FEATURE(multiview);
+        CHECK_PHYS_EXT_FEATURE(multiviewGeometryShader);
+        CHECK_PHYS_EXT_FEATURE(multiviewTessellationShader);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceProtectedMemoryFeatures,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROTECTED_MEMORY_FEATURES);
+      {
+        CHECK_PHYS_EXT_FEATURE(protectedMemory);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceSamplerYcbcrConversionFeatures,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_YCBCR_CONVERSION_FEATURES);
+      {
+        CHECK_PHYS_EXT_FEATURE(samplerYcbcrConversion);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceShaderAtomicInt64FeaturesKHR,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_INT64_FEATURES_KHR);
+      {
+        CHECK_PHYS_EXT_FEATURE(shaderBufferInt64Atomics);
+        CHECK_PHYS_EXT_FEATURE(shaderSharedInt64Atomics);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceShaderDrawParameterFeatures,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_DRAW_PARAMETER_FEATURES);
+      {
+        CHECK_PHYS_EXT_FEATURE(shaderDrawParameters);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceShaderImageFootprintFeaturesNV,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_IMAGE_FOOTPRINT_FEATURES_NV);
+      {
+        CHECK_PHYS_EXT_FEATURE(imageFootprint);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceTransformFeedbackFeaturesEXT,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT);
+      {
+        CHECK_PHYS_EXT_FEATURE(transformFeedback);
+        CHECK_PHYS_EXT_FEATURE(geometryStreams);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceVariablePointerFeatures,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VARIABLE_POINTER_FEATURES);
+      {
+        CHECK_PHYS_EXT_FEATURE(variablePointersStorageBuffer);
+        CHECK_PHYS_EXT_FEATURE(variablePointers);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT);
+      {
+        CHECK_PHYS_EXT_FEATURE(vertexAttributeInstanceRateDivisor);
+        CHECK_PHYS_EXT_FEATURE(vertexAttributeInstanceRateZeroDivisor);
+      }
+      END_PHYS_EXT_CHECK();
+
+      BEGIN_PHYS_EXT_CHECK(VkPhysicalDeviceVulkanMemoryModelFeaturesKHR,
+                           VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_MEMORY_MODEL_FEATURES_KHR);
+      {
+        CHECK_PHYS_EXT_FEATURE(vulkanMemoryModel);
+        CHECK_PHYS_EXT_FEATURE(vulkanMemoryModelDeviceScope);
+      }
+      END_PHYS_EXT_CHECK();
+    }
+
     if(availFeatures.depthClamp)
       enabledFeatures.depthClamp = true;
     else
@@ -1528,6 +1725,12 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
           "sampleRateShading = false, save/load from depth 2DMS textures will not be "
           "possible");
 
+    // patch the enabled features
+    if(enabledFeatures2)
+      enabledFeatures2->features = enabledFeatures;
+    else
+      createInfo.pEnabledFeatures = &enabledFeatures;
+
     uint32_t numExts = 0;
 
     VkResult vkr =
@@ -1564,14 +1767,12 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
     AddResource(Device, ResourceType::Device, "Device");
     DerivedResource(origPhysDevice, Device);
 
-    InstanceDeviceInfo extInfo;
-
 #undef CheckExt
 #define CheckExt(name, ver)                                         \
   if(!strcmp(createInfo.ppEnabledExtensionNames[i], "VK_" #name) || \
      (int)renderdocAppInfo.apiVersion >= ver)                       \
   {                                                                 \
-    extInfo.ext_##name = true;                                      \
+    m_EnabledExtensions.ext_##name = true;                          \
   }
 
     for(uint32_t i = 0; i < createInfo.enabledExtensionCount; i++)
@@ -1579,7 +1780,7 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
       CheckDeviceExts();
     }
 
-    InitDeviceExtensionTables(device, &extInfo);
+    InitDeviceExtensionTables(device, &m_EnabledExtensions);
 
     RDCASSERT(m_Device == VK_NULL_HANDLE);    // MULTIDEVICE
 
@@ -1647,8 +1848,12 @@ bool WrappedVulkan::Serialise_vkCreateDevice(SerialiserType &ser, VkPhysicalDevi
     ObjDisp(physicalDevice)
         ->GetPhysicalDeviceFeatures(Unwrap(physicalDevice), &m_PhysicalDeviceData.features);
 
+    m_Replay.SetDriverInformation(m_PhysicalDeviceData.props);
+
+    // MoltenVK reports 0x3fffffff for this limit so just ignore that value if it comes up
     RDCASSERT(m_PhysicalDeviceData.props.limits.maxBoundDescriptorSets <
-                  ARRAY_COUNT(BakedCmdBufferInfo::pushDescriptorID),
+                      ARRAY_COUNT(BakedCmdBufferInfo::pushDescriptorID[0]) ||
+                  m_PhysicalDeviceData.props.limits.maxBoundDescriptorSets >= 0x10000000,
               m_PhysicalDeviceData.props.limits.maxBoundDescriptorSets);
 
     for(int i = VK_FORMAT_BEGIN_RANGE + 1; i < VK_FORMAT_END_RANGE; i++)
@@ -1834,8 +2039,19 @@ VkResult WrappedVulkan::vkCreateDevice(VkPhysicalDevice physicalDevice,
   // default to all off. This is equivalent to createInfo.pEnabledFeatures == NULL
   VkPhysicalDeviceFeatures enabledFeatures = {0};
 
-  // if the user enabled features, of course we want to enable them.
-  if(createInfo.pEnabledFeatures)
+  // allocate and unwrap the next chain, so we can patch features if we need to, as well as removing
+  // the loader info later when it comes time to serialise
+  byte *tempMem = GetTempMemory(GetNextPatchSize(createInfo.pNext));
+
+  UnwrapNextChain(m_State, "VkDeviceCreateInfo", tempMem, (VkBaseInStructure *)&createInfo);
+
+  VkPhysicalDeviceFeatures2 *enabledFeatures2 = (VkPhysicalDeviceFeatures2 *)FindNextStruct(
+      &createInfo, VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2);
+
+  // VkPhysicalDeviceFeatures2 takes priority
+  if(enabledFeatures2)
+    enabledFeatures = enabledFeatures2->features;
+  else if(createInfo.pEnabledFeatures)
     enabledFeatures = *createInfo.pEnabledFeatures;
 
   if(availFeatures.shaderStorageImageWriteWithoutFormat)
@@ -1869,14 +2085,17 @@ VkResult WrappedVulkan::vkCreateDevice(VkPhysicalDevice physicalDevice,
   else
     RDCWARN("pipelineStatisticsQuery = false, pipeline counters will not work");
 
-  createInfo.pEnabledFeatures = &enabledFeatures;
+  // patch the enabled features
+  if(enabledFeatures2)
+    enabledFeatures2->features = enabledFeatures;
+  else
+    createInfo.pEnabledFeatures = &enabledFeatures;
 
   VkResult ret;
   SERIALISE_TIME_CALL(ret = createFunc(Unwrap(physicalDevice), &createInfo, pAllocator, pDevice));
 
   // don't serialise out any of the pNext stuff for layer initialisation
-  // (note that we asserted above that there was nothing else in the chain)
-  createInfo.pNext = NULL;
+  RemoveNextStruct(&createInfo, VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO);
 
   if(ret == VK_SUCCESS)
   {
