@@ -29,6 +29,7 @@
 
 #include <android/dlext.h>
 #include <dlfcn.h>
+#include <errno.h>
 #include <jni.h>
 #include <link.h>
 #include <stddef.h>
@@ -214,8 +215,19 @@ HookingInfo &GetHookInfo()
 
 void *intercept_dlopen(const char *filename, int flag)
 {
-  if(GetHookInfo().IsLibHook(filename))
-    return dlopen(RENDERDOC_ANDROID_LIBRARY, flag);
+  if(filename)
+  {
+    // if this is a library we're hooking, or a request for our own library in any form, return our
+    // own library.
+    // We need to intercept requests for our own library, because the android loader makes the
+    // completely ridiculous decision to load multiple copies of the same library into a process if
+    // it's dlopen'd with different paths. This obviously breaks with our hook install.
+    if(strstr(filename, RENDERDOC_ANDROID_LIBRARY) || GetHookInfo().IsLibHook(std::string(filename)))
+    {
+      HOOK_DEBUG_PRINT("Intercepting dlopen for %s", filename);
+      return dlopen(RENDERDOC_ANDROID_LIBRARY, flag);
+    }
+  }
 
   return NULL;
 }
@@ -420,7 +432,7 @@ uint64_t suppressTLS = 0;
 
 void process_dlopen(const char *filename, int flag)
 {
-  if(!GetHookInfo().IsHooked(filename))
+  if(filename && !GetHookInfo().IsHooked(std::string(filename)))
   {
     HOOK_DEBUG_PRINT("iterating after %s", filename);
     dl_iterate_phdr(dl_iterate_callback, NULL);
@@ -519,8 +531,6 @@ static void InstallHooksCommon()
   GetHookInfo().SetHooked("libc.so");
   GetHookInfo().SetHooked("libvndksupport.so");
 
-  GetHookInfo().AddLibHook(RENDERDOC_ANDROID_LIBRARY);
-
   real_android_dlopen_ext = &android_dlopen_ext;
 
   loader_dlopen = (pfn__loader_dlopen)dlsym(RTLD_NEXT, "__loader_dlopen");
