@@ -175,6 +175,16 @@ extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetVersionString()
   return MAJOR_MINOR_VERSION_STRING;
 }
 
+extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetCommitHash()
+{
+  return GitVersionHash;
+}
+
+extern "C" RENDERDOC_API DriverInformation RENDERDOC_CC RENDERDOC_GetDriverInformation(GraphicsAPI api)
+{
+  return RenderDoc::Inst().GetDriverInformation(api);
+}
+
 extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetConfigSetting(const char *name)
 {
   return RenderDoc::Inst().GetConfigSetting(name).c_str();
@@ -303,10 +313,10 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_UnregisterMemoryRegion(void
 
 extern "C" RENDERDOC_API ExecuteResult RENDERDOC_CC
 RENDERDOC_ExecuteAndInject(const char *app, const char *workingDir, const char *cmdLine,
-                           const rdcarray<EnvironmentModification> &env, const char *logfile,
+                           const rdcarray<EnvironmentModification> &env, const char *capturefile,
                            const CaptureOptions &opts, bool waitForExit)
 {
-  return Process::LaunchAndInjectIntoProcess(app, workingDir, cmdLine, env, logfile, opts,
+  return Process::LaunchAndInjectIntoProcess(app, workingDir, cmdLine, env, capturefile, opts,
                                              waitForExit != 0);
 }
 
@@ -316,10 +326,10 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_GetDefaultCaptureOptions(Ca
 }
 
 extern "C" RENDERDOC_API bool RENDERDOC_CC RENDERDOC_StartGlobalHook(const char *pathmatch,
-                                                                     const char *logfile,
+                                                                     const char *capturefile,
                                                                      const CaptureOptions &opts)
 {
-  return Process::StartGlobalHook(pathmatch, logfile, opts);
+  return Process::StartGlobalHook(pathmatch, capturefile, opts);
 }
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_StopGlobalHook()
@@ -339,9 +349,9 @@ extern "C" RENDERDOC_API bool RENDERDOC_CC RENDERDOC_CanGlobalHook()
 
 extern "C" RENDERDOC_API ExecuteResult RENDERDOC_CC
 RENDERDOC_InjectIntoProcess(uint32_t pid, const rdcarray<EnvironmentModification> &env,
-                            const char *logfile, const CaptureOptions &opts, bool waitForExit)
+                            const char *capturefile, const CaptureOptions &opts, bool waitForExit)
 {
-  return Process::InjectIntoProcess(pid, env, logfile, opts, waitForExit != 0);
+  return Process::InjectIntoProcess(pid, env, capturefile, opts, waitForExit != 0);
 }
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_FreeArrayMem(const void *mem)
@@ -475,8 +485,8 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_EndSelfHostCapture(const ch
   rdoc->EndFrameCapture(NULL, NULL);
 }
 
-extern "C" RENDERDOC_API bool RENDERDOC_CC RENDERDOC_NeedVulkanLayerRegistration(
-    VulkanLayerFlags *flagsPtr, rdcarray<rdcstr> *myJSONsPtr, rdcarray<rdcstr> *otherJSONsPtr)
+extern "C" RENDERDOC_API bool RENDERDOC_CC
+RENDERDOC_NeedVulkanLayerRegistration(VulkanLayerRegistrationInfo *info)
 {
   VulkanLayerFlags flags = VulkanLayerFlags::NoFlags;
   std::vector<std::string> myJSONs;
@@ -484,21 +494,17 @@ extern "C" RENDERDOC_API bool RENDERDOC_CC RENDERDOC_NeedVulkanLayerRegistration
 
   bool ret = RenderDoc::Inst().NeedVulkanLayerRegistration(flags, myJSONs, otherJSONs);
 
-  if(flagsPtr)
-    *flagsPtr = flags;
-
-  if(myJSONsPtr)
+  if(info)
   {
-    myJSONsPtr->resize(myJSONs.size());
+    info->flags = flags;
+
+    info->myJSONs.resize(myJSONs.size());
     for(size_t i = 0; i < myJSONs.size(); i++)
-      (*myJSONsPtr)[i] = myJSONs[i];
-  }
+      info->myJSONs[i] = myJSONs[i];
 
-  if(otherJSONsPtr)
-  {
-    otherJSONsPtr->resize(otherJSONs.size());
+    info->otherJSONs.resize(otherJSONs.size());
     for(size_t i = 0; i < otherJSONs.size(); i++)
-      (*otherJSONsPtr)[i] = otherJSONs[i];
+      info->otherJSONs[i] = otherJSONs[i];
   }
 
   return ret;
