@@ -53,10 +53,16 @@ void DoSerialise(SerialiserType &ser, GetTextureDataParams &el)
 
 INSTANTIATE_SERIALISE_TYPE(GetTextureDataParams);
 
-DrawcallDescription *SetupDrawcallPointers(vector<DrawcallDescription *> &drawcallTable,
-                                           rdcarray<DrawcallDescription> &draws,
-                                           DrawcallDescription *parent,
-                                           DrawcallDescription *&previous)
+static bool PreviousNextExcludedMarker(DrawcallDescription *draw)
+{
+  return bool(draw->flags & (DrawFlags::PushMarker | DrawFlags::SetMarker | DrawFlags::MultiDraw |
+                             DrawFlags::APICalls));
+}
+
+static DrawcallDescription *SetupDrawcallPointers(vector<DrawcallDescription *> &drawcallTable,
+                                                  rdcarray<DrawcallDescription> &draws,
+                                                  DrawcallDescription *parent,
+                                                  DrawcallDescription *&previous)
 {
   DrawcallDescription *ret = NULL;
 
@@ -76,8 +82,7 @@ DrawcallDescription *SetupDrawcallPointers(vector<DrawcallDescription *> &drawca
 
       ret = SetupDrawcallPointers(drawcallTable, draw->children, draw, previous);
     }
-    else if(draw->flags & (DrawFlags::PushMarker | DrawFlags::SetMarker | DrawFlags::MultiDraw |
-                           DrawFlags::APICalls))
+    else if(PreviousNextExcludedMarker(draw))
     {
       // don't want to set up previous/next links for markers, but still add them to the table
       // Some markers like Present should have previous/next, but API Calls we also skip
@@ -113,6 +118,53 @@ DrawcallDescription *SetupDrawcallPointers(vector<DrawcallDescription *> &drawca
   return ret;
 }
 
+void SetupDrawcallPointers(std::vector<DrawcallDescription *> &drawcallTable,
+                           rdcarray<DrawcallDescription> &draws)
+{
+  DrawcallDescription *previous = NULL;
+  SetupDrawcallPointers(drawcallTable, draws, NULL, previous);
+
+  // markers don't enter the previous/next chain, but we still want pointers for them that point to
+  // the next or previous actual draw (skipping any markers). This means that draw->next->previous
+  // != draw sometimes, but it's more useful than draw->next being NULL in the middle of the list.
+  // This enables searching for a marker string and then being able to navigate from there and
+  // joining the 'real' linked list after one step.
+
+  previous = NULL;
+  std::vector<DrawcallDescription *> markers;
+
+  for(DrawcallDescription *draw : drawcallTable)
+  {
+    if(!draw)
+      continue;
+
+    bool marker = PreviousNextExcludedMarker(draw);
+
+    if(marker)
+    {
+      // point the previous pointer to the last non-marker draw we got. If we haven't hit one yet
+      // because this is near the start, this will just be NULL.
+      draw->previous = previous;
+
+      // because there can be multiple markers consecutively we want to point all of their nexts to
+      // the next draw we encounter. Accumulate this list, though in most cases it will only be 1
+      // long as it's uncommon to have multiple markers one after the other
+      markers.push_back(draw);
+    }
+    else
+    {
+      // the next markers we encounter should point their previous to this.
+      previous = draw;
+
+      // all previous markers point to this one
+      for(DrawcallDescription *m : markers)
+        m->next = draw;
+
+      markers.clear();
+    }
+  }
+}
+
 void PatchLineStripIndexBuffer(const DrawcallDescription *draw, uint8_t *idx8, uint16_t *idx16,
                                uint32_t *idx32, std::vector<uint32_t> &patchedIndices)
 {
@@ -207,6 +259,20 @@ void PatchLineStripIndexBuffer(const DrawcallDescription *draw, uint8_t *idx8, u
 #undef IDX_VALUE
 }
 
+uint64_t CalcMeshOutputSize(uint64_t curSize, uint64_t requiredOutput)
+{
+  // resize exponentially up to 256MB to avoid repeated resizes
+  while(curSize < requiredOutput && curSize < 0x10000000ULL)
+    curSize *= 2;
+
+  // after that, just align the required size up to 16MB and allocate that. Otherwise we can
+  // vastly-overallocate at large sizes.
+  if(curSize < requiredOutput)
+    curSize = AlignUp(requiredOutput, (uint64_t)0x1000000ULL);
+
+  return curSize;
+}
+
 FloatVector HighlightCache::InterpretVertex(const byte *data, uint32_t vert, const MeshDisplay &cfg,
                                             const byte *end, bool useidx, bool &valid)
 {
@@ -748,3 +814,18 @@ bool HighlightCache::FetchHighlightPositions(const MeshDisplay &cfg, FloatVector
 
   return valid;
 }
+
+// colour ramp from http://www.ncl.ucar.edu/Document/Graphics/ColorTables/GMT_wysiwyg.shtml
+const Vec4f colorRamp[22] = {
+    Vec4f(0.000000f, 0.000000f, 0.000000f, 0.0f), Vec4f(0.250980f, 0.000000f, 0.250980f, 1.0f),
+    Vec4f(0.250980f, 0.000000f, 0.752941f, 1.0f), Vec4f(0.000000f, 0.250980f, 1.000000f, 1.0f),
+    Vec4f(0.000000f, 0.501961f, 1.000000f, 1.0f), Vec4f(0.000000f, 0.627451f, 1.000000f, 1.0f),
+    Vec4f(0.250980f, 0.752941f, 1.000000f, 1.0f), Vec4f(0.250980f, 0.878431f, 1.000000f, 1.0f),
+    Vec4f(0.250980f, 1.000000f, 1.000000f, 1.0f), Vec4f(0.250980f, 1.000000f, 0.752941f, 1.0f),
+    Vec4f(0.250980f, 1.000000f, 0.250980f, 1.0f), Vec4f(0.501961f, 1.000000f, 0.250980f, 1.0f),
+    Vec4f(0.752941f, 1.000000f, 0.250980f, 1.0f), Vec4f(1.000000f, 1.000000f, 0.250980f, 1.0f),
+    Vec4f(1.000000f, 0.878431f, 0.250980f, 1.0f), Vec4f(1.000000f, 0.627451f, 0.250980f, 1.0f),
+    Vec4f(1.000000f, 0.376471f, 0.250980f, 1.0f), Vec4f(1.000000f, 0.125490f, 0.250980f, 1.0f),
+    Vec4f(1.000000f, 0.376471f, 0.752941f, 1.0f), Vec4f(1.000000f, 0.627451f, 1.000000f, 1.0f),
+    Vec4f(1.000000f, 0.878431f, 1.000000f, 1.0f), Vec4f(1.000000f, 1.000000f, 1.000000f, 1.0f),
+};
