@@ -44,6 +44,15 @@ struct RDCThumb
   uint32_t len = 0;
   uint16_t width = 0;
   uint16_t height = 0;
+  FileType format = FileType::JPG;
+};
+
+struct ExtThumbnailHeader
+{
+  uint16_t width;
+  uint16_t height;
+  uint32_t len;
+  uint32_t format;
 };
 
 class RDCFile
