@@ -40,12 +40,16 @@ int64_t Chunk::m_TotalMem = 0;
 // Read Serialiser functions
 
 template <>
-Serialiser<SerialiserMode::Reading>::Serialiser(StreamReader *reader, Ownership own)
+Serialiser<SerialiserMode::Reading>::Serialiser(StreamReader *reader, Ownership own,
+                                                SDObject *rootStructuredObj)
 {
   m_Read = reader;
   m_Write = NULL;
 
   m_Ownership = own;
+
+  if(rootStructuredObj)
+    m_StructureStack.push_back(rootStructuredObj);
 }
 
 template <>
@@ -744,6 +748,18 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
 /////////////////////////////////////////////////////////////
 // Basic types
 
+template <>
+std::string DoStringise(const std::string &el)
+{
+  return el;
+}
+
+template <>
+std::string DoStringise(const rdcstr &el)
+{
+  return el;
+}
+
 template <>
 std::string DoStringise(void *const &el)
 {
