@@ -107,15 +107,13 @@ struct VulkanRegisterCommand : public Command
 
 void VerifyVulkanLayer(const GlobalEnvironment &env, int argc, char *argv[])
 {
-  VulkanLayerFlags flags = VulkanLayerFlags::NoFlags;
-  rdcarray<rdcstr> myJSONs;
-  rdcarray<rdcstr> otherJSONs;
+  VulkanLayerRegistrationInfo info;
 
-  bool needUpdate = RENDERDOC_NeedVulkanLayerRegistration(&flags, &myJSONs, &otherJSONs);
+  bool needUpdate = RENDERDOC_NeedVulkanLayerRegistration(&info);
 
   if(!needUpdate)
   {
-    if(!(flags & VulkanLayerFlags::Unfixable))
+    if(!(info.flags & VulkanLayerFlags::Unfixable))
       add_command("vulkanregister", new VulkanRegisterCommand(env));
     return;
   }
@@ -126,39 +124,39 @@ void VerifyVulkanLayer(const GlobalEnvironment &env, int argc, char *argv[])
             << std::endl;
   std::cerr << std::endl;
 
-  if(flags & VulkanLayerFlags::OtherInstallsRegistered)
+  if(info.flags & VulkanLayerFlags::OtherInstallsRegistered)
     std::cerr << "Multiple RenderDoc layers are registered, possibly from different builds."
               << std::endl;
 
-  if(!(flags & VulkanLayerFlags::ThisInstallRegistered))
+  if(!(info.flags & VulkanLayerFlags::ThisInstallRegistered))
     std::cerr << "This build's RenderDoc layer is not registered." << std::endl;
 
   std::cerr << "To fix this, the following actions must take place: " << std::endl << std::endl;
 
-  const bool registerAll = bool(flags & VulkanLayerFlags::RegisterAll);
-  const bool updateAllowed = bool(flags & VulkanLayerFlags::UpdateAllowed);
+  const bool registerAll = bool(info.flags & VulkanLayerFlags::RegisterAll);
+  const bool updateAllowed = bool(info.flags & VulkanLayerFlags::UpdateAllowed);
 
-  for(const rdcstr &j : otherJSONs)
+  for(const rdcstr &j : info.otherJSONs)
     std::cerr << (updateAllowed ? "Unregister/update: " : "Unregister: ") << j.c_str() << std::endl;
 
-  if(!(flags & VulkanLayerFlags::ThisInstallRegistered))
+  if(!(info.flags & VulkanLayerFlags::ThisInstallRegistered))
   {
     if(registerAll)
     {
-      for(const rdcstr &j : myJSONs)
+      for(const rdcstr &j : info.myJSONs)
         std::cerr << (updateAllowed ? "Register/update: " : "Register: ") << j.c_str() << std::endl;
     }
     else
     {
       std::cerr << (updateAllowed ? "Register one of:" : "Register/update one of:") << std::endl;
-      for(const rdcstr &j : myJSONs)
+      for(const rdcstr &j : info.myJSONs)
         std::cerr << "  -- " << j.c_str() << "\n";
     }
   }
 
   std::cerr << std::endl;
 
-  if(flags & VulkanLayerFlags::Unfixable)
+  if(info.flags & VulkanLayerFlags::Unfixable)
   {
     std::cerr << "NOTE: The renderdoc layer registered in /usr is reserved for distribution"
               << std::endl;
@@ -317,7 +315,7 @@ WindowingData DisplayRemoteServerPreview(bool active, const rdcarray<WindowingSy
 }
 
 void DisplayRendererPreview(IReplayController *renderer, TextureDisplay &displayCfg, uint32_t width,
-                            uint32_t height)
+                            uint32_t height, uint32_t numLoops)
 {
 // we only have the preview implemented for platforms that have xlib & xcb. It's unlikely
 // a meaningful platform exists with only one, and at the time of writing no other windowing
@@ -418,6 +416,8 @@ void DisplayRendererPreview(IReplayController *renderer, TextureDisplay &display
 
   xcb_flush(connection);
 
+  uint32_t loopCount = 0;
+
   bool done = false;
   while(!done)
   {
@@ -428,10 +428,7 @@ void DisplayRendererPreview(IReplayController *renderer, TextureDisplay &display
     {
       switch(event->response_type & 0x7f)
       {
-        case XCB_EXPOSE:
-          renderer->SetFrameEvent(10000000, true);
-          out->Display();
-          break;
+        case XCB_EXPOSE: break;
         case XCB_CLIENT_MESSAGE:
           if((*(xcb_client_message_event_t *)event).data.data32[0] == (*atom_wm_delete_window).atom)
           {
@@ -456,6 +453,11 @@ void DisplayRendererPreview(IReplayController *renderer, TextureDisplay &display
     out->Display();
 
     usleep(100000);
+
+    loopCount++;
+
+    if(numLoops > 0 && loopCount == numLoops)
+      break;
   }
 #else
   std::cerr << "No supporting windowing systems defined at build time (xlib and xcb)" << std::endl;
